//--------------------------------------------------------------------
//                                                                   -
// Module      : KxoInboundTargetBean.java                           -
//                                                                   -
// Description : CASP for Websphere Access Inbound Sample Target EJB -
//                                                                   -
//--------------------------------------------------------------------
//                                                                   -
// Copyright Candle Corporation 2001,2003                            -
//                                                                   -
//--------------------------------------------------------------------
package com.candle.roma.kxo.inbound.target;

import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.ejb.EJBException;
import javax.rmi.PortableRemoteObject;
import java.util.*;

// CASP Log Helper
import com.candle.roma.kxo.LogHelper;

/**
 * The CASP for Websphere Access Inbound Sample Target EJB
 *
 * A Sample EJB that can be invoked from a CASP environment through
 * the CASP for Websphere Access Inbound solution.
 *
 * The KxoInboundTargetBean is a simple Session EJB that provides a
 * number of simple interfaces that demonstrate the marshalling
 * of inbound JMS messages to a target EJB.
 *
 * @author  Candle
 * @version $Revision: 1.13 $
 */
public class KxoInboundTargetBean extends Object implements javax.ejb.SessionBean {

  private static final String KxoInboundTargetCopyright
      = "(c) Copyright Candle Corporation 2002";

  // =======================================================================
  // Websphere integrated log helper wrapper
  private static final LogHelper LOG  = 
      new LogHelper("com.candle.roma.kxo.inbound.target.KxoInboundTargetBean");

  // =======================================================================
  // EJB interface methods

  /**
    * Creates an instance of the KxoInboundTargetBean
    */
  public KxoInboundTargetBean (){}

  /**
    * This is not a part of SessionBean interface; it is defined implicitly
    * through the "home" interface.  This method is called after creation.
    */
  public void ejbCreate ()   {
    LOG.debug("ejbCreate()");
  }


  /**
    * The activate method is called when the instance is activated
    * from its "passive" state.
    * The instance should acquire any resource that it has released
    * earlier in the ejbPassivate() method.
    *
    * @exception EJBException Thrown if the remote method call fails.
    */
  public void ejbActivate () throws EJBException
  {
    LOG.debug("ejbActivate()");
  }

  /**
    * The passivate method is called before the instance enters the
    * "passive" state.
    * The instance should release any resources that that it
    * can re-acquire later in the ejbActivate() method.
    *
    * @exception EJBException Thrown if the remote method call fails.
    */
  public void ejbPassivate () throws EJBException
  {
    LOG.debug("ejbPassivate");
  }

  /**
    * A container invokes this method before it ends the life of
    * the session object.
    * This happens as a result of a client's invoking a remove
    * operation, or when a container decides to terminate the session
    * object after a timeout.
    * 
    * @exception EJBException Thrown if the remote method call fails.
    */
  public void ejbRemove () throws EJBException
  {
    LOG.debug("ejbRemove()");
  }

  /**
    * Set the associated session context.
    * The container calls this method after the instance creation.
    * 
    * @param ctx An SessionContext interface for the instance.
    * @exception EJBException Thrown if the remote method call fails.
    */
  public void setSessionContext (SessionContext ctx) throws EJBException
  {
    LOG.debug("setSessionContext()");
  }

  // =======================================================================
  // PUBLIC INTERFACE METHODS
  // =======================================================================
  public String methodSimpleTypes(String stringVal, Long longVal, Boolean boolVal) 
  {
    LOG.debug("methodSimpleTypes()");
    return(stringVal + ", " + longVal.toString() + ", " + boolVal.toString());
  }

  public String methodArray(String[] arrayVal) 
  {
    LOG.debug("methodArray()");
    StringBuffer responseString = new StringBuffer();
    for (int i = 0; i < arrayVal.length; i++)
    {
      responseString.append(arrayVal[i]);
      responseString.append(",");
    }
    return(responseString.toString());
  }

  public String methodDocument(String xmlDoc)
  {
    LOG.debug("methodDocument()");
    return(xmlDoc);
  }

  public String methodXADocument(String xmlDoc)
  {
    LOG.debug("methodXADocument()");
    return(xmlDoc);
  }

  public String methodList(String[] listVal)
  {
    LOG.debug("methodList()");
    StringBuffer responseString = new StringBuffer();
    for (int i = 0; i < listVal.length; i++)
    {
      responseString.append(listVal[i]);
      responseString.append(",");
    }
    return(responseString.toString());
  }

  public String methodUnion(Long longVal)
  {
    LOG.debug("methodUnion(longVal)");
    return(longVal.toString());
  }

  public String methodUnion(String stringVal)
  {
    LOG.debug("methodUnion(stringVal)");
    return(stringVal);
  }
}
