//-------------------------------------------------------------------
//                                                                  -
// Module      : KxoServlet.java                                    -
//                                                                  -
// Description : CandleNet eBP JMS Outbound toolkit Servlet         -
//                                                                  -
//-------------------------------------------------------------------
//                                                                  -
// Copyright Candle Corporation 2001,2003                           -
//                                                                  -
//-------------------------------------------------------------------
//                                                                  -
// Details:                                                         -
//    Kxo toolkit Servlet                                           -
//                                                                  -
// Invocation Syntax:                                               -
//    Access as                                                     -
//     http://<Hostname>/<web path for web app>/servlet/<name>      -
// Eg  http://mboag/app1/servlet/cejo_drop1                         -
//                                                                  -
//                                                                  -
//-------------------------------------------------------------------
//                                                                  -
// Processing:                                                      -
//                                                                  -
// 1. Invoked via an http request.                                  -
// 2. Builds a html response page.                                  -
// 3. Invokes the EJB via the helper agent class                    -
// 4. Includes the returned message in the page and returns it      -
//                                                                  -
//-------------------------------------------------------------------
//                                                                  -
// Return Value:                                                    -
//    One of the following integral codes:                          -
//       EXIT_SUCCESS -- Successful completion.                     -
//       EXIT_FAILURE -- Unsuccessful completion.                   -
//                                                                  -
//-------------------------------------------------------------------

package com.candle.roma.kxo.outbound.sample;

// JAVA Servlet
import javax.servlet.*;
import javax.servlet.http.*;

// utilities
import java.io.*;
import java.util.Date;

///////////////////////////////////////////////////////////////////////
//
// Class KxoServlet
//
/**
  * The KxoServlet class implements the sample servlet to demonstate
  * calling an eBP Business Service from a User EJB issuinig a method
  * call which is mapped via an Access Bean to the KxoEJB which
  * uses SOAP to invoke an eBP business service 
  * @author 	Candle
  * @version 1.0  $Revision: 1.5 $
  */
public class KxoServlet extends HttpServlet {

 /**
   * Create and Display a dynamic html page from the results of an EJB call 
   * via eBP SOAP formated message to invoke a Business Service.
   *
   * @param   req    HttpServletRequest giving the input parms for the EJB call
   * @param   res    HttpServletResponse contains the html to display containing
   *                 the results of the EJB call
   * @throws  IOException
   * @since   1.0
   */
  public void
  service(HttpServletRequest req, HttpServletResponse res) throws IOException
  {
    // Must set the content type first
    res.setContentType("text/html");

    // Now we can obtain a PrintWriter
    PrintWriter out = res.getWriter();
    
    out.println("<html><head><title>CASP for WebSphere Access KxoServlet</title></head>");

    // Create the helper servlet Agent object
    KxoServletAgent myAgent = new KxoServletAgent();

    // And the page html contents...
    String s1 = "<body><h1>";
    s1 += "CASP for WebSphere Access Outbound Sample.</h1>";
    s1 += "<h2>This simple sample servlet processes the html post and calls either the inquire() or buy() method on ";
    s1 += "the sample User EJB 'com.candle.roma.kxo.outbound.sample.UserEJB'.</h1></body>";
    

    out.println(s1);

    StringBuffer data_returned = new StringBuffer("");
    //
    // Invoke the EJB via the helper class
    //
    myAgent.run(req, data_returned);

    String s2 = "<body><h2>";
    s2 += data_returned;
    s2 += "</h2></body>";

    out.println(s2);

    String s3 = "<body><h3>";
    s3 += "Dynamic page created by KxoServlet on ";

    Date ts = new Date();
    s3 += ts.toString();
    s3 += "</h3></body>";
    s3 += "</html>";

    out.println(s3);
  }
}
