//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcobrqa.cpp,v 1.9 2003/12/11 18:30:04 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcobrqa.cpp,v $
// $Revision: 1.9 $
//--------------------------------------------------------------------
// The RomaBeRequest public interface.
//--------------------------------------------------------------------

#include <kxcointa.hpp>
#include <kxcoimsg.hpp>
#include <kxcobrq.hpp>

TRACE_MODULE_VERSION($Revision: 1.9 $);

//--------------------------------------------------------------------
// Define RTTI for the class.
//--------------------------------------------------------------------
ROMA_DEFINE_TYPEID(RomaBeRequest);

//--------------------------------------------------------------------
// Default constructor.
//--------------------------------------------------------------------
RomaBeRequest::RomaBeRequest()
{
    TRACE_ENTRY(RomaBeRequest::RomaBeRequest, << TRACE(this));

    m_impl = new RomaImplMsgIncoming;

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Constructor.
// Used to initialize the object with the specified implementation
// instance.
//--------------------------------------------------------------------
RomaBeRequest::RomaBeRequest(RomaImplMsgIncoming* impl)
    : m_impl(impl)
{
    TRACE_ENTRY(RomaBeRequest::RomaBeRequest, 
        << TRACE(this) << TRACE(impl));

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Destructor.
//--------------------------------------------------------------------
RomaBeRequest::~RomaBeRequest()
{
    TRACE_ENTRY(RomaBeRequest::~RomaBeRequest, << TRACE(this));

    m_impl->releaseInstance();

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Copy constructor.
// It creates the shallow copy of the request. The original request
// and its copy share the same internal implementation instance.
//--------------------------------------------------------------------
RomaBeRequest::RomaBeRequest(const RomaBeRequest& req) 
    : m_impl(req.m_impl)
{
    TRACE_ENTRY(RomaBeRequest::~RomaBeRequest, 
        << TRACE(this) << TRACE(req));

    m_impl->lockInstance();

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Assignment operator.
// It returns the shallow copy of the request. The original request
// and its copy share the same internal implementation instance.
//--------------------------------------------------------------------
RomaBeRequest& RomaBeRequest::operator = (const RomaBeRequest& req)
{
    TRACE_ENTRY(RomaBeRequest::operator =, 
        << TRACE(this) << TRACE(req));

    if (m_impl != req.m_impl)
    {
        m_impl->releaseInstance();
        m_impl = req.m_impl;
        m_impl->lockInstance();
    }

    TRACE_RETURN(*this);
}

//--------------------------------------------------------------------
// Get the message theme.
//--------------------------------------------------------------------
const RomaTheme& RomaBeRequest::theme() const
{
    return m_impl->getTheme();
}

//--------------------------------------------------------------------
// Get the message ID.
//--------------------------------------------------------------------
const RomaMsgId& RomaBeRequest::msgId() const
{
    return m_impl->getMsgId();
}

//--------------------------------------------------------------------
// Get the correlation ID.
//--------------------------------------------------------------------
const RomaMsgId& RomaBeRequest::correlId() const
{
    return m_impl->getCorrelId();
}

//--------------------------------------------------------------------
// Get the message priority.
//--------------------------------------------------------------------
int RomaBeRequest::priority() const
{
    return m_impl->getPriority();
}

//--------------------------------------------------------------------
// Get the message type.
//--------------------------------------------------------------------
Roma::MsgType RomaBeRequest::msgType() const
{
    return m_impl->getType();
}

//--------------------------------------------------------------------
// Get the message persistence.
//--------------------------------------------------------------------
Roma::Persistence RomaBeRequest::persistence() const
{
    return m_impl->getPersistence();
}

//--------------------------------------------------------------------
// Get the message format.
//--------------------------------------------------------------------
int RomaBeRequest::format() const
{
    return m_impl->getFormat();
}

//--------------------------------------------------------------------
// Get the message feedback.
//--------------------------------------------------------------------
int RomaBeRequest::feedback() const
{
    return m_impl->getFeedback();
}

//--------------------------------------------------------------------
// Get the message expiry.
//--------------------------------------------------------------------
int RomaBeRequest::expiry() const
{
    return m_impl->getExpiry();
}

//--------------------------------------------------------------------
// Get the message code page.
//--------------------------------------------------------------------
int RomaBeRequest::codepage() const
{
    return m_impl->getCodepage();
}

//--------------------------------------------------------------------
// Get the message encoding.
//--------------------------------------------------------------------
int RomaBeRequest::encoding() const
{
    return m_impl->getEncoding();
}

//--------------------------------------------------------------------
// Get the message status.
//--------------------------------------------------------------------
KXCULONG RomaBeRequest::status() const
{
    return m_impl->getStatus();
}

//--------------------------------------------------------------------
// Get the message backout count.
//--------------------------------------------------------------------
int RomaBeRequest::backoutCount() const
{
    return m_impl->getBackoutCount();
}

//--------------------------------------------------------------------
// Get a name of the message format.
//--------------------------------------------------------------------
const char* RomaBeRequest::formatName() const
{
    return m_impl->getFormatName();
}

//--------------------------------------------------------------------
// Get the message name.
//--------------------------------------------------------------------
const char* RomaBeRequest::msgName() const
{
    return m_impl->getMsgName();
}

//--------------------------------------------------------------------
// Get the message time stamp.
//--------------------------------------------------------------------
RomaTimestamp RomaBeRequest::timestamp() const
{
    return m_impl->getTimestamp();
}

//--------------------------------------------------------------------
// Get the pointer the message data.
//--------------------------------------------------------------------
void* RomaBeRequest::msgBuffer() const
{
    return m_impl->getMsgBuffer();
}

//--------------------------------------------------------------------
// Get the length of the message data.
//--------------------------------------------------------------------
int RomaBeRequest::msgDataLength() const
{
    return m_impl->getMsgLength();
}

//--------------------------------------------------------------------
// Get the pointer the message tag data.
//--------------------------------------------------------------------
void* RomaBeRequest::tagBuffer() const
{
    return m_impl->getTagBuffer();
}

//--------------------------------------------------------------------
// Get the length of the message tag data.
//--------------------------------------------------------------------
int RomaBeRequest::tagDataLength() const
{
    return m_impl->getTagLength();
}

//--------------------------------------------------------------------
// Get ILS data associated with the message.
//--------------------------------------------------------------------
RomaILSData* RomaBeRequest::getILSData() const
{
    return m_impl->getILSData();
}

//--------------------------------------------------------------------
// Output trace information.
//--------------------------------------------------------------------
void RomaBeRequest::trace(RomaTraceStream& s) const
{
    s << TRACE_CLASS(RomaBeRequest, << TRACE(m_impl)) << "->";
    m_impl->trace(s);
}

