//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcocla.cpp,v 1.14 2003/12/11 18:30:04 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcocla.cpp,v $
// $Revision: 1.14 $
//--------------------------------------------------------------------
// The RomaClient public interface.
//--------------------------------------------------------------------

#include <kxcointa.hpp>
#include <kxcoicxt.hpp>
#include <kxcoipxy.hpp>
#include <kxcocl.hpp>

TRACE_MODULE_VERSION($Revision: 1.14 $);

//--------------------------------------------------------------------
// Define RTTI for the class.
//--------------------------------------------------------------------
ROMA_DEFINE_TYPEID(RomaClient);


//--------------------------------------------------------------------
// Default constructor.
// Does not establish connection.
//--------------------------------------------------------------------
RomaClient::RomaClient()
{
    TRACE_ENTRY(RomaClient::RomaClient, << TRACE(this));

    m_impl = new RomaImplClient;

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Constructor.
// Establishes connection.
//--------------------------------------------------------------------
RomaClient::RomaClient(const char* name,
                       KXCULONG options, 
                       const RomaSecurityDef& secDef,
                       const char* model,
                       const RomaILSData& ilsData,
                       const RomaDirectoryAccess& auth)
{
    TRACE_ENTRY(RomaClient::RomaClient, << TRACE(this)); 

    m_impl = new RomaImplClient;
    connect(name, options, secDef, model, ilsData, auth);

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Destructor.
// The internal implementation instance will be released. The
// connection will be closed if there are no more objects using the
// connection (i.e. keeping references on the implementation
// instance).
//--------------------------------------------------------------------
RomaClient::~RomaClient()
{
    TRACE_ENTRY(RomaClient::~RomaClient, << TRACE(this));

    m_impl->releaseInstance();

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Establish connection.
//--------------------------------------------------------------------
bool RomaClient::connect(const char* name, 
                         KXCULONG options, 
                         const RomaSecurityDef& secDef,
                         const char* model,
                         const RomaILSData& ilsData,
                         const RomaDirectoryAccess& auth)
{
    TRACE_ENTRY(RomaClient::connect, << TRACE(this));

    bool success = m_impl->connect(KXCOCALLTYPE_CLIENTCONNECT, 
        name, options, secDef, model, ilsData, auth);

    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Put the request.
//--------------------------------------------------------------------
bool RomaClient::put(RomaClientRequest& request)
{
    TRACE_ENTRY(RomaClient::put, << TRACE(this));

    bool success = m_impl->put(request);

    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Publish the message with the subject.
//--------------------------------------------------------------------
bool RomaClient::publish(RomaClientRequest& publication, 
                         const RomaSubject& subject)
{
    TRACE_ENTRY(RomaClient::publish, << TRACE(this));

    bool success = m_impl->publish(publication, subject);

    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Publish the publication.
//--------------------------------------------------------------------
bool RomaClient::publish(RomaPublication& publication)
{
    TRACE_ENTRY(RomaClient::publish, << TRACE(this));

    bool success = m_impl->publish(publication);

    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Subscribe using the request and the subject.
//--------------------------------------------------------------------
bool RomaClient::subscribe(RomaClientRequest& request, 
                           const RomaSubject& subject)
{
    TRACE_ENTRY(RomaClient::subscribe, << TRACE(this));

    bool success = m_impl->subscribe(request, subject);

    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Subscribe using the subscription.
//--------------------------------------------------------------------
bool RomaClient::subscribe(RomaSubscription& subscription)
{
    TRACE_ENTRY(RomaClient::subscribe, << TRACE(this));

    bool success = m_impl->subscribe(subscription);

    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Request updates on the specified subsciption.
//--------------------------------------------------------------------
bool RomaClient::requestUpdates(RomaSubscription& subscription)
{
    TRACE_ENTRY(RomaClient::requestUpdates, << TRACE(this));

    bool success = m_impl->requestUpdates(subscription);

    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Delete the specified publication.
//--------------------------------------------------------------------
bool RomaClient::deletePublication(RomaPublication& publication)
{
    TRACE_ENTRY(RomaClient::deletePublication, << TRACE(this));

    bool success = m_impl->deletePublication(publication);

    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Unsubscribe from the subject.
//--------------------------------------------------------------------
bool RomaClient::unsubscribe(RomaClientRequest& request, 
                             const RomaSubject& subject)
{
    TRACE_ENTRY(RomaClient::unsubscribe, << TRACE(this));

    bool success = m_impl->unsubscribe(request, subject);

    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Unsubscribe from the subject defined by the subscription.
//--------------------------------------------------------------------
bool RomaClient::unsubscribe(RomaSubscription& subscription)
{
    TRACE_ENTRY(RomaClient::unsubscribe, << TRACE(this));

    bool success = m_impl->unsubscribe(subscription);

    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Unsubscribe from all subjects.
//--------------------------------------------------------------------
bool RomaClient::unsubscribeAll(RomaSubscription& subscription)
{
    TRACE_ENTRY(RomaClient::unsubscribeAll, << TRACE(this));

    bool success = m_impl->unsubscribeAll(subscription);

    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Get an incoming message.
// The message buffers will be auto-allocated.
//--------------------------------------------------------------------
RomaClientResponse* RomaClient::get()
{
    RomaGetOptions opt;
    return get(opt);
}

//--------------------------------------------------------------------
// Get an incoming message.
// The message buffers will be auto-allocated.
//--------------------------------------------------------------------
RomaClientResponse* RomaClient::get(RomaGetOptions& options)
{
    TRACE_ENTRY(RomaClient::get, << TRACE(this));

    RomaClientResponse* response = m_impl->get(*this, options);

    TRACE_INT_RETURN(response);
}

//--------------------------------------------------------------------
// Get incoming message. Message data will be placed into the
// specified buffers.
//--------------------------------------------------------------------
RomaClientResponse* RomaClient::get(RomaGetOptions& options,
                                    void* msgBuffer,
                                    int&  msgLength,
                                    void* tagBuffer,
                                    int&  tagLength)
{
    TRACE_ENTRY(RomaClient::get, << TRACE(this));

    RomaClientResponse* response = m_impl->get(*this, options,
        msgBuffer, msgLength, tagBuffer, tagLength);

    TRACE_INT_RETURN(response);
}

//--------------------------------------------------------------------
// This method will be called on get() to unmarshal the incoming
// message. The default implementation does nothing.
//--------------------------------------------------------------------
bool RomaClient::createResponse(const RomaClientResponse& baseResponse,
                                RomaClientResponse*& newResponse)
{
    TRACE_ENTRY(RomaClient::createResponse, << TRACE(this));

    // Fast copy, just the implementation pointer will be shared.
    newResponse = new RomaClientResponse(baseResponse);

    TRACE_INT_RETURN(newResponse != NULL);
}

//--------------------------------------------------------------------
// Get ILS data.
//--------------------------------------------------------------------
RomaILSData* RomaClient::getILSData() const
{
    return m_impl->getILSData();
}

//--------------------------------------------------------------------
// Disconnect the client.
//--------------------------------------------------------------------
bool RomaClient::disconnect()
{
    return m_impl->disconnect();
}

//--------------------------------------------------------------------
// Returns true if the client is connected.
//--------------------------------------------------------------------
bool RomaClient::isConnected() const
{
    return m_impl->isConnected();
}

//--------------------------------------------------------------------
// Commit a local transaction.
//--------------------------------------------------------------------
bool RomaClient::commitLocal()
{
    return m_impl->commitLocal();
}

//--------------------------------------------------------------------
// Rollback a local transaction.
//--------------------------------------------------------------------
bool RomaClient::rollbackLocal()
{
    return m_impl->rollbackLocal();
}

//--------------------------------------------------------------------
// Close the theme.
//--------------------------------------------------------------------
bool RomaClient::closeTheme(const RomaTheme& theme)
{
    return m_impl->closeTheme(theme);
}

//--------------------------------------------------------------------
// Delete the storage queue.
//--------------------------------------------------------------------
bool RomaClient::deleteStorageQueue(const char* queueName)
{
    return m_impl->deleteStorageQueue(queueName);
}

//--------------------------------------------------------------------
// Put the exception message.
//--------------------------------------------------------------------
bool RomaClient::putException(RomaExceptionMessage& message,
                              const RomaExceptionOptions& options)
{
    return m_impl->putException(message, options);
}

//--------------------------------------------------------------------
// Report the error.
//--------------------------------------------------------------------
bool RomaClient::reportError(RomaExceptionMessage& message,
                             KXCULONG& carriedOutActions,
                             const RomaReportOptions& options)
{
    return m_impl->reportError(message, carriedOutActions, options);
}

//--------------------------------------------------------------------
// Send the alert.
//--------------------------------------------------------------------
bool RomaClient::sendAlert(const RomaLogInformation& logInfo)
{
    return m_impl->sendAlert(logInfo);
}

//--------------------------------------------------------------------
// Indicates whether an error occured during the last operation.
//--------------------------------------------------------------------
bool RomaClient::anyError() const
{
    return m_impl->anyError();
}

//--------------------------------------------------------------------
// Get the last error information.
//--------------------------------------------------------------------
RomaError& RomaClient::error()
{
    return m_impl->getError();
}

//--------------------------------------------------------------------
// Erase the last error information.
//--------------------------------------------------------------------
void RomaClient::resetError()
{
    m_impl->resetError();
}

//--------------------------------------------------------------------
// Write the information to the user log.
//--------------------------------------------------------------------
bool RomaClient::writeLog(const RomaLogInformation& logInfo)
{
    return m_impl->writeLog(logInfo);
}

//--------------------------------------------------------------------
// Write the information to the system log.
//--------------------------------------------------------------------
bool RomaClient::writeSysLog(const RomaLogInformation& logInfo)
{
    return m_impl->writeSysLog(logInfo);
}

//--------------------------------------------------------------------
// Inserts the business service into the workflow of the theme.
//--------------------------------------------------------------------
bool RomaClient::insertWorkflow(RomaTheme& newTheme,
                                const RomaTheme& curTheme,
                                const RomaBusinessService& bs,
                                bool close) const
{
    return m_impl->insertWorkflow(newTheme, curTheme, bs, close, 
        false);
}

//--------------------------------------------------------------------
// Replaces the workflow of the theme with a workflow based on the
// specified business service.
//--------------------------------------------------------------------
bool RomaClient::replaceWorkflow(RomaTheme& newTheme,
                                 const RomaTheme& curTheme,
                                 const RomaBusinessService& bs,
                                 bool close) const
{
    return m_impl->insertWorkflow(newTheme, curTheme, bs, close, 
        true);
}

//--------------------------------------------------------------------
// Sets the business flow information on the context.
//--------------------------------------------------------------------
bool RomaClient::setBusinessFlow(const RomaBusinessFlow& bf) const
{
    return m_impl->setBusinessFlow(bf);
}

//--------------------------------------------------------------------
// Creates a new theme that is copy of the specified theme.
//--------------------------------------------------------------------
bool RomaClient::copyTheme(RomaTheme& newTheme,
                           const RomaTheme& curTheme) const
{
    return m_impl->copyTheme(newTheme, curTheme);
}

//--------------------------------------------------------------------
// Merges ETE information into the theme.
//--------------------------------------------------------------------
bool RomaClient::mergeEteInfo(RomaTheme& newTheme,
                              const RomaTheme& curTheme,
                              const RomaEteInfo& ete,
                              bool close) const
{
    return m_impl->mergeEteInfo(newTheme, curTheme, ete, close);
}

//--------------------------------------------------------------------
// Extracts theme information.
//--------------------------------------------------------------------
bool RomaClient::queryTheme(RomaThemeInfo& info,
                            const RomaTheme& theme) const
{
    return m_impl->queryTheme(info, theme, true);
}

//--------------------------------------------------------------------
// Creates a new theme from the specified theme information.
//--------------------------------------------------------------------
bool RomaClient::buildTheme(RomaTheme& newTheme,
                            const RomaTheme& curTheme,
                            const RomaThemeInfo& info,
                            unsigned options,
                            bool close) const
{
    return m_impl->buildTheme(newTheme, curTheme, info, options,
        close, true);
}

//--------------------------------------------------------------------
// Extracts message information.
//--------------------------------------------------------------------
bool RomaClient::interpretMessage(RomaMessageInfo& info,
                                  const RomaClientResponse& msg,
                                  unsigned options) const
{
    return m_impl->interpretMessage(
        info, RomaImplProxy::getImpl(msg), options);
}

//--------------------------------------------------------------------
// Oprator to cast the object to the HKXC handle.
//--------------------------------------------------------------------
RomaClient::operator HKXC() const
{
    return (HKXC)(*m_impl);
}

//--------------------------------------------------------------------
// Output trace information.
//--------------------------------------------------------------------
void RomaClient::trace(RomaTraceStream& s) const
{
    s << TRACE_CLASS(RomaClient, << TRACE(m_impl)) << "->";
    m_impl->trace(s);
}

//--------------------------------------------------------------------
// Deprecated.
//--------------------------------------------------------------------
HKXC RomaClient::operator *() const
{
    return (HKXC)(*m_impl);
}

