//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcocrqa.cpp,v 1.8 2003/02/21 22:48:03 emueh Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcocrqa.cpp,v $
// $Revision: 1.8 $
//--------------------------------------------------------------------
// The RomaClientRequest public interface.
//--------------------------------------------------------------------

#include <kxcointa.hpp>
#include <kxcoimsg.hpp>
#include <kxcocrq.hpp>

TRACE_MODULE_VERSION($Revision: 1.8 $);

//--------------------------------------------------------------------
// Define RTTI information.
//--------------------------------------------------------------------
ROMA_DEFINE_TYPEID(RomaClientRequest);


//--------------------------------------------------------------------
// Default constructor.
//--------------------------------------------------------------------
RomaClientRequest::RomaClientRequest()
{
    TRACE_ENTRY(RomaClientRequest::RomaClientRequest, << TRACE(this));

    m_impl = new RomaImplClientRequest;

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Constructor.
//--------------------------------------------------------------------
RomaClientRequest::RomaClientRequest(const char* serviceName)
{
    TRACE_ENTRY(RomaClientRequest::RomaClientRequest, 
        << TRACE(this) << TRACE(serviceName));

    m_impl = new RomaImplClientRequest;
    m_impl->setServiceName(serviceName);

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Constructor.
//--------------------------------------------------------------------
RomaClientRequest::RomaClientRequest(const char* serviceName,
                                     const void* msgBuffer,
                                     int msgLength)
{
    TRACE_ENTRY(RomaClientRequest::RomaClientRequest, 
        << TRACE(this) << TRACE(serviceName) 
        << TRACE(msgBuffer) << TRACE(msgLength));

    m_impl = new RomaImplClientRequest;
    m_impl->setServiceName(serviceName);
    m_impl->setMsgBuffer(msgBuffer, msgLength);

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Constructor.
//--------------------------------------------------------------------
RomaClientRequest::RomaClientRequest(const char* serviceName,
                                     const void* msgBuffer,
                                     int msgLength,
                                     const void* tagBuffer,
                                     int tagLength)
{
    TRACE_ENTRY(RomaClientRequest::RomaClientRequest, 
        << TRACE(this) << TRACE(serviceName)
        << TRACE(msgBuffer) << TRACE(msgLength)
        << TRACE(tagBuffer) << TRACE(tagLength));

    m_impl = new RomaImplClientRequest;
    m_impl->setServiceName(serviceName);
    m_impl->setMsgBuffer(msgBuffer, msgLength);
    m_impl->setTagBuffer(tagBuffer, tagLength);

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Destructor.
//--------------------------------------------------------------------
RomaClientRequest::~RomaClientRequest()
{
    TRACE_ENTRY(RomaClientRequest::~RomaClientRequest, << TRACE(this));

    m_impl->releaseInstance();

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Copy constructor.
//--------------------------------------------------------------------
RomaClientRequest::RomaClientRequest(const RomaClientRequest& req)
    : m_impl(req.m_impl)
{
    TRACE_ENTRY(RomaClientRequest::RomaClientRequest, << TRACE(this)
                                                      << TRACE(req));

    m_impl->lockInstance();

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Assignment operator.
//--------------------------------------------------------------------
RomaClientRequest& 
RomaClientRequest::operator = (const RomaClientRequest& req)
{
    TRACE_ENTRY(RomaClientRequest::operator =, << TRACE(this)
                                               << TRACE(req));

    if (m_impl != req.m_impl)
    {
        m_impl->releaseInstance();
        m_impl = req.m_impl;
        m_impl->lockInstance();
    }

    TRACE_RETURN(*this);
}

//--------------------------------------------------------------------
// This method will be called by the RomaClient::put() to marshal
// the message. The default implementation does nothing.
//--------------------------------------------------------------------
bool RomaClientRequest::marshalRequest()
{
    return true;
}

//--------------------------------------------------------------------
// Set the business service name.
//--------------------------------------------------------------------
void RomaClientRequest::setServiceName(const char* name)
{
    m_impl->setServiceName(name);
}

//--------------------------------------------------------------------
// Indicate whether a new theme must be created for the message.
//--------------------------------------------------------------------
void RomaClientRequest::setNewTheme(bool f)
{
    m_impl->setNewTheme(f);
}

//--------------------------------------------------------------------
// Indicate whether the specified theme must be used.
//--------------------------------------------------------------------
void RomaClientRequest::setUseTheme(bool f)
{
    m_impl->setUseTheme(f);
}

//--------------------------------------------------------------------
// Indicate whether the message theme must be closed.
//--------------------------------------------------------------------
void RomaClientRequest::setCloseTheme(bool f)
{
    m_impl->setCloseTheme(f);
}

//--------------------------------------------------------------------
// Indicate whether the message theme must be retained.
//--------------------------------------------------------------------
void RomaClientRequest::setRetainTheme(bool f)
{
    m_impl->setRetainTheme(f);
}

//--------------------------------------------------------------------
// Specify true to send the message as a datagram.
//--------------------------------------------------------------------
void RomaClientRequest::setDatagram(bool f)
{
    m_impl->setDatagram(f);
}

//--------------------------------------------------------------------
// Specify true to set "Confirmation of delivery" flag.
//--------------------------------------------------------------------
void RomaClientRequest::setCOD(bool f)
{
    m_impl->setCOD(f);
}

//--------------------------------------------------------------------
// Specify true to set "Confirmation of arrival" flag.
//--------------------------------------------------------------------
void RomaClientRequest::setCOA(bool f)
{
    m_impl->setCOA(f);
}

//--------------------------------------------------------------------
// Set the security definition.
//--------------------------------------------------------------------
void RomaClientRequest::setSecurityDef(const RomaSecurityDef& secDef)
{
    m_impl->setSecurityDef(secDef);
}

//--------------------------------------------------------------------
// Get the message theme.
//--------------------------------------------------------------------
const RomaTheme& RomaClientRequest::theme() const
{
    return m_impl->getTheme();
}

//--------------------------------------------------------------------
// Set the message theme.
//--------------------------------------------------------------------
void RomaClientRequest::setTheme(const RomaTheme& theme)
{
    m_impl->setTheme(theme);
}

//--------------------------------------------------------------------
// Get the message ID.
//--------------------------------------------------------------------
const RomaMsgId& RomaClientRequest::msgId() const
{
	return m_impl->getMsgId();
}

//--------------------------------------------------------------------
// Set the message ID.
//--------------------------------------------------------------------
void RomaClientRequest::setMsgId(const RomaMsgId& msgId)
{
    m_impl->setMsgId(msgId);
}

//--------------------------------------------------------------------
// Get the correlation ID.
//--------------------------------------------------------------------
const RomaMsgId& RomaClientRequest::correlId() const
{
    return m_impl->getCorrelId();
}

//--------------------------------------------------------------------
// Set the correlation ID.
//--------------------------------------------------------------------
void RomaClientRequest::setCorrelId(const RomaMsgId& correlId)
{
    m_impl->setCorrelId(correlId);
}

//--------------------------------------------------------------------
// Set the message priority.
//--------------------------------------------------------------------
void RomaClientRequest::setPriority(int priority)
{
    m_impl->setPriority(priority);
}

//--------------------------------------------------------------------
// Set the message persistence.
//--------------------------------------------------------------------
void RomaClientRequest::setPersistence(Roma::Persistence persistence)
{
    m_impl->setPersistence(persistence);
}

//--------------------------------------------------------------------
// Set the message format.
//--------------------------------------------------------------------
void RomaClientRequest::setFormat(int format)
{
    m_impl->setFormat(format);
}

//--------------------------------------------------------------------
// Set the message name.
//--------------------------------------------------------------------
void RomaClientRequest::setMsgName(const char* name)
{
    m_impl->setMsgName(name);
}

//--------------------------------------------------------------------
// Set the message feedback.
//--------------------------------------------------------------------
void RomaClientRequest::setFeedback(int feedback)
{
    m_impl->setFeedback(feedback);
}

//--------------------------------------------------------------------
// Set the message expiry.
//--------------------------------------------------------------------
void RomaClientRequest::setExpiry(int expiry)
{
    m_impl->setExpiry(expiry);
}

//--------------------------------------------------------------------
// Set the message code page.
//--------------------------------------------------------------------
void RomaClientRequest::setCodepage(int codepage)
{
    m_impl->setCodepage(codepage);
}

//--------------------------------------------------------------------
// Set the message encoding.
//--------------------------------------------------------------------
void RomaClientRequest::setEncoding(int encoding)
{
    m_impl->setEncoding(encoding);
}

//--------------------------------------------------------------------
// Set the message format name.
//--------------------------------------------------------------------
void RomaClientRequest::setFormatName(const char* name)
{
    m_impl->setFormatName(name);
}

//-----------------------------------------------------------------
// Set the message data buffer.
//-----------------------------------------------------------------
bool RomaClientRequest::setMsgBuffer(const void* buffer, int length)
{
    return m_impl->setMsgBuffer(buffer, length);
}

//-----------------------------------------------------------------
// Set the tag data buffer.
//-----------------------------------------------------------------
bool RomaClientRequest::setTagBuffer(const void* buffer, int length)
{
    return m_impl->setTagBuffer(buffer, length);
}

//-----------------------------------------------------------------
// Set the put selector.
//-----------------------------------------------------------------
void RomaClientRequest::setSelector(Roma::PutSelector selector)
{
    m_impl->setSelector(selector);
}

//-----------------------------------------------------------------
// Specify true to be able to send the message to a
// non-transactional queue.
//-----------------------------------------------------------------
void RomaClientRequest::setNoSyncAllowed(bool flag)
{
    m_impl->setNoSyncAllowed(flag);
}

//-----------------------------------------------------------------
// Set the API profile name.
//-----------------------------------------------------------------
void RomaClientRequest::setApiProfileName(const char* name)
{
    m_impl->setApiProfile(name);
}

//-----------------------------------------------------------------
// Set the error profile name.
//-----------------------------------------------------------------
void RomaClientRequest::setErrorProfileName(const char* name)
{
    m_impl->setErrorProfile(name);
}

//-----------------------------------------------------------------
// Specify true to avoid EMS invocation on en error.
//-----------------------------------------------------------------
void RomaClientRequest::setNoEms(bool flag)
{
    m_impl->setNoEms(flag);
}

//-----------------------------------------------------------------
// Specify true to deny notifications.
//-----------------------------------------------------------------
void RomaClientRequest::setNoNotification(bool flag)
{
    m_impl->setNoNotification(flag);
}

//-----------------------------------------------------------------
// Specify true to prohibit a message transformation.
//-----------------------------------------------------------------
void RomaClientRequest::setNoTransformation(bool flag)
{
    m_impl->setNoTransformation(flag);
}

//-----------------------------------------------------------------
// Set the termination indicator.
//-----------------------------------------------------------------
void RomaClientRequest::setTermSupported(bool flag)
{
    m_impl->setTermSupported(flag);
}

//-----------------------------------------------------------------
// Specify true to suppress the error profile use.
//-----------------------------------------------------------------
void RomaClientRequest::setSuppressErrorProfile(bool flag)
{
    m_impl->setSuppressErrorProfile(flag);
}

//-----------------------------------------------------------------
// Set ILS data.
//-----------------------------------------------------------------
void RomaClientRequest::setILSData(const RomaILSData& ILSDataSet)
{
    m_impl->setILSData(ILSDataSet);
}

//-----------------------------------------------------------------
// Get ILS data.
//-----------------------------------------------------------------
RomaILSData* RomaClientRequest::getILSData() const
{
    return m_impl->getILSData();
}

//-----------------------------------------------------------------
// Set the syncpoint definition.
//-----------------------------------------------------------------
void RomaClientRequest::setSyncpointDef(const RomaSyncpointDef& def)
{
    m_impl->setSyncpointDef(def);
}

//-----------------------------------------------------------------
// Output trace information.
//-----------------------------------------------------------------
void RomaClientRequest::trace(RomaTraceStream& s) const
{
    s << TRACE_CLASS(RomaClientRequest, << TRACE(m_impl)) << "->";
    m_impl->trace(s);
}

