//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcoicxt.hpp,v 1.20 2004/01/22 17:53:25 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcoicxt.hpp,v $
// $Revision: 1.20 $
//--------------------------------------------------------------------
// The internal implementation of the RomaClient, the
// RomaBusinessElement and the RomaStorageQueue public interfaces.
//--------------------------------------------------------------------

#ifndef __KXCOICXT_HPP__
#define __KXCOICXT_HPP__

#include <kxcotrb.hpp>
#include <kxcotb.hpp>
#include <kxcosyn.hpp>
#include <kxcoerr.hpp>
#include <kxcoimsg.hpp>
#include <kxcoem.hpp>
#include <kxcocrq.hpp>
#include <kxcogto.hpp>
#include <kxcoso.hpp>
#include <kxcorvo.hpp>
#include <kxcopbl.hpp>
#include <kxcosbs.hpp>
#include <kxcoems.hpp>

//--------------------------------------------------------------------
// Forward declarations.
//--------------------------------------------------------------------
class RomaClient;
class RomaBusinessElement;
class RomaStorable;
class RomaLogInformation;
class RomaStorageQueue;
class RomaDirectoryAccess;


//--------------------------------------------------------------------
// The RomaImplContext class.
// It is the base class for the RomaImplClient and the
// RomaImplElement. This class represents a Roma connection. It can
// be shared by multiply objects which need the connection.
//--------------------------------------------------------------------
class KXCODLL RomaImplContext : public RomaImplShareable, 
                                public RomaTraceable
{
public:
    //----------------------------------------------------------------
    // Default constructor.
    //----------------------------------------------------------------
    RomaImplContext();

    //----------------------------------------------------------------
    // Operator casting the object to the C API context handle (HKXC).
    //----------------------------------------------------------------
    operator HKXC() const { return m_handle; }

    //----------------------------------------------------------------
    // Terminate connection.
    //----------------------------------------------------------------
    bool disconnect();

    //----------------------------------------------------------------
    // Indicates whether connection is established.
    //----------------------------------------------------------------
    bool isConnected() const { return m_handle != NULL_HKXC; }

    //----------------------------------------------------------------
    // Commit local transaction.
    //----------------------------------------------------------------
    bool commitLocal();

    //----------------------------------------------------------------
    // Rollback local transaction.
    //----------------------------------------------------------------
    bool rollbackLocal();

    //----------------------------------------------------------------
    // Close the theme.
    //----------------------------------------------------------------
    bool closeTheme(const RomaTheme&);
    bool closeTheme(HKXCTHEME);

    //----------------------------------------------------------------
    // Open the named storage queue.
    //----------------------------------------------------------------
    HKXCSTORQ openStorageQueue(const char*, KXCULONG, const char*);

    //----------------------------------------------------------------
    // Close the storage queue.
    //----------------------------------------------------------------
    bool closeStorageQueue(HKXCSTORQ);

    //----------------------------------------------------------------
    // Delete the named storage queue.
    //----------------------------------------------------------------
    bool deleteStorageQueue(const char*);

    //----------------------------------------------------------------
    // Store the message on the storage queue.
    //----------------------------------------------------------------
    bool storeStorable(HKXCSTORQ, 
                       RomaStorable&,
                       const RomaStoreOptions&);

    //----------------------------------------------------------------
    // Get a message from the storage queue.
    //----------------------------------------------------------------
    RomaStorable* retrieveStorable(HKXCSTORQ, 
                                   KXCMSG&,
                                   const RomaRetrieveOptions&, 
                                   RomaStorageQueue&,
                                   bool);

    //----------------------------------------------------------------
    // Put the exception message.
    //----------------------------------------------------------------
    bool putException(RomaExceptionMessage&, 
                      const RomaExceptionOptions&);

    //----------------------------------------------------------------
    // Report the error.
    //----------------------------------------------------------------
    bool reportError(RomaExceptionMessage&, 
                     KXCULONG&, 
                     const RomaReportOptions&);

    //----------------------------------------------------------------
    // Send the alert.
    //----------------------------------------------------------------
    bool sendAlert(const RomaLogInformation&);

    //----------------------------------------------------------------
    // Get data provided by the ILSes during context initialization.
    //----------------------------------------------------------------
    RomaILSData* getILSData();

    //----------------------------------------------------------------
    // Indicates whether an error occured during the last operation.
    //----------------------------------------------------------------
    bool anyError() const { return (m_error && m_error->anyError()); }

    //----------------------------------------------------------------
    // Delete the last error information.
    //----------------------------------------------------------------
    void resetError() { delete m_error; m_error = NULL; }

    //----------------------------------------------------------------
    // Get the last error information.
    //----------------------------------------------------------------
    RomaError& getError();

    //----------------------------------------------------------------
    // Write information to the user log.
    //----------------------------------------------------------------
    bool writeLog(const RomaLogInformation&);

    //----------------------------------------------------------------
    // Write information to the system log.
    //----------------------------------------------------------------
    bool writeSysLog(const RomaLogInformation&);

    //----------------------------------------------------------------
    // If false specified, gather the last error information.
    //----------------------------------------------------------------
    void setErrorStatus(bool success);

    void setCppError(KXCULONG retcode,
                     KXCULONG position,
                     KXCULONG errorType,
                     const char* method,
                     KXCULONG line,
                     const char* module);

    void reportCppError(KXCULONG callType,
                        KXCMSG* msg = NULL,
                        HKXCTHEME theme = NULL_HKXCTHEME,
                        KXCSYNC* sync = NULL);

    //----------------------------------------------------------------
    // Establish connection.
    //----------------------------------------------------------------
    bool connect(KXCULONG, 
                 const char*, 
                 KXCULONG, 
                 const RomaSecurityDef&, 
                 const char*, 
                 const RomaILSData&,
                 const RomaDirectoryAccess&);

    //----------------------------------------------------------------
    // If the object is not connected, returns false and sets the
    // corresponding error on the object.
    //----------------------------------------------------------------
    bool checkConnected(KXCULONG);

    //----------------------------------------------------------------
    // If the object is connected, returns false and sets the
    // corresponding error on the object.
    //----------------------------------------------------------------
    bool checkDisconnected(KXCULONG);

    //----------------------------------------------------------------
    // Inserts the business service into the workflow or replaces
    // the workflow with the business service.
    //----------------------------------------------------------------
    bool insertWorkflow(RomaTheme& newTheme,
                        const RomaTheme& curTheme,
                        const RomaBusinessService& bs,
                        bool close, bool replace);

    //----------------------------------------------------------------
    // Sets the business flow information on the context.
    //----------------------------------------------------------------
    bool setBusinessFlow(const RomaBusinessFlow& bf);

    //----------------------------------------------------------------
    // Creates a new theme that is copy of the specified theme.
    //----------------------------------------------------------------
    bool copyTheme(RomaTheme& newTheme, const RomaTheme& curTheme);

    //----------------------------------------------------------------
    // Merges ETE information into the theme.
    //----------------------------------------------------------------
    bool mergeEteInfo(RomaTheme& newTheme,
                      const RomaTheme& curTheme,
                      const RomaEteInfo& ete,
                      bool close);

    //----------------------------------------------------------------
    // Extracts theme information.
    //----------------------------------------------------------------
    bool queryTheme(RomaThemeInfo& info,
                    const RomaTheme& theme,
                    bool client);

    //----------------------------------------------------------------
    // Creates a new theme from the specified theme information.
    //----------------------------------------------------------------
    bool buildTheme(RomaTheme& newTheme,
                    const RomaTheme& curTheme,
                    const RomaThemeInfo& info,
                    unsigned options,
                    bool close,
                    bool client);

    //----------------------------------------------------------------
    // Extracts message information.
    //----------------------------------------------------------------
    bool interpretMessage(RomaMessageInfo& info,
                          const RomaImplMessage& msg,
                          unsigned options);

    //----------------------------------------------------------------
    // Output trace information.
    //----------------------------------------------------------------
    virtual void trace(RomaTraceStream&) const;

protected:
    //----------------------------------------------------------------
    // Protected destructor.
    // The object should be deleted with the releaseInstance() method
    // only, because it may be shared by muliply objects.
    //----------------------------------------------------------------
    virtual ~RomaImplContext();

    //----------------------------------------------------------------
    // Context handle.
    //----------------------------------------------------------------
    HKXC m_handle;

    //----------------------------------------------------------------
    // ILS data handle.
    //----------------------------------------------------------------
    HKXCILS m_ilsIncoming;

    //----------------------------------------------------------------
    // Pointer to the last error.
    //----------------------------------------------------------------
    RomaError* m_error;

private:
    //----------------------------------------------------------------
    // NOT IMPLEMENTED.
    //----------------------------------------------------------------
    RomaImplContext(const RomaImplContext&);
    RomaImplContext& operator = (const RomaImplContext&);
};


//--------------------------------------------------------------------
// The RomaImplClient class.
// This is the internal implementation of the RomaClient public
// interface.
//--------------------------------------------------------------------
class RomaImplClient : public RomaImplContext
{
public:
    //----------------------------------------------------------------
    // Default constructor.
    //----------------------------------------------------------------
    RomaImplClient();

    //----------------------------------------------------------------
    // Put the message to a Roma server.
    //----------------------------------------------------------------
    bool put(RomaClientRequest&);
    bool put(RomaImplClientRequest&, KXCPUTOPT&);

    //----------------------------------------------------------------
    // Get a message from the queue.
    //----------------------------------------------------------------
    RomaClientResponse* get(RomaClient&, RomaGetOptions&);
    RomaClientResponse* get(RomaClient&, RomaGetOptions&,
                            void*, int&,
                            void*, int&);

    //----------------------------------------------------------------
    // Publish the message.
    //----------------------------------------------------------------
    bool publish(RomaClientRequest&, const RomaSubject&);
    bool publish(RomaPublication&);

    //----------------------------------------------------------------
    // Subscribe to the publication identified by the subject.
    //----------------------------------------------------------------
    bool subscribe(RomaClientRequest&, const RomaSubject&);
    bool subscribe(RomaSubscription&);

    //----------------------------------------------------------------
    // Unsubscribe from the publication identified by the subject.
    //----------------------------------------------------------------
    bool unsubscribe(RomaClientRequest&, const RomaSubject&);
    bool unsubscribe(RomaSubscription&);

    //----------------------------------------------------------------
    // Request updates on the publication identified by the subject.
    //----------------------------------------------------------------
    bool requestUpdates(RomaSubscription&);

    //----------------------------------------------------------------
    // Delete the retained publication.
    //----------------------------------------------------------------
    bool deletePublication(RomaPublication&);

    //----------------------------------------------------------------
    // Unsubscribe from all publications.
    //----------------------------------------------------------------
    bool unsubscribeAll(RomaSubscription&);

private:
    //----------------------------------------------------------------
    // Setup the KXCPUTOPT and the KXCPSLIST structures for the
    // publish() operation.
    //----------------------------------------------------------------
    bool preparePubSub(KXCPUTOPT&, 
                       KXCPSLIST&, 
                       const RomaSubject&, 
                       int, 
                       int) const;

    //----------------------------------------------------------------
    // Common routine for the get() methods.
    //----------------------------------------------------------------
    RomaClientResponse* realGet(RomaClient&,
                                KXCMSG&,
                                KXCGETOPT&,
                                KXCSYNC&, 
                                HKXCTHEME);

    //----------------------------------------------------------------
    // NOT IMPLEMENTED.
    //----------------------------------------------------------------
    RomaImplClient(const RomaImplClient&);
    RomaImplClient& operator = (const RomaImplClient&);  
};


//--------------------------------------------------------------------
// The RomaImplElement class.
// This is the internal implementation of the RomaBusinessElement
// public interface.
//--------------------------------------------------------------------
class KXCODLL RomaImplElement : public RomaImplContext
{
public:
    //----------------------------------------------------------------
    // Default constructor.
    //----------------------------------------------------------------
    RomaImplElement();

    //----------------------------------------------------------------
    // Put the message to the client.
    //----------------------------------------------------------------
    bool put(RomaBeResponse&);

    //----------------------------------------------------------------
    // Get a message from a client.
    //----------------------------------------------------------------
    RomaBeRequest* get(RomaBusinessElement&, RomaGetOptions&);
    RomaBeRequest* get(RomaBusinessElement&, RomaGetOptions&, 
                       void*, int&, void*, int&);

    //----------------------------------------------------------------
    // Get/set the termination indicator.
    //----------------------------------------------------------------
    bool getTermIndicator() const { return m_termIndicator; }
    void setTermIndicator(bool b) { m_termIndicator = b; }

    //----------------------------------------------------------------
    // Output trace information.
    //----------------------------------------------------------------
    virtual void trace(RomaTraceStream&) const;

private:
    //----------------------------------------------------------------
    // Common routine for the get() methods.
    //----------------------------------------------------------------
    RomaBeRequest* realGet(KXCMSG&, RomaGetOptions&, 
                           RomaBusinessElement&, bool = true);

    //----------------------------------------------------------------
    // The termination indicator.
    //----------------------------------------------------------------
    bool m_termIndicator;

    //----------------------------------------------------------------
    // NOT IMPLEMENTED.
    //----------------------------------------------------------------
    RomaImplElement(const RomaImplElement&);
    RomaImplElement& operator = (const RomaImplElement&);
};


//--------------------------------------------------------------------
// The RomaImplStorageQueue class.
// This is the internal implementation of the RomaStorageQueue
// public interface.
//--------------------------------------------------------------------
class KXCODLL RomaImplStorageQueue : public RomaTraceable
{
public:
    //----------------------------------------------------------------
    // Default constructor.
    //----------------------------------------------------------------
    RomaImplStorageQueue();

    //----------------------------------------------------------------
    // Destructor.
    //----------------------------------------------------------------
    ~RomaImplStorageQueue();

    //----------------------------------------------------------------
    // Open the queue.
    //----------------------------------------------------------------
    bool open(RomaImplContext&, const char*, KXCULONG, const char*);

    //----------------------------------------------------------------
    // Close the queue.
    //----------------------------------------------------------------
    bool close();

    //----------------------------------------------------------------
    // Indicates whether the queue is open.
    //----------------------------------------------------------------
    bool isOpen() const;

    //----------------------------------------------------------------
    // Store the message on the queue.
    //----------------------------------------------------------------
    bool store(RomaStorable&, const RomaStoreOptions&);

    //----------------------------------------------------------------
    // Get a message from the queue
    // (auto-allocate the message data buffer).
    //----------------------------------------------------------------
    RomaStorable* retrieve(RomaStorageQueue&, 
                           const RomaRetrieveOptions&);

    //----------------------------------------------------------------
    // Get a message from the queue
    // (the specified buffer will be used to store message data).
    //----------------------------------------------------------------
    RomaStorable* retrieve(RomaStorageQueue&, 
                           void*, 
                           int&, 
                           const RomaRetrieveOptions&);

    //----------------------------------------------------------------
    // Output trace information.
    //----------------------------------------------------------------
    virtual void trace(RomaTraceStream&) const;

private:
    //----------------------------------------------------------------
    // If the queue is not open, returns false and sets the
    // corresponding error on the context.
    //----------------------------------------------------------------
    bool checkOpen(KXCULONG);

    //----------------------------------------------------------------
    // Storage queue handle.
    //----------------------------------------------------------------
    HKXCSTORQ m_handle;

    //----------------------------------------------------------------
    // Pointer to the shared context.
    //----------------------------------------------------------------
    RomaImplContext* m_context;

    //----------------------------------------------------------------
    // NOT IMPLEMENTED.
    //----------------------------------------------------------------
    RomaImplStorageQueue(const RomaImplStorageQueue&);
    RomaImplStorageQueue& operator = (const RomaImplStorageQueue&);
};


#endif // __KXCOICXT_HPP__

