//--------------------------------------------------------------------
//
// Copyright Candle Corporation 2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcoiems.cpp,v 1.4 2004/02/18 16:35:08 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcoiems.cpp,v $
// $Revision: 1.4 $
//--------------------------------------------------------------------
// The internal implementation of Roma EMS objects.
//--------------------------------------------------------------------


#include "kxcoiems.hpp"
#include "kxcoutil.hpp"
#include "kxcoipxy.hpp"
#include "kxcoimsg.hpp"
#include "kxcoierr.hpp"
#include "kxcoilst.hpp"


//====================================================================
// The RomaImplBusFlow class.
//====================================================================

RomaImplBusFlow::RomaImplBusFlow(const char* name, const char* oper)
{
    m_flow.Version = KXCBUSFLOW_VERSION;
    setFlowName(name);
    setOperationName(oper);
}

RomaImplBusFlow::~RomaImplBusFlow()
{
}

const char* RomaImplBusFlow::getFlowName() const
{
    return (char*)m_flow.BusinessFlowName;
}

void RomaImplBusFlow::setFlowName(const char* name)
{
    RomaUtil::setString(
        m_flow.BusinessFlowName, name, sizeof(m_flow.BusinessFlowName));
}

const char* RomaImplBusFlow::getOperationName() const
{
    return (char*)m_flow.OperationName;
}

void RomaImplBusFlow::setOperationName(const char* oper)
{
    RomaUtil::setString(
        m_flow.OperationName, oper, sizeof(m_flow.OperationName));
}

void RomaImplBusFlow::populate(KXCBUSFLOW& bf) const
{
    bf = m_flow;
}


//====================================================================
// The RomaImplBusService class.
//====================================================================

RomaImplBusService::RomaImplBusService(const char* name)
{
    KXCSERVICE s = { KXCSERVICE_DEFAULT };
    m_service = s;
    setName(name);
}

RomaImplBusService::~RomaImplBusService()
{
}

const char* RomaImplBusService::getName() const
{
    return (char*)m_service.ServiceName;
}

void RomaImplBusService::setName(const char* name)
{
    RomaUtil::setString(
        m_service.ServiceName, name, sizeof(m_service.ServiceName));
}

void RomaImplBusService::populate(KXCSERVICE& bs) const
{
    bs = m_service;
}


//====================================================================
// The RomaImplEteInfo class.
//====================================================================

RomaImplEteInfo::RomaImplEteInfo() : m_data(NULL), m_length(0)
{
}

RomaImplEteInfo::RomaImplEteInfo(const void* data, unsigned long length)
    : m_data(NULL), m_length(0)
{
    if (data != NULL && length)
    {
        m_data = new char[length];
        memcpy(m_data, data, length);
        m_length = length;
    }
}

RomaImplEteInfo::~RomaImplEteInfo()
{
    delete[] m_data;
}

const void* RomaImplEteInfo::getData(unsigned long& length) const
{
    length = m_length;
    return m_data;
}


//====================================================================
// The RomaImplRomaConfig class.
//====================================================================

RomaImplRomaConfig::RomaImplRomaConfig()
{
    KXCEXCROMACONF rc = { KXCEXCROMACONF_DEFAULT };
    m_conf = rc;
}

RomaImplRomaConfig::RomaImplRomaConfig(const KXCEXCROMACONF& rc)
    : m_conf(rc)
{
}

RomaImplRomaConfig::~RomaImplRomaConfig()
{
}

unsigned long RomaImplRomaConfig::getElementType() const
{
    return m_conf.ElementType;
}

const char* RomaImplRomaConfig::getElementName() const
{
    return (char*)m_conf.ElementName;
}

const char* RomaImplRomaConfig::getComponentName() const
{
    return (char*)m_conf.ComponentName;
}

const char* RomaImplRomaConfig::getQueueName() const
{
    return (char*)m_conf.QueueName;
}

const char* RomaImplRomaConfig::getServiceName() const
{
    return (char*)m_conf.ServiceName;
}

const char* RomaImplRomaConfig::getNodeName() const
{
    return (char*)m_conf.NodeName;
}


//====================================================================
// The RomaImplExcInfo class.
//====================================================================

RomaImplExcInfo::RomaImplExcInfo()
    : m_ed(NULL), m_ts(NULL), m_rc(NULL), m_sc(NULL),
      m_udb(NULL), m_udl(0)
{
}

RomaImplExcInfo::RomaImplExcInfo(const KXCEXCEPTION& ex)
    : m_ed(NULL), m_ts(NULL), m_rc(NULL), m_sc(NULL), 
      m_udb(NULL), m_udl(0)
{
    if ((ex.ExceptionFlags & KXCEXC_ERRORDATA) != 0)
    {
        m_ed = new RomaImplExcErrorData(ex.ErrorData);
    }

    if ((ex.ExceptionFlags & KXCEXC_TIMESTAMP) != 0)
    {
        m_ts = new RomaTimestamp(
            ex.TimeStamp.Seconds, ex.TimeStamp.Millisecs);
    }

    if ((ex.ExceptionFlags & KXCEXC_ROMADATA) != 0)
    {
        m_rc = new RomaImplRomaConfig(ex.RomaConfData);
    }

    if ((ex.ExceptionFlags & KXCEXC_SYSTEMDATA) != 0)
    {
        m_sc = new RomaImplSysConfig(ex.SystemConfData);
    }

    if ((ex.ExceptionFlags & KXCEXC_USERDATA) != 0)
    {
        m_udb = new char[ex.UserData.UserBufLen];

        if (m_udb != NULL)
        {
            m_udl = ex.UserData.UserBufLen;
            memcpy(m_udb, ex.UserData.UserBuf.pBuffer, m_udl);
        }
    }
}

RomaImplExcInfo::~RomaImplExcInfo()
{
    clear();
}

void RomaImplExcInfo::clear()
{
    delete[] m_udb;
    m_udb = NULL;
    m_udl = 0;

    delete m_ts;
    m_ts = NULL;

    if (m_rc != NULL) 
    {
        m_rc->releaseInstance();
        m_rc = NULL;
    }

    if (m_sc != NULL) 
    {
        m_sc->releaseInstance();
        m_sc = NULL;
    }

    if (m_ed != NULL) 
    {
        m_ed->releaseInstance();
        m_ed = NULL;
    }
}

RomaTimestamp* RomaImplExcInfo::getTimestamp()
{
    return m_ts;
}

RomaImplRomaConfig* RomaImplExcInfo::getRomaConfig()
{
    return m_rc;
}

RomaImplSysConfig* RomaImplExcInfo::getSystemConfig()
{
    return m_sc;
}

RomaImplExcErrorData* RomaImplExcInfo::getErrorData()
{
    return m_ed;
}

const void* RomaImplExcInfo::getUserData(unsigned long& length) const
{
    length = m_udl;
    return m_udb;
}


//====================================================================
// The RomaImplExcErrorData class.
//====================================================================

RomaImplExcErrorData::RomaImplExcErrorData()
    : m_call(0), m_re(NULL)
{
    memset(m_queue, 0, sizeof(m_queue));
}

RomaImplExcErrorData::RomaImplExcErrorData(const KXCEXCERRDATA& ed)
    : m_call(ed.CallType)
{
    memcpy(m_queue, ed.QueueName, sizeof(m_queue));
    m_re = RomaImplProxy::createPublic(new RomaImplError(ed.Error));
}

RomaImplExcErrorData::~RomaImplExcErrorData()
{
    delete m_re;
}

const RomaError& RomaImplExcErrorData::getError()
{
    if (m_re == NULL) m_re = new RomaError;
    return *m_re;
}

unsigned long RomaImplExcErrorData::getCallType() const
{
    return m_call;
}

const char* RomaImplExcErrorData::getQueueName() const
{
    return m_queue;
}


//====================================================================
// The RomaImplRoute class.
//====================================================================

RomaImplRoute::RomaImplRoute()
{
    memset(&m_route, 0, sizeof(m_route));
    m_route.NodeVersion = KXCNODEVER_CURR;
}

RomaImplRoute::RomaImplRoute(const KXCWROUTE& rt) : m_route(rt)
{
}

RomaImplRoute::RomaImplRoute(const KXCWROUTE_3& rt)
{
    memcpy(&m_route, &rt, sizeof(KXCWROUTE_3));
    *m_route.BusinessService = *m_route.BusinessFlow = 
        *m_route.Operation = 0;
}

RomaImplRoute::~RomaImplRoute()
{
}

RomaImplRoute::RomaImplRoute(const RomaImplRoute& r) : m_route(r.m_route)
{
}

RomaImplRoute& RomaImplRoute::operator = (const RomaImplRoute& r)
{
    if (this != &r)
    {
        m_route = r.m_route;
    }

    return *this;
}

unsigned long RomaImplRoute::getType() const
{
    return m_route.RouteType;
}

void RomaImplRoute::setType(unsigned long type)
{
    m_route.RouteType = type;
}

unsigned long RomaImplRoute::getFlags() const
{
    return m_route.RouteFlags;
}

void RomaImplRoute::setFlags(unsigned long flags)
{
    m_route.RouteFlags = flags;
}

const char* RomaImplRoute::getTargetQueue() const
{
    return (char*)m_route.TargetQueue;
}

void RomaImplRoute::setTargetQueue(const char* str)
{
    RomaUtil::setString(
        m_route.TargetQueue, str, sizeof(m_route.TargetQueue));
}

const char* RomaImplRoute::getTargetElement() const
{
    return (char*)m_route.TargetElement;
}

void RomaImplRoute::setTargetElement(const char* str)
{
    RomaUtil::setString(
        m_route.TargetElement, str, sizeof(m_route.TargetElement));
}

const char* RomaImplRoute::getTargetComponent() const
{
    return (char*)m_route.TargetComponent;
}

void RomaImplRoute::setTargetComponent(const char* str)
{
    RomaUtil::setString(
        m_route.TargetComponent, str, sizeof(m_route.TargetComponent));
}

const char* RomaImplRoute::getTargetCodePage() const
{
    return (char*)m_route.TargetCodePage;
}

void RomaImplRoute::setTargetCodePage(const char* str)
{
    RomaUtil::setString(
        m_route.TargetCodePage, str, sizeof(m_route.TargetCodePage));
}

const char* RomaImplRoute::getSourceService() const
{
    return (char*)m_route.BusinessService;
}

void RomaImplRoute::setSourceService(const char* str)
{
    RomaUtil::setString(
        m_route.BusinessService, str, sizeof(m_route.BusinessService));
}

const char* RomaImplRoute::getSourceFlow() const
{
    return (char*)m_route.BusinessFlow;
}

void RomaImplRoute::setSourceFlow(const char* str)
{
    RomaUtil::setString(
        m_route.BusinessFlow, str, sizeof(m_route.BusinessFlow));
}

const char* RomaImplRoute::getSourceOperation() const
{
    return (char*)m_route.Operation;
}

void RomaImplRoute::setSourceOperation(const char* str)
{
    RomaUtil::setString(
        m_route.Operation, str, sizeof(m_route.Operation));
}

void RomaImplRoute::populate(KXCWROUTE& r) const
{
    r = m_route;
}


//====================================================================
// The RomaImplSysConfig class.
//====================================================================

RomaImplSysConfig::RomaImplSysConfig()
{
    memset(&m_conf, 0, sizeof(m_conf));
}

RomaImplSysConfig::RomaImplSysConfig(const KXCEXCSYSCONF& sc)
    : m_conf(sc)
{
}

RomaImplSysConfig::~RomaImplSysConfig()
{
}

unsigned long RomaImplSysConfig::getEnvType() const
{
    return m_conf.EnvironmentType;
}

const char* RomaImplSysConfig::getDomainName() const
{
    return (char*)m_conf.DomainName;
}

const char* RomaImplSysConfig::getProgramName() const
{
    return (char*)m_conf.ProgramName;
}

const char* RomaImplSysConfig::getUserId() const
{
    return (char*)m_conf.UserId;
}

unsigned long RomaImplSysConfig::getThreadId() const
{
    return m_conf.ThreadId;
}

unsigned long RomaImplSysConfig::getProcessId() const
{
    return m_conf.ProcessId;
}


//====================================================================
// The RomaImplTrailer class.
//====================================================================

RomaImplTrailer::RomaImplTrailer()
    : m_type(0), m_method(0), m_length(0), m_data(NULL)
{
}

RomaImplTrailer::RomaImplTrailer(unsigned long type, unsigned long method,
                                 const void* data, unsigned long length)
    : m_type(type), m_method(method), m_length(length), m_data(NULL)
{
    setData(data, length);
}

RomaImplTrailer::RomaImplTrailer(const PKXCTHEMEPTRLDATA pData)
    : m_type(pData->Type), m_method(pData->Method), m_length(0), m_data(NULL)
{
    setData(pData + 1, pData->BufLen);
}

RomaImplTrailer::~RomaImplTrailer()
{
    delete[] m_data;
}

RomaImplTrailer::RomaImplTrailer(const RomaImplTrailer& tr)
    : m_type(tr.m_type), m_method(tr.m_method), m_length(0), m_data(NULL)
{
    setData(tr.m_data, tr.m_length);
}

RomaImplTrailer& RomaImplTrailer::operator=(const RomaImplTrailer& tr)
{
    if (this != &tr)
    {
        m_type = tr.m_type;
        m_method = tr.m_method;
        setData(tr.m_data, tr.m_length);
    }

    return *this;
}

unsigned long RomaImplTrailer::getType() const
{
    return m_type;
}

void RomaImplTrailer::setType(unsigned long type)
{
    m_type = type;
}

unsigned long RomaImplTrailer::getMethod() const
{
    return m_method;
}

void RomaImplTrailer::setMethod(unsigned long method)
{
    m_method = method;
}

const void* RomaImplTrailer::getData(unsigned long& length) const
{
    length = m_length;
    return m_data;
}

void RomaImplTrailer::setData(const void* data, unsigned long length)
{
    delete[] m_data;

    if (data != NULL && length)
    {
        m_data = new char[length];
        memcpy(m_data, data, length);
        m_length = length;
    }
    else
    {
        m_data = NULL;
        m_length = 0;
    }
}


//====================================================================
// The RomaImplWfService class.
//====================================================================

RomaImplWfService::RomaImplWfService()
{
    memset(&m_data, 0, sizeof(m_data));
}

RomaImplWfService::RomaImplWfService(const KXCWSERVICE& ws)
    : m_data(ws)
{
}

RomaImplWfService::RomaImplWfService(const KXCWSERVICE_2& ws)
{
    memcpy(&m_data, &ws, sizeof(KXCWSERVICE_2));
    *m_data.BusinessService = *m_data.BusinessFlow = *m_data.Operation = 0;
}

RomaImplWfService::~RomaImplWfService()
{
}

RomaImplWfService::RomaImplWfService(const RomaImplWfService& ws)
    : m_data(ws.m_data)
{
}

RomaImplWfService& 
RomaImplWfService::operator=(const RomaImplWfService& ws)
{
    if (this != &ws)
    {
        m_data = ws.m_data;
    }

    return *this;
}

const char* RomaImplWfService::getService() const
{
    return (char*)m_data.Service;
}

void RomaImplWfService::setService(const char* str)
{
    RomaUtil::setString(
        m_data.Service, str, sizeof(m_data.Service));
}

const char* RomaImplWfService::getComponent() const
{
    return (char*)m_data.CurComponent;
}

void RomaImplWfService::setComponent(const char* str)
{
    RomaUtil::setString(
        m_data.CurComponent, str, sizeof(m_data.CurComponent));
}

const char* RomaImplWfService::getAudit() const
{
    return (char*)m_data.AuditName;
}

void RomaImplWfService::setAudit(const char* str)
{
    RomaUtil::setString(
        m_data.AuditName, str, sizeof(m_data.AuditName));
}

const char* RomaImplWfService::getSourceService() const
{
    return (char*)m_data.BusinessService;
}

void RomaImplWfService::setSourceService(const char* str)
{
    RomaUtil::setString(
        m_data.BusinessFlow, str, sizeof(m_data.BusinessService));
}

const char* RomaImplWfService::getSourceFlow() const
{
    return (char*)m_data.BusinessFlow;
}

void RomaImplWfService::setSourceFlow(const char* str)
{
    RomaUtil::setString(
        m_data.BusinessFlow, str, sizeof(m_data.BusinessFlow));
}

const char* RomaImplWfService::getSourceOperation() const
{
    return (char*)m_data.Operation;
}

void RomaImplWfService::setSourceOperation(const char* str)
{
    RomaUtil::setString(
        m_data.Operation, str, sizeof(m_data.Operation));
}

void RomaImplWfService::populate(KXCWSERVICE& ws) const
{
    ws = m_data;
}


//====================================================================
// The RomaImplWorkflow class.
//====================================================================

RomaImplWorkflow::RomaImplWorkflow()
{
    m_wserv = new RomaImplWfService;
    m_rlist = new RomaImplRouteList;
}

RomaImplWorkflow::RomaImplWorkflow(const KXCWFLOW& wf)
{
    if (wf.Version == KXCWFLOW_VERSION)
    {
        m_wserv = new RomaImplWfService(wf.Service);
    }
    else
    {
        PKXCWFLOW_4 pWFlow = (PKXCWFLOW_4)&wf;
        m_wserv = new RomaImplWfService(pWFlow->Service);
    }

    m_rlist = new RomaImplRouteList(wf);
}

RomaImplWorkflow::~RomaImplWorkflow()
{
    m_wserv->releaseInstance();
    m_rlist->releaseInstance();
}

RomaImplWorkflow::RomaImplWorkflow(const RomaImplWorkflow& wf)
{
    m_wserv = new RomaImplWfService(*wf.m_wserv);
    m_rlist = new RomaImplRouteList(*wf.m_rlist);
}

RomaImplWorkflow& RomaImplWorkflow::operator=(const RomaImplWorkflow& wf)
{
    if (this != &wf)
    {
        *m_wserv = *wf.m_wserv;
        *m_rlist = *wf.m_rlist;
    }

    return *this;
}

RomaImplWfService* RomaImplWorkflow::getWorkflowService()
{
    return m_wserv;
}

void RomaImplWorkflow::setWorkflowService(const RomaWorkflowService& ws)
{
    m_wserv->releaseInstance();
    m_wserv = RomaImplProxy::getImplPtr(ws);
    m_wserv->lockInstance();
}

RomaImplRouteList* RomaImplWorkflow::getRoutes()
{
    return m_rlist;
}

void RomaImplWorkflow::setRoutes(const RomaRouteList& list)
{
    m_rlist->releaseInstance();
    m_rlist = RomaImplProxy::getImplPtr(list);
    m_rlist->lockInstance();
}


//====================================================================
// The RomaImplMsgInfo class.
//====================================================================

RomaImplMsgInfo::RomaImplMsgInfo()
    : m_msg(NULL), m_wflow(NULL), m_einfo(NULL), m_theme(NULL_HKXCTHEME)
{
}

RomaImplMsgInfo::RomaImplMsgInfo(const KXCMSGINFO& mi, RomaImplContext& cxt)
    : m_msg(NULL), m_wflow(NULL), m_einfo(NULL), m_theme(NULL_HKXCTHEME)
{
    if ((mi.Status & KXCMSGPART_MESSAGE) != 0)
    {
        m_msg = new RomaImplMsgIncoming(
            mi.Message, NULL_HKXCTHEME, NULL, cxt);
    }

    if ((mi.Status & KXCMSGPART_WORKFLOW) != 0)
    {
        PKXCWFLOW pFlow = (PKXCWFLOW)mi.WorkFlow.pBuffer;
        m_wflow = new RomaImplWorkflow(*pFlow);
    }

    if ((mi.Status & KXCMSGPART_EXCEPTION) != 0)
    {
        m_einfo = new RomaImplExcInfo(mi.Exception);
    }

    if ((mi.Status & KXCMSGPART_HTHEME) != 0)
    {
        m_theme = mi.hTheme;
    }
}

RomaImplMsgInfo::~RomaImplMsgInfo()
{
    if (m_msg != NULL) m_msg->releaseInstance();
    if (m_wflow != NULL) m_wflow->releaseInstance();
    if (m_einfo != NULL) m_einfo->releaseInstance();
}

RomaImplWorkflow* RomaImplMsgInfo::getWorkflow()
{
    return m_wflow;
}

HKXCTHEME RomaImplMsgInfo::getTheme() const
{
    return m_theme;
}

RomaImplExcInfo* RomaImplMsgInfo::getException()
{
    return m_einfo;
}

RomaImplMsgIncoming* RomaImplMsgInfo::getMessage()
{
    return m_msg;
}


//====================================================================
// The RomaImplThemeInfo class.
//====================================================================

RomaImplThemeInfo::RomaImplThemeInfo()
    : m_retry(0), m_confirm(0), m_ete(NULL), m_wflow(NULL),
      m_trailers(NULL), m_themeId(NULL)
{
}

RomaImplThemeInfo::~RomaImplThemeInfo()
{
    if (m_ete != NULL) m_ete->releaseInstance();
    if (m_wflow != NULL) m_wflow->releaseInstance();
    if (m_trailers != NULL) m_trailers->releaseInstance();
    if (m_themeId != NULL) m_themeId->releaseInstance();
}

RomaImplThemeInfo::RomaImplThemeInfo(const void* buf, bool client)
{
    void* ptr;
    kxcQryLookUp(buf, KXCTID_THEMEID, &ptr);
    m_themeId = new RomaImplMsgId(ptr);

    KXCULONG* lp;
    kxcQryLookUp(buf, KXCTID_RETRY, &lp);
    m_retry = *lp;

    PKXCWFLOW pFlow;
    KXCULONG tmp = client ? KXCTID_REQWFLOW : KXCTID_RSPWFLOW;
    kxcQryLookUp(buf, tmp, &pFlow);
    m_wflow = new RomaImplWorkflow(*pFlow);

    kxcQryLookUpWithLen(buf, KXCTID_ETEINFO, &ptr, &tmp);
    m_ete = new RomaImplEteInfo(ptr, tmp);

    PKXCTHEMEPTRLHDR thdr;
    kxcQryLookUp(buf, KXCTID_PTRAILER, &thdr);

    if (thdr->Number)
    {
        m_trailers = new RomaImplTrailerList;

        PKXCTHEMEPTRLDATA pData = (PKXCTHEMEPTRLDATA)(thdr + 1);

        for (KXCULONG i = 0; i < thdr->Number; ++i)
        {
            RomaImplTrailer* trailer = new RomaImplTrailer(pData);
            m_trailers->add(trailer);
            trailer->releaseInstance();
            
            char* ptr = (char*)(pData + 1);
            pData = (PKXCTHEMEPTRLDATA)(ptr + pData->BufLen);
        }
    }
    else
    {
        m_trailers = NULL;
    }

    KXCCONFIRMSTATUS* conf;
    kxcQryLookUp(buf, KXCTID_CONFIRM, &conf);
    m_confirm = (conf->CoA) ? KXCPOPT_COA : 0;
    if (conf->CoD) m_confirm |= KXCPOPT_COD;
}

RomaImplThemeInfo::RomaImplThemeInfo(const RomaImplThemeInfo& ti)
    : m_retry(ti.m_retry), m_confirm(ti.m_confirm), m_ete(ti.m_ete),
      m_wflow(NULL), m_trailers(NULL), m_themeId(NULL)
{
    if (m_ete != NULL) m_ete->lockInstance();

    if (ti.m_wflow != NULL)
    {
        m_wflow = new RomaImplWorkflow(*ti.m_wflow);
    }
        
    if (ti.m_trailers != NULL)
    {
        m_trailers = new RomaImplTrailerList(*ti.m_trailers);
    }

    if (ti.m_themeId != NULL)
    {
        m_themeId = new RomaImplMsgId(*ti.m_themeId);
    }
}

RomaImplThemeInfo& RomaImplThemeInfo::operator=(const RomaImplThemeInfo& ti)
{
    if (this != &ti)
    {
        m_retry = ti.m_retry;
        m_confirm = ti.m_confirm;

        if (m_ete != NULL) m_ete->releaseInstance();
        m_ete = ti.m_ete;
        if (m_ete != NULL) m_ete->lockInstance();

        if (m_wflow != NULL)
        {
            m_wflow->releaseInstance();
            m_wflow = NULL;
        }

        if (ti.m_wflow != NULL)
        {
            m_wflow = new RomaImplWorkflow(*ti.m_wflow);
        }
        
        if (m_trailers != NULL)
        {
            m_trailers->releaseInstance();
            m_trailers = NULL;
        }

        if (ti.m_trailers != NULL)
        {
            m_trailers = new RomaImplTrailerList(*ti.m_trailers);
        }

        if (ti.m_themeId != NULL)
        {
            if (m_themeId == NULL)
            {
                m_themeId = new RomaImplMsgId(*ti.m_themeId);
            }
            else
            {
                *m_themeId = *ti.m_themeId;
            }
        }
    }

    return *this;
}

RomaImplWorkflow* RomaImplThemeInfo::getWorkflow()
{
    if (m_wflow == NULL)
    {
        m_wflow = new RomaImplWorkflow;
    }

    return m_wflow;
}

void RomaImplThemeInfo::setWorkflow(const RomaWorkflow& wf)
{
    if (m_wflow != NULL) m_wflow->releaseInstance();
    m_wflow = RomaImplProxy::getImplPtr(wf);
    m_wflow->lockInstance();
}

unsigned long RomaImplThemeInfo::getRetryCount() const
{
    return m_retry;
}

void RomaImplThemeInfo::setRetryCount(unsigned long rc)
{
    m_retry = rc;
}

RomaImplMsgId* RomaImplThemeInfo::getThemeId()
{
    if (m_themeId == NULL)
    {
        m_themeId = new RomaImplMsgId;
    }

    return m_themeId;
}

void RomaImplThemeInfo::setThemeId(const RomaMsgId& id)
{
    if (m_themeId == NULL)
    {
        m_themeId = new RomaImplMsgId(RomaImplProxy::getImpl(id));
    }
    else
    {
        m_themeId->set((const void*)id);
    }
}

unsigned long RomaImplThemeInfo::getConfirmFlags() const
{
    return m_confirm;
}

void RomaImplThemeInfo::setConfirmFlags(unsigned long cf)
{
    m_confirm = cf;
}

RomaImplTrailerList* RomaImplThemeInfo::getTrailers()
{
    if (m_trailers == NULL)
    {
        m_trailers = new RomaImplTrailerList;
    }

    return m_trailers;
}

void RomaImplThemeInfo::setTrailers(const RomaTrailerList& tl)
{
    if (m_trailers != NULL) m_trailers->releaseInstance();
    m_trailers = RomaImplProxy::getImplPtr(tl);
    m_trailers->lockInstance();
}

RomaImplEteInfo* RomaImplThemeInfo::getEteInfo()
{
    return m_ete;
}

void RomaImplThemeInfo::setEteInfo(const RomaEteInfo& info)
{
    if (m_ete != NULL) m_ete->releaseInstance();
    m_ete = RomaImplProxy::getImplPtr(info);
    if (m_ete != NULL) m_ete->lockInstance();
}


//====================================================================
// The RomaImplTrailerList class.
//====================================================================

RomaImplTrailerList::RomaImplTrailerList(const RomaImplTrailerList& list)
    : RomaImplList<RomaImplTrailer>(list)
{
}

RomaImplTrailerList&
RomaImplTrailerList::operator=(const RomaImplTrailerList& list)
{
    RomaImplList<RomaImplTrailer>::operator=(list);
    return *this;
}


//====================================================================
// The RomaImplRouteList class.
//====================================================================

RomaImplRouteList::RomaImplRouteList(const RomaImplRouteList& list)
    : RomaImplList<RomaImplRoute>(list)
{
}

RomaImplRouteList&
RomaImplRouteList::operator=(const RomaImplRouteList& list)
{
    RomaImplList<RomaImplRoute>::operator=(list);
    return *this;
}

RomaImplRouteList::RomaImplRouteList(const KXCWFLOW& wf)
{
    RomaImplRoute* p = m_first;
    KXCULONG i = 0;

    if (wf.Version == KXCWFLOW_VERSION)
    {
        for (; i < wf.NumRoutes; ++i)
        {
            RomaImplRoute* r = new RomaImplRoute(wf.Routes[i]);

            if (m_first == NULL)
            {
                m_first = r;
            }
            else
            {
                p->setNextItem(r);
            }

            p = r;
        }
    }
    else
    {
        PKXCWFLOW_4 pWFlow = (PKXCWFLOW_4)&wf;
        PKXCWROUTE_3 pRoute = pWFlow->Routes;

        for (; i < wf.NumRoutes; ++i, ++pRoute)
        {
            RomaImplRoute* r = new RomaImplRoute(*pRoute);

            if (m_first == NULL)
            {
                m_first = r;
            }
            else
            {
                p->setNextItem(r);
            }

            p = r;
        }
    }
}


//====================================================================
// The RomaImplRouteListIter class.
//====================================================================

RomaImplRouteListIter::RomaImplRouteListIter(RomaImplRouteList& list)
    : RomaImplListIter<RomaImplRoute,RomaImplRouteList>(list)
{
}

RomaImplRouteListIter::RomaImplRouteListIter(
    const RomaImplRouteListIter& it)
    : RomaImplListIter<RomaImplRoute,RomaImplRouteList>(it) 
{
}

RomaImplRouteListIter& 
RomaImplRouteListIter::operator=(const RomaImplRouteListIter& it)
{
    RomaImplListIter<RomaImplRoute,RomaImplRouteList>::operator=(it);
    return *this;
}


//====================================================================
// The RomaImplTrailerListIter class.
//====================================================================

RomaImplTrailerListIter::RomaImplTrailerListIter(
    RomaImplTrailerList& list)
    : RomaImplListIter<RomaImplTrailer,RomaImplTrailerList>(list) 
{
}

RomaImplTrailerListIter::RomaImplTrailerListIter(
    const RomaImplTrailerListIter& it)
    : RomaImplListIter<RomaImplTrailer,RomaImplTrailerList>(it) 
{
}

RomaImplTrailerListIter& 
RomaImplTrailerListIter::operator=(const RomaImplTrailerListIter& it)
{
    RomaImplListIter<RomaImplTrailer,RomaImplTrailerList>::operator=(it);
    return *this;
}


