//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcoierr.cpp,v 1.14 2004/02/18 16:35:08 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcoierr.cpp,v $
// $Revision: 1.14 $
//--------------------------------------------------------------------
// The internal implementation of the RomaError public interface.
//--------------------------------------------------------------------

#include "kxcointa.hpp"
#include "kxcoierr.hpp"
#include "kxcodefs.hpp"

TRACE_MODULE_VERSION($Revision: 1.14 $);

//--------------------------------------------------------------------
// Links code with its text representation.
//--------------------------------------------------------------------
struct KxcoErrorText
{
    KXCULONG m_code;
    const char* m_text;
};

//--------------------------------------------------------------------
// Initializer for the KxcoErrorText structure.
//--------------------------------------------------------------------
#define KXCO_ERROR_TEXT(x) {x,#x}

//--------------------------------------------------------------------
// The codes returned by the KXC C++ API.
//--------------------------------------------------------------------
static KxcoErrorText s_retCodes[] =
{
    KXCO_ERROR_TEXT(KXCORC_MARSHAL_REQUEST),
    KXCO_ERROR_TEXT(KXCORC_MARSHAL_RESPONSE),
    KXCO_ERROR_TEXT(KXCORC_MARSHAL_STORABLE),
    KXCO_ERROR_TEXT(KXCORC_CREATE_REQUEST),
    KXCO_ERROR_TEXT(KXCORC_CREATE_RESPONSE),
    KXCO_ERROR_TEXT(KXCORC_CREATE_STORABLE),
    KXCO_ERROR_TEXT(KXCORC_BUFFER_ERROR),
    KXCO_ERROR_TEXT(KXCORC_NOT_OPEN),
    KXCO_ERROR_TEXT(KXCORC_NOT_CONNECTED),
    KXCO_ERROR_TEXT(KXCORC_ALREADY_CONNECTED),
    KXCO_ERROR_TEXT(KXCORC_ALREADY_OPEN)
};

//--------------------------------------------------------------------
// Error types.
//--------------------------------------------------------------------
static KxcoErrorText s_errorTypes[] =
{
    KXCO_ERROR_TEXT(KXCETYPE_CRITICAL),
    KXCO_ERROR_TEXT(KXCETYPE_ERROR),
    KXCO_ERROR_TEXT(KXCETYPE_TRANSIENT),
    KXCO_ERROR_TEXT(KXCETYPE_API)
};


#define FAIL_CODE   -1


//--------------------------------------------------------------------
// Default constructor.
//--------------------------------------------------------------------
RomaImplError::RomaImplError() 
    : m_status(Roma::STATUS_NONE), m_buffer(NULL), m_size(0),
      m_extraBuf1(NULL), m_extraBuf2(NULL), 
      m_extraSize1(0), m_extraSize2(0)
{
    TRACE_ENTRY(RomaImplError::RomaImplError, << TRACE(this));

    KXCSUBERRINFO tmp = { KXCSUBERRINFO_DEFAULT };
    m_info1 = m_info2 = tmp;

    TRACE_VOID_RETURN;
}

RomaImplError::RomaImplError(const KXCERROR& e)
    : m_status(e.Status), m_buffer(NULL), m_size(0),
      m_extraBuf1(NULL), m_extraBuf2(NULL), 
      m_extraSize1(0), m_extraSize2(0)
{
    const KXCSUBERROR* pErr;
    size_t sl;

    KXCSUBERRINFO tmp = { KXCSUBERRINFO_DEFAULT };
    m_info1 = m_info2 = tmp;

    switch (m_status)
    {
        case Roma::STATUS_BOTH:
            pErr = &e.SubResult;
            m_info2.LastCompCode = pErr->LastCompCode;
            m_info2.LastReturnCode = pErr->LastReturnCode;
            m_info2.LastOpType = pErr->LastOpType;
            m_info2.LastReason = pErr->LastReason;
            m_info2.LastPosition = pErr->LastPosition;
            m_info2.LastErrorType = pErr->LastErrorType;

            sl = strlen((char*)pErr->LastLocation);

            if (sl)
            {
                ++sl;
                m_info2.Data = new KXCCHAR[sl];
                memcpy(m_info2.Data, pErr->LastLocation, sl);
                m_info2.DataLen = sl;
            }

        case Roma::STATUS_PRIMARY:
            pErr = &e.Result;
            m_info1.LastCompCode = pErr->LastCompCode;
            m_info1.LastReturnCode = pErr->LastReturnCode;
            m_info1.LastOpType = pErr->LastOpType;
            m_info1.LastReason = pErr->LastReason;
            m_info1.LastPosition = pErr->LastPosition;
            m_info1.LastErrorType = pErr->LastErrorType;

            sl = strlen((char*)pErr->LastLocation);

            if (sl)
            {
                ++sl;
                m_info1.Data = new KXCCHAR[sl];
                memcpy(m_info1.Data, pErr->LastLocation, sl);
                m_info1.DataLen = sl;
            }
    }
}

//--------------------------------------------------------------------
// Copy constructor.
//--------------------------------------------------------------------
RomaImplError::RomaImplError(const RomaImplError& error)
    : m_status(Roma::STATUS_NONE), m_size(0), m_buffer(NULL),
      m_extraBuf1(NULL), m_extraBuf2(NULL),
      m_extraSize1(0), m_extraSize2(0)
{
    TRACE_ENTRY(RomaImplError::RomaImplError, 
        << TRACE(this) << TRACE(error));

    KXCSUBERRINFO tmp = { KXCSUBERRINFO_DEFAULT };
    m_info1 = m_info2 = tmp;

    *this = error;

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Destructor.
//--------------------------------------------------------------------
RomaImplError::~RomaImplError()
{
    TRACE_ENTRY(RomaImplError::~RomaImplError, << TRACE(this));

    clearInfo(m_info1);
    clearInfo(m_info2);

    delete[] m_buffer;
    delete[] m_extraBuf1;
    delete[] m_extraBuf2;

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Get the error status.
//--------------------------------------------------------------------
Roma::ErrorStatus RomaImplError::getStatus() const
{
    switch (m_status)
    {
    case KXCERRSTAT_NONE:
        return Roma::STATUS_NONE;

    case KXCERRSTAT_MAIN:
        return Roma::STATUS_PRIMARY;
    }

    return Roma::STATUS_BOTH;
}

//--------------------------------------------------------------------
// Get the completion code.
//--------------------------------------------------------------------
Roma::CompCode
RomaImplError::getCompCode(Roma::ErrorSubclass which) const
{
    if (!anyError())
    {
        return Roma::CC_OK;
    }

    return (Roma::CompCode)(which == Roma::SUBCLASS_SECONDARY ?
                            m_info2.LastCompCode :
                            m_info1.LastCompCode);
}

//--------------------------------------------------------------------
// Get the return code.
//--------------------------------------------------------------------
int RomaImplError::getReturnCode(Roma::ErrorSubclass which) const
{
    if (!anyError())
    {
        return KXCRC_OK;
    }

    return (which == Roma::SUBCLASS_SECONDARY ?
            m_info2.LastReturnCode :
            m_info1.LastReturnCode);
}

//--------------------------------------------------------------------
// Get the error reason.
//--------------------------------------------------------------------
int RomaImplError::getReason(Roma::ErrorSubclass which) const
{
    if (!anyError())
    {
        return FAIL_CODE;
    }

    return (which == Roma::SUBCLASS_SECONDARY ?
            m_info2.LastReason :
            m_info1.LastReason);
}

//--------------------------------------------------------------------
// Get the operation type.
//--------------------------------------------------------------------
int RomaImplError::getOperationType(Roma::ErrorSubclass which) const
{
    if (!anyError())
    {
        return FAIL_CODE;
    }

    return (which == Roma::SUBCLASS_SECONDARY ?
            m_info2.LastOpType :
            m_info1.LastOpType);
}

//--------------------------------------------------------------------
// Get the error position.
//--------------------------------------------------------------------
int RomaImplError::getPosition(Roma::ErrorSubclass which) const
{
    if (!anyError())
    {
        return FAIL_CODE;
    }

    return (which == Roma::SUBCLASS_SECONDARY ?
            m_info2.LastPosition :
            m_info1.LastPosition);
}

//--------------------------------------------------------------------
// Get the error type.
//--------------------------------------------------------------------
Roma::ErrorType 
RomaImplError::getType(Roma::ErrorSubclass which) const
{
    if (!anyError())
    {
        return (Roma::ErrorType)FAIL_CODE;
    }

    return (Roma::ErrorType)(which == Roma::SUBCLASS_SECONDARY ?
                             m_info2.LastErrorType :
                             m_info1.LastErrorType);
}

//--------------------------------------------------------------------
// Get additional error information.
//--------------------------------------------------------------------
const char*
RomaImplError::getAdditionalInfo(Roma::ErrorSubclass which)
{
    TRACE_ENTRY(RomaImplError::getAdditionalInfo, << TRACE(this)
                                                  << TRACE(which));

    if (!anyError())
    {
        TRACE_EVENT(KXCOM_NO_ERROR);
        TRACE_INT_RETURN(NULL);
    }

    if (which == Roma::SUBCLASS_SECONDARY)
    {
        // Format secondary error text.
        formatInfo(m_info2, m_extraBuf2, m_extraSize2);
        TRACE_INT_RETURN(m_extraBuf2);
    }

    // Format primary error text.
    formatInfo(m_info1, m_extraBuf1, m_extraSize1);
    TRACE_INT_RETURN(m_extraBuf1);
}

//--------------------------------------------------------------------
// Format additional error information into the buffer.
//--------------------------------------------------------------------
void RomaImplError::formatInfo(const KXCSUBERRINFO& info, 
                               char*& buffer,
                               unsigned& size)
{
    TRACE_ENTRY(RomaImplError::formatInfo, << TRACE(this));

    // Calculate the required buffer size.
    unsigned total = info.DataLen + info.LastModuleLen + 
        info.LastBuildLen + info.LastFunctionLen + 100;

    // If the buffer does not exist or it's size is less than
    // the required buffer size, create a new buffer.
    if (!buffer || total > size)
    {
        delete[] buffer;
        buffer = new char[total];

        if (!buffer)
        {
            // Out of memory.
            size = 0;
            TRACE_WARNING(KXCOM_MEMALLOC_FAILED(total));
            TRACE_VOID_RETURN;
        }

        size = total;
    }

    *buffer = '\0';

    // If Data is present, put it into the buffer.
    if (info.Data && info.DataLen)
    {
        sprintf(buffer, "Data(%s) ", (char*)info.Data);
    }

    // Put Module, Build and Function into the buffer.
    const char* func = ((info.LastFunction && info.LastFunctionLen) ? 
                       (const char*)info.LastFunction : "");

    const char* module = ((info.LastModule && info.LastModuleLen) ? 
                         (const char*)info.LastModule : "");

    const char* build = ((info.LastBuild && info.LastBuildLen) ? 
                        (const char*)info.LastBuild : "");

    sprintf(buffer + strlen(buffer), 
        "Module (%s:%d) Build(%s) Function(%s)",
        module, info.LastLineNum, build, func);

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Get error text.
//--------------------------------------------------------------------
const char* RomaImplError::getText(Roma::ErrorSubclass which,
                                   Roma::ErrorFormat format)
{
    TRACE_ENTRY(RomaImplError::getText, << TRACE(this)
                                        << TRACE(which)
                                        << TRACE(format));

    if (!anyError())
    {
        // No error, nothing to do further.
        TRACE_EVENT(KXCOM_NO_ERROR);
        TRACE_INT_RETURN(NULL);
    }

    if (!m_buffer)
    {
        // If the buffer does not exist, allocate 4k for it.
        m_size = 4096;
        m_buffer = new char[m_size];

        if (!m_buffer)
        {
            // Out of memory.
            TRACE_WARNING(KXCOM_MEMALLOC_FAILED(m_size));
            m_size = 0;
            TRACE_INT_RETURN(NULL);
        }
    }

    const KXCSUBERRINFO* info = 
        (which == Roma::SUBCLASS_PRIMARY ? &m_info1 : &m_info2);

    if (info->LastOpType != KXCOP_KXC_OM)
    {
        // This is not a C++ API error.
        // Use the C API method to format error data.
        KXCULONG req = m_size;
        kxcFormatError((void*)info, format, m_buffer, &req);

        if (req > m_size)
        {
            // The buffer is too small, reallocate to the
            // required size.
            m_size = req;
            delete[] m_buffer;
            m_buffer = new char[m_size];

            if (!m_buffer)
            {
                // Out of memory.
                TRACE_WARNING(KXCOM_MEMALLOC_FAILED(m_size));
                TRACE_INT_RETURN(NULL);
            }

            kxcFormatError((void*)info, format, m_buffer, &req);
        }

        TRACE_INT_RETURN(m_buffer);
    }

    // C++ API error formatting.

    int idx = 0;
    const char* retCode = NULL;
    const char* errorType = NULL;

    // Find a text representation of the return code.
    for (; idx < sizeof(s_retCodes) / sizeof(*s_retCodes); ++idx) 
    {
        if (s_retCodes[idx].m_code == info->LastReturnCode)
        {
            retCode = s_retCodes[idx].m_text;
            break;
        }
    }

    char unkRetCode[40];
    if (!retCode) 
    {
        sprintf(unkRetCode, "Unknown[0x%x]", info->LastReturnCode);
        retCode = unkRetCode;
    }

    // Find a text representation of the error type.
    for (idx = 0; idx < sizeof(s_errorTypes) / sizeof(*s_errorTypes); ++idx) 
    {
        if (s_errorTypes[idx].m_code == info->LastErrorType)
        {
            errorType = s_errorTypes[idx].m_text;
            break;
        }
    }

    char unkErrorType[40];
    if (!errorType) 
    {
        sprintf(unkErrorType, "Unknown[0x%x]", info->LastErrorType);
        errorType = unkErrorType;
    }

    // Format error data into the buffer.
    sprintf(m_buffer, "Error(%s) OpType(KXCOP_KXC_CPP) Pos(%d) ErrorType(%s)",
        retCode, info->LastPosition, errorType);

    if (format == Roma::ERROR_FULL) 
    {
        // An application requested to get full error information,
        // thus we need to put additional information into the
        // buffer also.
        const char* extra = getAdditionalInfo(which);

        if (!extra)
        {
            TRACE_INT_RETURN(m_buffer);
        }

        // Calculate the required buffer length.
        size_t fLen = strlen(m_buffer);
        size_t total = strlen(extra) + fLen + 2;

        // If the buffer is too small, expand it.
        if (total > m_size)
        {
            char* tmp = new char[total];

            if (!tmp)
            {
                // Out of memory.
                TRACE_WARNING(KXCOM_MEMALLOC_FAILED(total));
                TRACE_INT_RETURN(m_buffer);
            }

            memcpy(tmp, m_buffer, fLen + 1);
            delete[] m_buffer;
            m_buffer = tmp;
            m_size = total;
        }

        // Put additional information to the buffer.
        sprintf(m_buffer + fLen, " %s", extra);
    }

    TRACE_INT_RETURN(m_buffer);
}

//--------------------------------------------------------------------
// Set additional error information.
//--------------------------------------------------------------------
void RomaImplError::setAdditionalInfo(Roma::ErrorSubclass which, 
                                      const char* module, 
                                      const char* build, 
                                      const char* func, 
                                      const char* data)
{
    TRACE_ENTRY(RomaImplError::setAdditionalInfo, << TRACE(this)
                                                  << TRACE(which)
                                                  << TRACE(module)
                                                  << TRACE(build)
                                                  << TRACE(func)
                                                  << TRACE(data));

    KXCSUBERRINFO& info = (which == Roma::SUBCLASS_SECONDARY ? 
                           m_info2 : m_info1);

    // Remove existing data.
    clearInfo(info);

    // Copy LastModule data.
    if (module)
    {
        size_t size = strlen(module) + 1;
        info.LastModule = (PKXCCHAR)new char[size];

        if (!info.LastModule)
        {
            TRACE_WARNING(KXCOM_MEMALLOC_FAILED(size));
            TRACE_VOID_RETURN;
        }

        memcpy(info.LastModule, module, size);
        info.LastModuleLen = size - 1;
    }

    // Copy LastBuild data.
    if (build)
    {
        size_t size = strlen(build) + 1;
        info.LastBuild = (PKXCCHAR)new char[size];

        if (!info.LastBuild)
        {
            TRACE_WARNING(KXCOM_MEMALLOC_FAILED(size));
            TRACE_VOID_RETURN;
        }

        memcpy(info.LastBuild, build, size);
        info.LastBuildLen = size - 1;
    }

    // Copy LastFunction data.
    if (func)
    {
        size_t size = strlen(func) + 1;
        info.LastFunction = (PKXCCHAR)new char[size];

        if (!info.LastFunction)
        {
            TRACE_WARNING(KXCOM_MEMALLOC_FAILED(size));
            TRACE_VOID_RETURN;
        }

        memcpy(info.LastFunction, func, size);
        info.LastFunctionLen = size - 1;
    }

    // Copy Data data.
    if (data)
    {
        size_t size = strlen(data) + 1;
        info.Data = (PKXCCHAR)new char[size];

        if (!info.Data)
        {
            TRACE_WARNING(KXCOM_MEMALLOC_FAILED(size));
            TRACE_VOID_RETURN;
        }

        memcpy(info.Data, data, size);
        info.DataLen = size - 1;
    }

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Set the completion code.
//--------------------------------------------------------------------
void RomaImplError::setCompCode(Roma::ErrorSubclass which, 
                                Roma::CompCode code)
{
    if (which == Roma::SUBCLASS_SECONDARY)
    {
        m_info2.LastCompCode = code;
    }
    else
    {
        m_info1.LastCompCode = code;
    }
}

//--------------------------------------------------------------------
// Set the return code.
//--------------------------------------------------------------------
void RomaImplError::setReturnCode(Roma::ErrorSubclass which, int code)
{
    if (which == Roma::SUBCLASS_SECONDARY)
    {
        m_info2.LastReturnCode = code;
    }
    else
    {
        m_info1.LastReturnCode = code;
    }
}

//--------------------------------------------------------------------
// Set the error reason.
//--------------------------------------------------------------------
void RomaImplError::setReason(Roma::ErrorSubclass which, int reason)
{
    if (which == Roma::SUBCLASS_SECONDARY)
    {
        m_info2.LastReason = reason;
    }
    else
    {
        m_info1.LastReason = reason;
    }
}

//--------------------------------------------------------------------
// Set the operation type.
//--------------------------------------------------------------------
void RomaImplError::setOperationType(Roma::ErrorSubclass which,
                                     Roma::OperationType type)
{
    if (which == Roma::SUBCLASS_SECONDARY)
    {
        m_info2.LastOpType = type;
    }
    else
    {
        m_info1.LastOpType = type;
    }
}

//--------------------------------------------------------------------
// Set the error position.
//--------------------------------------------------------------------
void RomaImplError::setPosition(Roma::ErrorSubclass which, int pos)
{
    if (which == Roma::SUBCLASS_SECONDARY)
    {
        m_info2.LastPosition = pos;
    }
    else
    {
        m_info1.LastPosition = pos;
    }
}

//--------------------------------------------------------------------
// Set the error type.
//--------------------------------------------------------------------
void RomaImplError::setType(Roma::ErrorSubclass which,
                            Roma::ErrorType type)
{
    if (which == Roma::SUBCLASS_SECONDARY)
    {
        m_info2.LastErrorType = type;
    }
    else
    {
        m_info1.LastErrorType = type;
    }
}

//--------------------------------------------------------------------
// Set the source line number.
//--------------------------------------------------------------------
void RomaImplError::setLineNum(Roma::ErrorSubclass which, KXCULONG num)
{
    if (which == Roma::SUBCLASS_SECONDARY)
    {
        m_info2.LastLineNum = num;
    }
    else
    {
        m_info1.LastLineNum = num;
    }
}

//--------------------------------------------------------------------
// Assignment operator.
//--------------------------------------------------------------------
RomaImplError& RomaImplError::operator = (const RomaImplError& error)
{
    if (this != &error) 
    {
        m_status = error.m_status;
        capture(Roma::SUBCLASS_PRIMARY, error.m_info1);
        capture(Roma::SUBCLASS_SECONDARY, error.m_info2);
    }

    return *this;
}

//--------------------------------------------------------------------
// Fill the KXCSUBERROR structure with error data.
//--------------------------------------------------------------------
void RomaImplError::populate(Roma::ErrorSubclass which, 
                             KXCSUBERROR& tgt)
{
    const KXCSUBERRINFO& src = 
        (which == Roma::SUBCLASS_SECONDARY ? m_info2 : m_info1);

    tgt.LastCompCode = src.LastCompCode;
    tgt.LastReturnCode = src.LastReturnCode;
    tgt.LastOpType = src.LastOpType;
    tgt.LastReason = src.LastReason;
    tgt.LastPosition = src.LastPosition;
    tgt.LastErrorType = src.LastErrorType;

    const char* extra = getAdditionalInfo(which);

    if (extra)
    {
        strncpy((char*)tgt.LastLocation, extra, KXCFORMAT_BUFFER_LEN - 1);
        tgt.LastLocation[KXCFORMAT_BUFFER_LEN - 1] = 0;
    }
    else
    {
        *tgt.LastLocation = 0;
    }
}

//--------------------------------------------------------------------
// Fill the KXCERROR structure with error data.
//--------------------------------------------------------------------
void RomaImplError::populate(KXCERROR& error)
{
    error.Status = m_status;
    populate(Roma::SUBCLASS_PRIMARY, error.Result);
    populate(Roma::SUBCLASS_SECONDARY, error.SubResult);
}

//--------------------------------------------------------------------
// Set KXCSUBERROR data to the object.
//--------------------------------------------------------------------
void RomaImplError::capture(Roma::ErrorSubclass which, 
                            const KXCSUBERRINFO& src)
{
    TRACE_ENTRY(RomaImplError::capture, << TRACE(this)
                                        << TRACE(which));

    KXCSUBERRINFO& tgt = 
        (which == Roma::SUBCLASS_SECONDARY ? m_info2 : m_info1);

    // Remove existing data.
    clearInfo(tgt);

    // Copy error properties.
    tgt.LastCompCode = src.LastCompCode;
    tgt.LastReturnCode = src.LastReturnCode;
    tgt.LastOpType = src.LastOpType;
    tgt.LastReason = src.LastReason;
    tgt.LastPosition = src.LastPosition;
    tgt.LastErrorType = src.LastErrorType;
    tgt.LastLineNum = src.LastLineNum;

    // Copy LastModule data.
    if (src.LastModule && src.LastModuleLen)
    {
        size_t len = src.LastModuleLen + 1;
        tgt.LastModule = new KXCCHAR[len];

        if (!tgt.LastModule)
        {
            TRACE_WARNING(KXCOM_MEMALLOC_FAILED(len));
            TRACE_VOID_RETURN;
        }

        memcpy(tgt.LastModule, src.LastModule, len);
        tgt.LastModuleLen = src.LastModuleLen;
    }

    // Copy LastBuild data.
    if (src.LastBuild && src.LastBuildLen)
    {
        size_t len = src.LastBuildLen + 1;
        tgt.LastBuild = new KXCCHAR[len];

        if (!tgt.LastBuild)
        {
            TRACE_WARNING(KXCOM_MEMALLOC_FAILED(len));
            TRACE_VOID_RETURN;
        }

        memcpy(tgt.LastBuild, src.LastBuild, len);
        tgt.LastBuildLen = src.LastBuildLen;
    }

    // Copy LastFunction data.
    if (src.LastFunction && src.LastFunctionLen)
    {
        size_t len = src.LastFunctionLen + 1;
        tgt.LastFunction = new KXCCHAR[len];

        if (!tgt.LastFunction)
        {
            TRACE_WARNING(KXCOM_MEMALLOC_FAILED(len));
            TRACE_VOID_RETURN;
        }

        memcpy(tgt.LastFunction, src.LastFunction, len);
        tgt.LastFunctionLen = src.LastFunctionLen;
    }

    // Copy Data data.
    if (src.Data && src.DataLen)
    {
        size_t len = src.DataLen + 1;
        tgt.Data = new KXCCHAR[len];

        if (!tgt.Data)
        {
            TRACE_WARNING(KXCOM_MEMALLOC_FAILED(len));
            TRACE_VOID_RETURN;
        }

        memcpy(tgt.Data, src.Data, len);
        tgt.DataLen = src.DataLen;
    }

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Gather the last error data.
//--------------------------------------------------------------------
void RomaImplError::capture()
{
    KXCERRINFO info = { KXCERRINFO_DEFAULT };

    // Get the last error information.
    kxcGetError(&info);

    m_status = info.Status;
    capture(Roma::SUBCLASS_PRIMARY, info.Result);
    capture(Roma::SUBCLASS_SECONDARY, info.SubResult);
}

//--------------------------------------------------------------------
// Free dynamically allocated memory for error data.
//--------------------------------------------------------------------
void RomaImplError::clearInfo(KXCSUBERRINFO& info)
{
    delete[] info.LastModule;
    info.LastModule = NULL;
    info.LastModuleLen = 0;

    delete[] info.LastBuild;
    info.LastBuild = NULL;
    info.LastBuildLen = 0;

    delete[] info.LastFunction;
    info.LastFunction = NULL;
    info.LastFunctionLen = 0;

    delete[] info.Data;
    info.Data = NULL;
    info.DataLen = 0;
}

//--------------------------------------------------------------------
// Output trace information.
//--------------------------------------------------------------------
void RomaImplError::trace(RomaTraceStream& s) const
{
    switch(m_status) 
    {
    case Roma::STATUS_NONE:
        s << TRACE_CLASS(RomaImplError, << "STATUS_NONE");
        break;

    case Roma::STATUS_BOTH:
        s << TRACE_CLASS(RomaError, << "STATUS_SECONDARY: "
          << TRACE(m_info2.LastCompCode)
          << TRACE(m_info2.LastReturnCode)
          << TRACE(m_info2.LastOpType)
          << TRACE(m_info2.LastReason)
          << TRACE(m_info2.LastPosition)
          << TRACE(m_info2.LastErrorType)
          << TRACE(m_info2.LastModule)
          << TRACE(m_info2.LastModuleLen)
          << TRACE(m_info2.LastBuild)
          << TRACE(m_info2.LastBuildLen)
          << TRACE(m_info2.LastFunction)
          << TRACE(m_info2.LastFunctionLen)
          << TRACE(m_info2.LastLineNum)
          << TRACE(m_info2.Data)
          << TRACE(m_info2.DataLen));

    case Roma::STATUS_PRIMARY:
        s << TRACE_CLASS(RomaImplError, << "STATUS_PRIMARY: "
          << TRACE(m_info1.LastCompCode)
          << TRACE(m_info1.LastReturnCode)
          << TRACE(m_info1.LastOpType)
          << TRACE(m_info1.LastReason)
          << TRACE(m_info1.LastPosition)
          << TRACE(m_info1.LastErrorType)
          << TRACE(m_info1.LastModule)
          << TRACE(m_info1.LastModuleLen)
          << TRACE(m_info1.LastBuild)
          << TRACE(m_info1.LastBuildLen)
          << TRACE(m_info1.LastFunction)
          << TRACE(m_info1.LastFunctionLen)
          << TRACE(m_info1.LastLineNum)
          << TRACE(m_info1.Data)
          << TRACE(m_info1.DataLen));
        break;

    default:
        s << TRACE_CLASS(RomaError, << "UNKNOWN; ");
    }
}

