//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcoierr.hpp,v 1.11 2003/12/11 18:30:06 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcoierr.hpp,v $
// $Revision: 1.11 $
//--------------------------------------------------------------------
// The internal implementation of the RomaError public interface.
//--------------------------------------------------------------------

#ifndef __KXCOIERR_HPP__
#define __KXCOIERR_HPP__

#include "kxcotrb.hpp"
#include "kxcoigdt.hpp"


//--------------------------------------------------------------------
// The RomaImplError class.
// This is the internal implementation of the RomaError public
// interface.
//--------------------------------------------------------------------
class KXCODLL RomaImplError : public RomaTraceable,
                              public RomaImplShareable
{
public:
    //----------------------------------------------------------------
    // Default constructor.
    //----------------------------------------------------------------
    RomaImplError();
    RomaImplError(const KXCERROR&);

    //----------------------------------------------------------------
    // Copy and assignment.
    //----------------------------------------------------------------
    RomaImplError(const RomaImplError&);
    RomaImplError& operator = (const RomaImplError&);

    //----------------------------------------------------------------
    // Indicate whether error information is set.
    //----------------------------------------------------------------
    bool anyError() const { return (m_status != KXCERRSTAT_NONE); }

    //----------------------------------------------------------------
    // Get/set the error status.
    //----------------------------------------------------------------
    Roma::ErrorStatus getStatus() const;
    void setStatus(Roma::ErrorStatus s) { m_status = s; }

    //----------------------------------------------------------------
    // Get/set the error type.
    //----------------------------------------------------------------
    Roma::ErrorType getType(Roma::ErrorSubclass) const;
    void setType(Roma::ErrorSubclass, Roma::ErrorType);

    //----------------------------------------------------------------
    // Get/set the completion code.
    //----------------------------------------------------------------
    Roma::CompCode getCompCode(Roma::ErrorSubclass) const;
    void setCompCode(Roma::ErrorSubclass, Roma::CompCode);

    //----------------------------------------------------------------
    // Get/set the return code.
    //----------------------------------------------------------------
    int getReturnCode(Roma::ErrorSubclass) const;
    void setReturnCode(Roma::ErrorSubclass, int);

    //----------------------------------------------------------------
    // Get/set the error reason.
    //----------------------------------------------------------------
    int getReason(Roma::ErrorSubclass) const;
    void setReason(Roma::ErrorSubclass, int);

    //----------------------------------------------------------------
    // Get/set the operation type.
    //----------------------------------------------------------------
    int getOperationType(Roma::ErrorSubclass) const;
    void setOperationType(Roma::ErrorSubclass, Roma::OperationType);

    //----------------------------------------------------------------
    // Get/set the error position.
    //----------------------------------------------------------------
    int getPosition(Roma::ErrorSubclass) const;
    void setPosition(Roma::ErrorSubclass, int);

    void setLineNum(Roma::ErrorSubclass, KXCULONG);

    //----------------------------------------------------------------
    // Get/set additional error information.
    //----------------------------------------------------------------
    const char* getAdditionalInfo(Roma::ErrorSubclass);
    void setAdditionalInfo(Roma::ErrorSubclass, const char*, 
        const char*, const char*, const char* = NULL);

    //----------------------------------------------------------------
    // Format error data.
    //----------------------------------------------------------------
    void formatInfo(const KXCSUBERRINFO&, char*&, unsigned&);

    //----------------------------------------------------------------
    // Get error text.
    //----------------------------------------------------------------
    const char* getText(Roma::ErrorSubclass, Roma::ErrorFormat);

    //----------------------------------------------------------------
    // Fill the C API structures with error data.
    //----------------------------------------------------------------
    void populate(Roma::ErrorSubclass, KXCSUBERROR&);
    void populate(KXCERROR&);

    //----------------------------------------------------------------
    // Gather the last error details.
    //----------------------------------------------------------------
    void capture(Roma::ErrorSubclass, const KXCSUBERRINFO&);
    void capture();

    //----------------------------------------------------------------
    // Output trace information.
    //----------------------------------------------------------------
    void trace(RomaTraceStream&) const;


private:
    //----------------------------------------------------------------
    // Destructor.
    //----------------------------------------------------------------
    ~RomaImplError();

    //----------------------------------------------------------------
    // Free dynamically allocated memory.
    //----------------------------------------------------------------
    void clearInfo(KXCSUBERRINFO&);

    //----------------------------------------------------------------
    // Error status.
    //----------------------------------------------------------------
    KXCULONG m_status;

    //----------------------------------------------------------------
    // Primary and secondary error data.
    //----------------------------------------------------------------
    KXCSUBERRINFO m_info1;
    KXCSUBERRINFO m_info2;

    //----------------------------------------------------------------
    // Buffers and sizes of primary and secondary additional
    // information.
    //----------------------------------------------------------------
    char* m_extraBuf1;
    char* m_extraBuf2;
    unsigned m_extraSize1;
    unsigned m_extraSize2;

    //----------------------------------------------------------------
    // Buffer and size of formatted error text.
    //----------------------------------------------------------------
    char* m_buffer;
    KXCULONG m_size;

    friend class KxcoStreamer;
};


#endif // __KXCOIERR_HPP__

