//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcomida.cpp,v 1.11 2003/12/11 18:30:07 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcomida.cpp,v $
// $Revision: 1.11 $
//--------------------------------------------------------------------
// The RomaMsgId public interface.
//--------------------------------------------------------------------

#include <kxcointa.hpp>
#include <kxcoigdt.hpp>
#include <kxcomid.hpp>


//--------------------------------------------------------------------
// Default constructor.
//--------------------------------------------------------------------
RomaMsgId::RomaMsgId()
{
    m_impl = new RomaImplMsgId;
}

RomaMsgId::RomaMsgId(RomaImplMsgId* impl) : m_impl(impl)
{
}

//--------------------------------------------------------------------
// Copy constructor.
//--------------------------------------------------------------------
RomaMsgId::RomaMsgId(const RomaMsgId& msgId) : m_impl(msgId.m_impl)
{
    m_impl->lockInstance();
}

//--------------------------------------------------------------------
// Constructor.
// Initializes the object with the specified ID data.
//--------------------------------------------------------------------
RomaMsgId::RomaMsgId(const void* data, int length)
{
    m_impl = new RomaImplMsgId;
    m_impl->set(data, length);
}

//--------------------------------------------------------------------
// Destructor.
//--------------------------------------------------------------------
RomaMsgId::~RomaMsgId()
{
    m_impl->releaseInstance();
}

//--------------------------------------------------------------------
// Assignment operator.
//--------------------------------------------------------------------
RomaMsgId& RomaMsgId::operator = (const RomaMsgId& msgId)
{
    KXCO_ASSIGN_SHARED(msgId);
}

//--------------------------------------------------------------------
// Set ID data.
//--------------------------------------------------------------------
void RomaMsgId::set(const void* data, int length)
{
    m_impl->set(data, length);
}

//--------------------------------------------------------------------
// Comparison operator.
//--------------------------------------------------------------------
bool RomaMsgId::operator == (const RomaMsgId& msgId) const
{
    return (*m_impl == *msgId.m_impl);
}

//--------------------------------------------------------------------
// Comparison operator.
//--------------------------------------------------------------------
bool RomaMsgId::operator != (const RomaMsgId& msgId) const
{
    return (*m_impl != *msgId.m_impl);
}

//--------------------------------------------------------------------
// Operator returning ID data.
//--------------------------------------------------------------------
const char* RomaMsgId::operator *() const
{
    return (const char*)(*m_impl);
}

//--------------------------------------------------------------------
// Operator returning ID data.
//--------------------------------------------------------------------
RomaMsgId::operator const void*() const
{
    return (const char*)(*m_impl);
}

//--------------------------------------------------------------------
// Output trace information.
//--------------------------------------------------------------------
void RomaMsgId::trace(RomaTraceStream& s) const
{
    s << TRACE_CLASS(RomaMsgId, << TRACE(m_impl)) << "->";
    m_impl->trace(s);
}

