//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcosuba.cpp,v 1.7 2003/02/21 22:48:09 emueh Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcosuba.cpp,v $
// $Revision: 1.7 $
//--------------------------------------------------------------------
// The RomaSubject public interface.
//--------------------------------------------------------------------

#include "kxcointa.hpp"
#include "kxcoimsg.hpp"
#include "kxcosub.hpp"

TRACE_MODULE_VERSION($Revision: 1.7 $);

//--------------------------------------------------------------------
// Default constructor.
//--------------------------------------------------------------------
RomaSubject::RomaSubject()
{
    TRACE_ENTRY(RomaSubject::RomaSubject, << TRACE(this));

    m_impl = new RomaImplSubject;

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Constructor.
// Initializes the subject with a name and a style.
//--------------------------------------------------------------------
RomaSubject::RomaSubject(const char* subject, Roma::Style style)
{
    TRACE_ENTRY(RomaSubject::RomaSubject, << TRACE(this));

    m_impl = new RomaImplSubject;
    m_impl->setName(subject);
    m_impl->setStyle(style);

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Destructor.
//--------------------------------------------------------------------
RomaSubject::~RomaSubject()
{
    TRACE_ENTRY(RomaSubject::~RomaSubject, << TRACE(this));

    delete m_impl;

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Set a subject style.
//--------------------------------------------------------------------
void RomaSubject::setSubjectStyle(Roma::Style style)
{
    m_impl->setStyle(style);
}

//--------------------------------------------------------------------
// Set a subject name.
//--------------------------------------------------------------------
void RomaSubject::setSubject(const char* subject)
{
    m_impl->setName(subject);
}

//--------------------------------------------------------------------
// Copy constructor.
//--------------------------------------------------------------------
RomaSubject::RomaSubject(const RomaSubject& other)
{
    TRACE_ENTRY(RomaSubject::RomaSubject, << TRACE(this));

    m_impl = new RomaImplSubject(*other.m_impl);

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Assignment operator.
//--------------------------------------------------------------------
RomaSubject& RomaSubject::operator = (const RomaSubject& other)
{
    *m_impl = *other.m_impl;
    return *this;
}

//--------------------------------------------------------------------
// Output trace information.
//--------------------------------------------------------------------
void RomaSubject::trace(RomaTraceStream& s) const
{
    s << TRACE_CLASS(RomaSubject, << TRACE(m_impl)) << "->";
    m_impl->trace(s);
}

