//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// Module       : kxcotrca.cpp
//
// Implements   : RomaTraceStream
//                RomaTraceRadix
//
// Description  : Implementation module for the RomaTraceStream and 
//                RomaTraceRadix classes.
//
//--------------------------------------------------------------------
// $Id: kxcotrca.cpp,v 1.5 2003/02/21 22:48:09 emueh Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcotrca.cpp,v $
// $Revision: 1.5 $
//--------------------------------------------------------------------

#include <kxcointa.hpp>

// The length of the buffer large enough to hold
// the result of the sprintf("%ld") and sprintf("%p")
const int KXCO_DIG_BUFFER_LEN = 64;

const RomaTraceRadix RomaTraceRadix::hex(16);
const RomaTraceRadix RomaTraceRadix::dec(10);

KXCO_ASSERT_PROHIBITED_OPS(RomaTraceStream)

RomaTraceStream::RomaTraceStream()
{
  m_pos = 0;
  m_radix = 10;
}

//--------------------------------------------------------------------
// Write character, put '+' if buffer overflow
//--------------------------------------------------------------------
void RomaTraceStream::write(char ch)
{
  if(m_pos == MAX_LEN)
    m_buff[m_pos++] = '+';
  else if(m_pos < MAX_LEN)
    m_buff[m_pos++] = ch;
}

void RomaTraceStream::write(int v)
{
  write((long)v);
}

#if defined(__alpha)
void RomaTraceStream::write(unsigned long v)
{
  char buff[KXCO_DIG_BUFFER_LEN];
  if (m_radix == 10)
    sprintf(buff, "%lu", (long)v);
  else
    sprintf(buff, "%lx", (long)v);
  write(buff);
}
#else
void RomaTraceStream::write(unsigned int v)
{
  write((unsigned long)v);
}
#endif

void RomaTraceStream::write(long v)
{
  char buff[KXCO_DIG_BUFFER_LEN];
  if(m_radix == 10)
    sprintf(buff, "%ld", v);
  else
    sprintf(buff, "%lx", v);
  write(buff);
}

void RomaTraceStream::write(KXCULONG v)
{
  char buff[KXCO_DIG_BUFFER_LEN];
  if(m_radix == 10)
    sprintf(buff, "%lu", v);
  else
    sprintf(buff, "%lx", v);
  write(buff);
}

//--------------------------------------------------------------------
// Write string, put '+' if buffer overflow
//--------------------------------------------------------------------
void RomaTraceStream::write(const char* v)
{
  //----------------------------------------------------------------
  // 0 pointer
  //----------------------------------------------------------------
  if(v == 0)
  {
    write("(null)");
  }
  //----------------------------------------------------------------
  // good pointer
  //----------------------------------------------------------------
  else
  {
    //------------------------------------------------------------
    // buffer has 0 or more free bytes
    //------------------------------------------------------------
    if(m_pos <= MAX_LEN)
    {
      unsigned strLen = strlen(v);
      unsigned copyLen = min(strLen, ((unsigned)MAX_LEN)-m_pos);
      //--------------------------------------------------------
      // Write string
      //--------------------------------------------------------
      if(copyLen > 0)
      {
        memcpy(m_buff + m_pos, v, copyLen);
        m_pos += copyLen;
      }
      //--------------------------------------------------------
      // Mark overflow
      //--------------------------------------------------------
      if(copyLen < strLen)
        m_buff[m_pos++] = '+';
    }
  }
}

void RomaTraceStream::write(const void* v)
{
  char buff[KXCO_DIG_BUFFER_LEN];
  // prints the pointer value
  sprintf(buff, "%p", v);
  write(buff);
}
