//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1999-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// Module       : kxcoutil.cpp
//
// Implements   : interface Roma
//
// Description  : Utilities (including linked list)
//
//--------------------------------------------------------------------
// $Id: kxcoutil.cpp,v 1.5 2003/02/21 22:48:09 emueh Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcoutil.cpp,v $
// $Revision: 1.5 $
//--------------------------------------------------------------------

#include "kxcoutil.hpp"
#include <stddef.h>

#include <string.h>
#include <stdarg.h>
#include <errno.h>


//--------------------------------------------------------------------
// Classmember : kLinkedList::kLinkedList
// Description : Create an empty linked list
//--------------------------------------------------------------------
kLinkedList::kLinkedList()
{
  m_pHead = 0;
  m_len = 0;
}

//--------------------------------------------------------------------
// Classmember : kLinkedList::kLinkedList(kLinkedList &l)
// Description : Create a complete separate copy of an existing
//               linked list
//--------------------------------------------------------------------
kLinkedList::kLinkedList(const kLinkedList &otherList)
{
  m_pHead = 0;
  m_len = otherList.Length();
  if ( m_len > 0 && otherList.m_pHead != NULL )
  {
    kxcLListEnt * eInl = otherList.m_pHead;
    m_pHead = new kxcLListEnt(*eInl);
    kxcLListEnt ** ppoeNxt = &(m_pHead->m_pNext);
    while ( (eInl = eInl->m_pNext) != NULL )
    {
      *ppoeNxt = new kxcLListEnt(*eInl);
      ppoeNxt = &((*ppoeNxt)->m_pNext);
    }
  }
}

//--------------------------------------------------------------------
// Classmember : kLinkedList::~kLinkedList()
// Description : Destruct a linked list, just delete the entry
//               objects not the user data
//--------------------------------------------------------------------
kLinkedList::~kLinkedList()
{
  Clear();
}


//--------------------------------------------------------------------
// Classmember : kLinkedList::kLinkedList(kLinkedList &l)
// Description : Clear the current list and add a complete separate
//               copy of all the enties of an exiting linked list
//--------------------------------------------------------------------
const kLinkedList& kLinkedList::operator = (const kLinkedList &otherList)
{
  Clear();

  m_pHead = 0;
  m_len = otherList.m_len;
  if ( m_len > 0 && otherList.m_pHead != NULL )
  {
    kxcLListEnt * eInl = otherList.m_pHead;
    m_pHead = new kxcLListEnt(*eInl);
    kxcLListEnt ** ppoeNxt = &(m_pHead->m_pNext);
    while ( (eInl = eInl->m_pNext) != NULL )
    {
      *ppoeNxt = new kxcLListEnt(*eInl);
      ppoeNxt = &((*ppoeNxt)->m_pNext);
    }
  }
  return *this;
}


//--------------------------------------------------------------------
// Classmember : kLinkedList::Length
// Description : get the m_number of entries in the list
//--------------------------------------------------------------------
int kLinkedList::Length() const
{
  return m_len;
}


//--------------------------------------------------------------------
// Classmember : kLinkedList::Append
// Input       : LGENERICTYPE *pUserData
//                  - Pointer to user data object being added
// Description : Adding elements to ends of the list
//--------------------------------------------------------------------
void kLinkedList::Append(LGENERICTYPE *pUserData)
{
  kxcLListEnt **ppInsPoint = &m_pHead;
  while ( *ppInsPoint != NULL)
  {
    ppInsPoint = &(*ppInsPoint)->m_pNext;
  }
  *ppInsPoint = new kxcLListEnt(pUserData, 0);
  m_len++;
}

//--------------------------------------------------------------------
// Classmember : kLinkedList::Prepend
// Input       : LGENERICTYPE *pUserData
//                  - Pointer to user data object being added
// Description : Adding elements to ends of the list
//--------------------------------------------------------------------
void kLinkedList::Prepend(LGENERICTYPE *pUserData)
{
  m_pHead = new kxcLListEnt(pUserData, m_pHead);
  m_len++;
}

//--------------------------------------------------------------------
// Classmember : kLinkedList::ExtractFirst
// Description : Removing elements at ends of the list
// Return      : User Data pointer from removed entry or Null
// Note        : If null is valid for user data then cannot tell
//               if an entry was removed
//--------------------------------------------------------------------
LGENERICTYPE * kLinkedList::ExtractFirst(void)
{
  LGENERICTYPE * pReturn = NULL;

  if (m_len > 0 && m_pHead != NULL)
  {
    //----------------------------------------------------------------
    // Store the object to be removed
    //----------------------------------------------------------------
    kxcLListEnt * pGoing = m_pHead;
    pReturn = pGoing->m_pEntryData;

    //----------------------------------------------------------------
    // Unlink first entry
    //----------------------------------------------------------------
    m_pHead = m_pHead->m_pNext;
    delete pGoing;
    m_len--;
  }
  return pReturn;
}

//--------------------------------------------------------------------
// Classmember : kLinkedList::ExtractLast
// Description : Removing elements at ends of the list
// Return      : User Data pointer from removed entry or Null
// Note        : If null is valid for user data then cannot tell
//               if an entry was removed
//--------------------------------------------------------------------
LGENERICTYPE * kLinkedList::ExtractLast(void)
{
  LGENERICTYPE * pReturn = NULL;

  if ( m_len > 0 && m_pHead != NULL)
  {
    //----------------------------------------------------------------
    // The entry we will remove, start from the head of the list
    //----------------------------------------------------------------
    kxcLListEnt * pGoing = m_pHead;

    //----------------------------------------------------------------
    // the point at the previous entry which links to the removed
    // entry and must thus be cleared
    //----------------------------------------------------------------
    kxcLListEnt ** ppGoingLink = &m_pHead;

    //----------------------------------------------------------------
    // While the current entry has a next link follow it
    //----------------------------------------------------------------
    while ( pGoing->m_pNext != NULL)
    {
      //--------------------------------------------------------------
      // Update the current entry and the link pointer to that entry
      //--------------------------------------------------------------
      ppGoingLink = &(pGoing->m_pNext);
      pGoing = pGoing->m_pNext;
    }
    pReturn = pGoing->m_pEntryData;
    delete pGoing;
    *ppGoingLink = NULL;
    m_len--;
  }
  return pReturn;
}

//--------------------------------------------------------------------
// Classmember : bool kLinkedList::ExtractFirst
//
// Description : Removing elements at ends of the list
// Return      : Bool removed flag and User Data pointer from removed
//               entry
//--------------------------------------------------------------------
bool kLinkedList::ExtractFirst(LGENERICTYPE ** ppRetUserData)
{
  bool bReturn = false;

  if (m_len > 0)
  {
    *ppRetUserData = ExtractFirst();
    bReturn = true;
  }
  return bReturn;
}

//--------------------------------------------------------------------
// Classmember : bool kLinkedList::ExtractLast
// Description : Removing elements at ends of the list
// Return      : Bool removed flag and User Data pointer from removed
//               entry
//--------------------------------------------------------------------
bool kLinkedList::ExtractLast(LGENERICTYPE ** ppRetUserData)
{
  bool bReturn = false;

  if (m_len > 0)
  {
    *ppRetUserData = ExtractLast();
    bReturn = true;
  }
  return bReturn;
}

//--------------------------------------------------------------------
// Classmember : kLinkedList::GetAtIndex
// Description : Access N th element of the list
// Note        : Caller can not distinguish out of bound and
//               NULL entry
//--------------------------------------------------------------------
LGENERICTYPE * kLinkedList::GetAtIndex(int atIndex) const
{
  LGENERICTYPE * pReturn;

  if ( atIndex < 0 )
  {
    pReturn = NULL;
  }
  else if ( m_len <= atIndex )
  {
    pReturn = NULL;
  }
  else if ( m_pHead != NULL )
  {
    kxcLListEnt *pEntry = m_pHead;
    for (int index = atIndex; index > 0; index--)
    {
      pEntry = pEntry->m_pNext;
    }
    pReturn = pEntry->m_pEntryData;
  }
  else
  {
    pReturn = NULL;
  }
  return pReturn;
}

//--------------------------------------------------------------------
// Classmember : kLinkedList::GetAtIndex
// Description : Access N th element of the list
// Note        : Caller can not distinguish out of bound and
//               NULL entry
//--------------------------------------------------------------------
bool kLinkedList::GetAtIndex(int            atIndex,
                          LGENERICTYPE **ppRetUserData) const
{
  bool bReturn;

  if ( atIndex < 0 )
  {
    bReturn = false;
  }
  else if ( m_len <= atIndex )
  {
    bReturn = false;
  }
  else
  {
    bReturn = true;
    *ppRetUserData = GetAtIndex(atIndex);
  }
  return bReturn;
}

//--------------------------------------------------------------------
// Classmember : kLinkedList::Clear
// Description : Destruct all list elements
// Note        : User is responsible for destruction of user
//               data objects
//--------------------------------------------------------------------
void kLinkedList::Clear(void)
{
  kxcLListEnt *pCurrent,*pNext;

  if (m_len == 0)
    return;

  pCurrent = pNext = m_pHead;

  //------------------------------------------------------------------
  // While there is a current
  //------------------------------------------------------------------
  while (pCurrent != NULL)
  {
    //----------------------------------------------------------------
    // Move onto next then delete current
  //------------------------------------------------------------------
    pNext = pCurrent->m_pNext;
    delete pCurrent;
    pCurrent = pNext;
  }
  m_pHead = 0;
  m_len = 0;
}


//--------------------------------------------------------------------
// Classmember : kxcLListIterator::kxcLListIterator
//
// Description : Provide an iterator for a list
//--------------------------------------------------------------------
kxcLListIterator::kxcLListIterator(const kLinkedList &list)
{
  //------------------------------------------------------------------
  // Init for first iteration
  //------------------------------------------------------------------
  m_pListEntIterator = m_pListHead = list.m_pHead;
}

//--------------------------------------------------------------------
// Classmember : kxcLListIterator::GetNext
// Description : Return the UserData from here and move the iterator
//               on one
// Note        : If an entry has a null m_pEntryData then user
//               will assume they have hit end of list
//--------------------------------------------------------------------
LGENERICTYPE * kxcLListIterator::GetNext()
{
  LGENERICTYPE * pReturn = NULL;

  if (m_pListEntIterator != NULL)
  {
    //----------------------------------------------------------------
    // return userData and iterate
    //----------------------------------------------------------------
    pReturn = m_pListEntIterator->m_pEntryData;
    m_pListEntIterator = m_pListEntIterator->m_pNext;
  }
  return pReturn;
}

//--------------------------------------------------------------------
// Classmember : kxcLListIterator::GetNext
// Description : Iterate to next returning false for end of list
//--------------------------------------------------------------------
bool kxcLListIterator::GetNext(LGENERICTYPE ** ppRetUserData)
{
  bool bReturn = false;

  if (m_pListEntIterator != NULL)
  {
    //----------------------------------------------------------------
    // return userData and iterate
    //----------------------------------------------------------------
    *ppRetUserData = GetNext();
    bReturn = true;
  }
  return bReturn;
}

//--------------------------------------------------------------------
// Classmember : kxcLListIterator::Reset
// Description : Reset iterator to head of list
//--------------------------------------------------------------------
void kxcLListIterator::Reset()
{
  m_pListEntIterator = m_pListHead;
}


//--------------------------------------------------------------------
// Classmember : kxcSpliceLListIterator::kxcSpliceLListIterator
// Description : Provide an iterator with splicing for a list
//--------------------------------------------------------------------
kxcSpliceLListIterator::kxcSpliceLListIterator(kLinkedList &llist)
{
    m_pList = &llist;
    m_ppSpliceBeforeLink = m_ppSpliceAfterLink = &(llist.m_pHead);
    m_pEntIterator = llist.m_pHead;
}


//--------------------------------------------------------------------
// Classmember : kxcSpliceLListIterator::GetNext
// Description : Return the UserData from here and move the iterator
//               on one
// Note        : If an entry has a null m_pEntryData then user will
//               assume they have hit end of list
//--------------------------------------------------------------------
LGENERICTYPE * kxcSpliceLListIterator::GetNext()
{
  LGENERICTYPE * pReturn;

  if (m_pEntIterator == NULL)
  {
    pReturn = NULL;
  }
  else
  {
    //----------------------------------------------------------------
    // Return this entries data and move pionters on one entry
    //----------------------------------------------------------------
    pReturn = m_pEntIterator->m_pEntryData;
    m_ppSpliceBeforeLink = m_ppSpliceAfterLink;
    m_ppSpliceAfterLink = & (m_pEntIterator->m_pNext);
    m_pEntIterator = m_pEntIterator->m_pNext;
  }
  return pReturn;
}

//--------------------------------------------------------------------
// Classmember : bool kxcSpliceLListIterator::GetNext
// Description : Iterate to next returning false for end of list
//--------------------------------------------------------------------
bool kxcSpliceLListIterator::GetNext(LGENERICTYPE ** ppRetUserData)
{
  bool bReturn = false;

  if (m_pEntIterator != NULL)
  {
    //----------------------------------------------------------------
    // return userData and iterate
    //----------------------------------------------------------------
    *ppRetUserData = GetNext();
    bReturn = true;
  }
  return bReturn;
}

//--------------------------------------------------------------------
// Classmember : kxcSpliceLListIterator::Reset
// Description : Reset iterator to head of list
//--------------------------------------------------------------------
void kxcSpliceLListIterator::Reset()
{
    m_ppSpliceBeforeLink = m_ppSpliceAfterLink = &(m_pList->m_pHead);
    m_pEntIterator = m_pList->m_pHead;
}



//--------------------------------------------------------------------
// Classmember : kxcSpliceLListIterator::InsertAfter
// Description : Splice in a new entry after current iterated entry
// Note        : Multiple uses of this call without iterating
//               inbetween inserts after current but before previous
//               insert
//--------------------------------------------------------------------
void kxcSpliceLListIterator::InsertAfter(LGENERICTYPE *pUserData)
{
  //------------------------------------------------------------------
  // Create a new entry, and pass the after point as its "next"
  //------------------------------------------------------------------
  kxcLListEnt *newEnt = new kxcLListEnt(pUserData,
                                        *m_ppSpliceAfterLink);

  //------------------------------------------------------------------
  // Plumb the new entry into the list
  //------------------------------------------------------------------
  *m_ppSpliceAfterLink = newEnt;
  m_pList->m_len += 1;
}

//--------------------------------------------------------------------
// Classmember : kxcSpliceLListIterator::InsertBefore
// Description : Splice in a new entry before current iterated entry
// Note        : Multiple calls to this function without iterating
//               inbetween inserts before the current and previously
//               inserted
//--------------------------------------------------------------------
void kxcSpliceLListIterator::InsertBefore(LGENERICTYPE *pUserData)
{
  //------------------------------------------------------------------
  // Create a new entry, and pass the before point as its "next"
  //------------------------------------------------------------------
  kxcLListEnt *newEnt = new kxcLListEnt(pUserData,
                                        *m_ppSpliceBeforeLink);

  //------------------------------------------------------------------
  // Plumb the new entry into the list
  //------------------------------------------------------------------
  *m_ppSpliceBeforeLink = newEnt;
  m_pList->m_len += 1;
}


//--------------------------------------------------------------------
// Classmember : kxcSpliceLListIterator::RemoveCurrent
// Description : Splice list to remove current iterated entry
// Note        : InsertAfter and Insert before have the same
//               effect if they directly follow this member
//--------------------------------------------------------------------
LGENERICTYPE * kxcSpliceLListIterator::RemoveCurrent(void)
{
  LGENERICTYPE * pReturn;

  if (m_pList->Length() > 0)
  {
    //----------------------------------------------------------------
    // Store current entry to be removed
    //----------------------------------------------------------------
    kxcLListEnt * goingEnt = *m_ppSpliceBeforeLink;

    //----------------------------------------------------------------
    // Return this entries user data as the user has responsibility
    // to delete this
    //----------------------------------------------------------------
    pReturn = (*m_ppSpliceBeforeLink)->m_pEntryData;

    //----------------------------------------------------------------
    // before next becomes currents next, unlinking the current
    // Remeber the iterator is always already moved on
    //----------------------------------------------------------------
    *m_ppSpliceBeforeLink = m_pEntIterator;

    //----------------------------------------------------------------
    // Make After safe as we are about to delete this entry
    //----------------------------------------------------------------
    *m_ppSpliceAfterLink = *m_ppSpliceBeforeLink;

    //----------------------------------------------------------------
    // Before should backup one - but we are only singly linked so
    // cannot do that
    // Delete the unlinked entry
    //----------------------------------------------------------------
    delete goingEnt;
    m_pList->m_len -= 1;
  }
  else
  {
    //----------------------------------------------------------------
    // No user data to be deleted by user
    //----------------------------------------------------------------
    pReturn = NULL;
  }
  return pReturn;
}


//--------------------------------------------------------------------
// STRING
//--------------------------------------------------------------------

//
// Class : kString
// Member : kString ()
//
// Description : Initialise as empty string
//

kString::kString ()
{
  m_pStr = NULL;
  m_Len = 0;
}

kString::kString (const char *_pInitStr)
{
int NewLen;

  if (_pInitStr == NULL)
  {
    NewLen = 0;
  }
  else
  {
    NewLen = strlen (_pInitStr);
  }

  if (NewLen > 0)
  {
    // An actual string - make space for it and copy it
    m_pStr = new char [NewLen+1];
    strcpy (m_pStr, _pInitStr);
    m_Len = NewLen;
  }
  else
  {
    m_pStr = NULL;
    m_Len = 0;
  }
}

kString::kString(const kString &_Str)
{
  m_Len = _Str.m_Len;
  if (m_Len > 0)
  {
    m_pStr = new char [m_Len+1];
    strcpy (m_pStr, _Str.m_pStr);
  }
  else
  {
    m_pStr = NULL;
  }
}

kString::kString (const char *_pInitStr, int _ExtractLen)
{
  if (_ExtractLen > 0 && _pInitStr != NULL)
  {
    // Only copy 'NewLen' characters but terminate it properly with an extra '\0'
    // If the supplied length is larger than the string then it will still work
    // but not transfer the expected number of characters
    m_pStr = new char [_ExtractLen+1];
    // Copy specified number of characters
    // - end prematurely if _ExtractLen > strlen (_pInitStr))
    strncpy (m_pStr, _pInitStr, _ExtractLen);
    m_pStr [_ExtractLen] = '\0';
    m_Len = _ExtractLen;
  }
  else
  {
    m_pStr = NULL;
    m_Len = 0;
  }
}

kString::kString (char _InitCh, int _SetLen)
{
  m_Len = _SetLen;
  m_pStr = new char [m_Len+1];
  memset (m_pStr, _InitCh, m_Len);
  // Ensure NULL terminated
  m_pStr [m_Len] = '\0';
}

kString::~kString ()
{
  // Must delete string data
  Empty ();
}

void kString::Empty ()
{
  if (m_pStr != NULL) delete m_pStr;
  m_pStr = NULL;
  m_Len = 0;
}

kString::operator const char* () const
{
  // Return the empty string if string is empty rather than NULL
  // This allows someone to happily use a kString in a trace report or a printf knowing
  // that an empty string rather than a NULL will be returned.
  if (m_pStr == NULL) return "";
  return (const char *) m_pStr;
}

const kString& kString::operator = (const char *_pReplStr)
{
 int Len;

  if (_pReplStr == NULL || *_pReplStr == '\0')
  {
    if (m_pStr != NULL) delete m_pStr;
    m_Len = 0;
  }
  else
  {
    Len = strlen (_pReplStr);

    if (Len == m_Len)
    {
      strcpy (m_pStr, _pReplStr);
    }
    else
    {
      // Need to create a different sized string
      if (m_pStr != NULL) delete m_pStr;

      m_pStr = new char [Len+1];
      strcpy (m_pStr, _pReplStr);
      m_Len = Len;
    }
  }
  return *this;
}

const kString& kString::operator = (const kString &_pStr1)
{
  if (m_Len == _pStr1.m_Len)
  {
    if (m_Len > 0)
    {
      strcpy (m_pStr, _pStr1);
    }
  }
  else
  {
    // Need to create a different sized string
    if (m_pStr != NULL) delete m_pStr;

    if (_pStr1.m_Len > 0)
    {
      m_pStr = new char [_pStr1.m_Len+1];
      strcpy (m_pStr, _pStr1.m_pStr);
      m_Len = _pStr1.m_Len;
    }
    else
    {
      m_pStr = NULL;
      m_Len = 0;
    }
  }

  return *this;
}

void kString::Concatenate (const char * _pStr1, int _Len1, const char * _pStr2, int _Len2)
{
int NewLen;

  // Remove current contents (assumes not concaternating to self !)
  Empty ();
  NewLen = _Len1 + _Len2;

  if (NewLen > 0)
  {
    m_pStr = new char [NewLen+1];
    m_pStr [0] = '\0';
    if (_Len1 > 0)
    {
      strcpy (m_pStr, _pStr1);
    }
    if (_Len2 > 0)
    {
      strcat (m_pStr, _pStr2);
    }
  }
  m_Len = NewLen;
}


bool kString::operator == (const char * _pStr) const
{
  if (m_pStr == NULL)
  {
    // If both empty then they are the same
    if (m_Len == 0)
    {
      if ( _pStr == NULL  ||
           (_pStr != NULL && *_pStr == '\0') )
      {
        return true;
      }
    }
  }
  else
  {
    return strcmp (m_pStr, _pStr) == 0;
  }
  return false;
}


bool kString::operator != (const char * _pStr) const
{
  // Reuse the == case !
  return !(*this == _pStr);
}

const kString& kString::operator+=(const char * _pStr)
{
char * pCombined;
int Len2;
int NewLen;

  if (_pStr != NULL)
  {
    Len2 = strlen (_pStr);
    if (Len2 > 0)
    {
      // Just add extra string into enlarged string area
      NewLen = m_Len + Len2;

      pCombined = new char [NewLen+1];
      pCombined [0] = '\0';
      if (m_Len > 0)
      {
        strcpy (pCombined, m_pStr);
      }

      strcat (pCombined, _pStr);

      // Must delay deletion in case _pStr is actually this
      if (m_Len > 0)
      {
        delete m_pStr;
      }

      // Set new combined contents
      m_pStr = pCombined;
      m_Len = NewLen;
    }
  }
  return *this;
}

kString operator + (const char *_pStr1, const kString &_pStr2)
{
kString TempStr;

  TempStr.Concatenate (_pStr1, strlen (_pStr1), _pStr2, _pStr2.GetLength ());
  return TempStr;
}

kString operator + (const kString &_pStr1, const kString &_pStr2)
{
kString TempStr;

  TempStr.Concatenate (_pStr1, _pStr1.GetLength (), _pStr2, _pStr2.GetLength ());
  return TempStr;
}

kString operator + (const kString &_pStr1, const char * _pStr2)
{
kString TempStr;

  TempStr.Concatenate (_pStr1, _pStr1.GetLength (), _pStr2, strlen (_pStr2));
  return TempStr;
}

void kString::Printf (const char *_pFmt, ...)
{
// Assumes this is big enough
char TempBuff [1025];
va_list Mark;

  va_start (Mark, _pFmt);

#ifdef KXC_WINNT
  // Use special call, truncate string if buffer not large enough
  if (_vsnprintf (TempBuff, sizeof (TempBuff)-1, _pFmt, Mark) == -1)
  {
    TempBuff [sizeof (TempBuff)-1] = '\0';
  }
#else
  // There is no size limited vsprintf - just trust to luck that buffer is large enough
  vsprintf (TempBuff,  _pFmt, Mark);
#endif

  va_end (Mark);

  // Assign contents
  kString::operator = (TempBuff);
}
