//--------------------------------------------------------------------
//
// Copyright Candle Corporation 2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: RomaSystemConfig.java,v 1.2 2003/11/18 15:18:37 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/java/com/candle/roma/kxc/RomaSystemConfig.java,v $
//--------------------------------------------------------------------
//
// Module       : RomaSystemConfig.java
//
// Defines      : class RomaSystemConfig
//
//--------------------------------------------------------------------

package com.candle.roma.kxc;


/**
* This class defines the system environment configuration of the application.
*
* @see RomaExceptionInfo
*/
public final class RomaSystemConfig
{
  /**
  * Returns the architecture type of the environment on which the reporting
  * application is running.
  * 
  * @return one of <br><ul>
  * <li><code>Roma.ENVTYPE_INTEL</code></li>
  * <li><code>Roma.ENVTYPE_UNIX</code></li>
  * <li><code>Roma.ENVTYPE_MVS</code></li>
  * <li><code>Roma.ENVTYPE_OS400</code></li>
  * <li><code>Roma.ENVTYPE_VMS</code></li>
  * </ul>
  */
  public int getEnvType()
  {
     return (m_etype & 0xFF000000);
  }

  /**
  * Returns the OS type of the environment on which the reporting
  * application is running.
  * 
  * @return one of <br><ul>
  * <li><code>Roma.ENVOS_AIX</code></li>
  * <li><code>Roma.ENVOS_HPUX102</code></li>
  * <li><code>Roma.ENVOS_HPUX11</code></li>
  * <li><code>Roma.ENVOS_MVSBATCH</code></li>
  * <li><code>Roma.ENVOS_MVSCICS</code></li>
  * <li><code>Roma.ENVOS_SUN</code></li>
  * <li><code>Roma.ENVOS_WIN95</code></li>
  * <li><code>Roma.ENVOS_WINNT</code></li>
  * </ul>
  */
  public int getEnvOS()
  {
     return m_etype;
  }

  /**
  * Returns the domain name. This is a Windows NT or AIX domain name
  * or OS/390 system ID.
  */
  public String getDomainName()
  {
     return m_domain;
  }

  /**
  * Returns the name of the application reporting the error.
  */
  public String getProgramName()
  {
     return m_prog;
  }

  /**
  * Returns the user ID under which the reporting application is
  * executing.
  */
  public String getUserId()
  {
     return m_uid;
  }

  /**
  * Returns the thread ID assigned to the reporting threads active thread.
  */
  public long getThreadId()
  {
     return m_tid;
  }

  /**
  * Returns the process ID assigned to the reporting application.
  */
  public long getProcessId()
  {
     return m_pid;
  }


  RomaSystemConfig()
  {
  }

  RomaSystemConfig(int etype, String domain, String uid, String prog,
                   long pid, long tid)
  {
    m_pid = pid;
    m_tid = tid;
    m_etype = etype;
    if (domain != null) m_domain = domain;
    if (uid != null) m_uid = uid;
    if (prog != null) m_prog = prog;
  }

  private long m_pid;
  private long m_tid;
  private int m_etype;
  private String m_uid = "";
  private String m_prog = "";
  private String m_domain = "";
}

