//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1999-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// Module       : RomaTheme.java
//
// Defines      : class RomaTheme
//
//--------------------------------------------------------------------

package com.candle.roma.kxc;

import com.candle.roma.kxc.raw.*;

/**
* This class encapsulates a Roma theme.
* <p>Themes are a Roma concept that provides a mechanism to associate
* requests and responses.  Roma allocates themes when the client
* application sends a message to a Business Element.
* <p>The client can use the theme in further messages sent to the
* Business Element (or indeed to a different Business Element) and
* also use the theme to tie requests with responses.
*
* @see RomaBusinessElement
*/
public final class RomaTheme
{
  private static final String copyrightNotice =
    "Copyright Candle Corporation 1999,2000,2001";
  
  /**
  * Constant representing no theme allocated.
  */
  static final long NULL_THEME = Kxc.NULL_HANDLE;
  
  /**
  * Initializes a null theme.
  */
  public RomaTheme()
  {
    setNull();
  }
  
  /**
  * Initialises a theme using an existing theme handle.
  * Package visibility.
  * 
  * @param handle  theme handle previously created in the Pipe.
  */
  RomaTheme( long handle )
  {
    m_handle = handle;
  }
  
  /**
  * Updates this object so that it represents an identical
  * theme to the input RomaTheme object.
  *
  * @param theme  another RomaTheme object.
  */
  public void set( RomaTheme theme )
  {
    if( theme != null )
      m_handle = theme.m_handle;
  }
  
  /**
  * Returns true if this object represent a null theme.
  */
  public boolean isNull()
  {
    return m_handle == NULL_THEME;
  }
  
  /**
  * Updates object to represent a null theme.
  */
  public void setNull()
  {
    m_handle = NULL_THEME;
  }
  
  /**
  * Returns the theme handle as an integer.
  * Package visibility.
  */
  long handle()
  {
    return m_handle;
  }
  
  /**
  * Object equality test.
  * Returns true if passed object is of RomaTheme type and
  * represents an identical theme to this object. Otherwise
  * returns false.
  *
  * @param input any object.
  */
  public boolean equals( Object input )
  {
    if ( input instanceof RomaTheme )
      return m_handle == ((RomaTheme)input).m_handle;

    return false;
  }
  
  private long m_handle = NULL_THEME;
}
