//--------------------------------------------------------------------
//
// Copyright Candle Corporation 2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: RomaTrailer.java,v 1.2 2003/11/18 15:18:37 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/java/com/candle/roma/kxc/RomaTrailer.java,v $
//--------------------------------------------------------------------
//
// Module       : RomaTrailer.java
//
// Defines      : class RomaTrailer
//
//--------------------------------------------------------------------

package com.candle.roma.kxc;


/**
* This class defines the persistent trailer information.
*
* @see RomaThemeInfo
* @see RomaILSData
*/
public class RomaTrailer
{
   /**
   * Constructs the empty object.
   */
   public RomaTrailer()
   {
        m_data = new byte[0];
   }

   /**
   * Constructs the object from the specified data.
   *
   * @param type ILS type
   * @param number ILS number
   * @param data Application-defined data
   */
   public RomaTrailer(int type, int number, byte[] data)
   {
      m_type = type;
      m_method = number;
      setData(data);
   }

   /**
   * Returns the type of the ILS that has added this persistent trailer.
   * @return one of
   * <br><ul>
   * <li><code>Roma.ILS_TYPE_XPORT</code> - transport (MQSeries, MSMQ, TIB/Rendezvous) inline service</li>
   * <li><code>Roma.ILS_TYPE_USER</code> - user inline service</li>
   * <li><code>Roma.ILS_TYPE_ROUTER</code> - router inline service</li>
   * <li><code>Roma.ILS_TYPE_SYSMGMT</code> - application management inline service</li>
   * <li><code>Roma.ILS_TYPE_ERROR</code> - error analysis inline service</li>
   * <li><code>Roma.ILS_TYPE_WEB</code> - XML conversion inline service</li>
   * <li><code>Roma.ILS_TYPE_XPTDATA</code> - transport data structure</li>
   * </ul>
   */
   public int getType()
   {
       return m_type;
   }

   /**
   * Sets the ILS type of the trailer.
   *
   * @param type ILS type, one of
   * <br><ul>
   * <li><code>Roma.ILS_TYPE_XPORT</code> - transport (MQSeries, MSMQ, TIB/Rendezvous) inline service</li>
   * <li><code>Roma.ILS_TYPE_USER</code> - user inline service</li>
   * <li><code>Roma.ILS_TYPE_ROUTER</code> - router inline service</li>
   * <li><code>Roma.ILS_TYPE_SYSMGMT</code> - application management inline service</li>
   * <li><code>Roma.ILS_TYPE_ERROR</code> - error analysis inline service</li>
   * <li><code>Roma.ILS_TYPE_WEB</code> - XML conversion inline service</li>
   * <li><code>Roma.ILS_TYPE_XPTDATA</code> - transport data structure</li>
   * </ul>
   */
   public void setType(int type)
   {
       m_type = type;
   }

   /**
   * Returns the number of the ILS that has added this persistent trailer.
   */
   public int getMethod()
   {
       return m_method;
   }

   /**
   * Sets the ILS number of the trailer.
   *
   * @param md ILS number
   */
   public void setMethod(int md)
   {
       m_method = md;
   }

   /**
   * Returns the application-defined data of the trailer.
   */
   public byte[] getData()
   {
       return m_data;
   }

   /**
   * Sets the trailer data.
   *
   * @param data Trailer data
   */
   public void setData(byte[] data)
   {
       m_data = (data == null ? new byte[0] : data);
   }

   /**
   * Returns a copy of the object.
   */
   public Object clone()
   {
        RomaTrailer copy = null;

        try
        {
            copy = (RomaTrailer)super.clone();
            copy.m_data = new byte[m_data.length];

            if (m_data.length != 0)
            {
                System.arraycopy(m_data, 0, copy.m_data, 0, m_data.length);
            }
        }
        catch (Exception e)
        {
        }

        return copy;
   }


   private int m_type;
   private int m_method;
   private byte[] m_data;
}

