/*********************************************************************
 * $Id: krrerr.hpp,v 1.4 2003/12/11 17:41:37 e_ab Exp $    
 * $Source: /romacvs/root/roma/src/krr/krrutil/krrerr.hpp,v $
 *********************************************************************
 * Description: KRR Abstract Error Interface
 *********************************************************************
 *                                                                    
 * Copyright 1999-2002, Candle Corporation, a California                   
 * corporation.  All rights reserved.                                 
 * International copyright secured.                                   
 *                                                                    
 * This material is proprietary to Candle Corporation and is not to   
 * be reproduced, used, or disclosed except in accordance with        
 * program licenses or upon written authorization of Candle           
 * Corporation.                                                       
 *                                                                    
 ********************************************************************/
#ifndef __KRRERR_HPP
#define __KRRERR_HPP

#include <krrenv.h>
#include <krrerror.h>
#include <string.h>

class KrrErrorImpl;

// Incapsulates error information.
class KRRUTILAPI KrrError
{
public:

    enum ErrorType
    {
        ET_API          = 8,
        ET_TRANSIENT    = 4,
        ET_ERROR        = 2,
        ET_CRITICAL     = 1
    };

    KrrError();

    KrrError( KRRUINT32 returnCode,
        KRRUINT32 position,
        KRRUINT32 failedLayer,
        KRRUINT32 reason,
        ErrorType errorType,
        const char* location,
        const char* errorMessage,
        const KrrError* secondary = 0 );

    virtual ~KrrError();

    KrrError( const KrrError& );
    KrrError& operator = ( const KrrError& );

    KRRUINT32 getReturnCode() const;
    void setReturnCode( KRRUINT32 );

    KRRUINT32 getPosition() const;
    void setPosition( KRRUINT32 );

    KRRUINT32 getFailedLayer() const;
    void setFailedLayer( KRRUINT32 );

    KRRUINT32 getReason() const;
    void setReason( KRRUINT32 );

    ErrorType getErrorType() const;
    void setErrorType( ErrorType );

    virtual const char* getLocation() const;
    void setLocation( const char* );

    virtual const char* getErrorMessage() const;
    void setErrorMessage( const char* );

    const KrrError* getSecondary() const;
    void setSecondary( const KrrError* );

    virtual KrrError* clone() const;

    virtual void raise() const;

    void logRaise() const;

protected:

    void setMods( bool b = true );
    bool hasMods() const;

private:

    const KrrErrorImpl& getForView() const;
    KrrErrorImpl& getForEdit();

    KrrErrorImpl* m_impl;

};

#endif
