//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcocrs.hpp,v 1.12 2003/12/11 18:30:04 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcocrs.hpp,v $
// $Revision: 1.12 $
//--------------------------------------------------------------------
// The RomaClientResponse public interface.
//--------------------------------------------------------------------

#ifndef __KXCOCRS_HPP
#define __KXCOCRS_HPP

#include <kxcotb.hpp>
#include <kxcotrb.hpp>
#include <kxcothm.hpp>
#include <kxcots.hpp>
#include <kxcoils.hpp>
#include <kxcomid.hpp>

//--------------------------------------------------------------------
// The internal implementation class.
//--------------------------------------------------------------------
class RomaImplMsgIncoming;


//--------------------------------------------------------------------
// The RomaClientResponse class.
//
// This class encapsulates responses returned to Roma client
// applications from a Roma Business Service.
//
// It also provides a framework for marshalling the response into an
// application defined RomaClientResponse subclass.
//
// This is a base for the custom client responses.
// The RomaClient::get() returns objects of this and derived classes.
//--------------------------------------------------------------------
class KXCODLL RomaClientResponse : public RomaTypeBase, 
                                   public RomaTraceable,
                                   public RomaAllocator
{
public:
    //----------------------------------------------------------------
    // Destructor.
    //----------------------------------------------------------------
    virtual ~RomaClientResponse();

    //-----------------------------------------------------------------
    // Copy and assignment.
    //
    // Note: these methods don't create independent copy of the
    // message, but just share common implementation instance.
    //-----------------------------------------------------------------
    RomaClientResponse(const RomaClientResponse&);
    RomaClientResponse& operator = (const RomaClientResponse&);

    //-----------------------------------------------------------------
    // Get the message theme.
    //-----------------------------------------------------------------
    const RomaTheme& theme() const;

    //-----------------------------------------------------------------
    // Get the Message ID.
    //-----------------------------------------------------------------
    const RomaMsgId& msgId() const;

    //-----------------------------------------------------------------
    // Get the Correlation ID.
    //-----------------------------------------------------------------
    const RomaMsgId& correlId() const;

    //-----------------------------------------------------------------
    // Get the message subject name.
    //-----------------------------------------------------------------
    const char* subject() const;

    //-----------------------------------------------------------------
    // Get the message priority.
    //  A numeric value or one of the following constants:
    //  PRIORITY_PROFILE - use priority specified in API profile, or
    //  PRIORITY_DEFAULT - default transport priority
    //-----------------------------------------------------------------
    int priority() const;

    //-----------------------------------------------------------------
    // Get the message type.
    //
    // It is one of the following:
    //   DATAGRAM -  datagram received by the business element
    //   REQUEST -   request received by the business element
    //   RESPONSE -  response received by the client
    //   NOTIFICATION - notification response received by the client
    //-----------------------------------------------------------------
    Roma::MsgType msgType() const;

    //-----------------------------------------------------------------
    // Get the message persistence. Values can be ...
    //  PERSISTENCE_YES     - persistent
    //  PERSISTENCE_NO      - not persistent
    //  PERSISTENCE_PROFILE - use value in the API profile, default NO
    //-----------------------------------------------------------------
    Roma::Persistence persistence() const;

    //-----------------------------------------------------------------
    // Get the message format number.
    //-----------------------------------------------------------------
    int format() const;

    //-----------------------------------------------------------------
    // Get the message feedback code.
    //-----------------------------------------------------------------
    int feedback() const;

    //-----------------------------------------------------------------
    // Get the message expiry interval.
    //-----------------------------------------------------------------
    int expiry() const;

    //-----------------------------------------------------------------
    // Get the message codepage.
    //-----------------------------------------------------------------
    int codepage() const;

    //-----------------------------------------------------------------
    // Get the message encoding.
    //-----------------------------------------------------------------
    int encoding() const;

    //-----------------------------------------------------------------
    // Get the message status.
    //-----------------------------------------------------------------
    KXCULONG status() const;

    //-----------------------------------------------------------------
    // Get the message backout count.
    //-----------------------------------------------------------------
    int backoutCount() const;

    //-----------------------------------------------------------------
    // Get the message format name.
    //-----------------------------------------------------------------
    const char* formatName() const;

    //-----------------------------------------------------------------
    // Get the message name, as known to the Application Management
    // inline service.
    //-----------------------------------------------------------------
    const char* msgName() const;

    //-----------------------------------------------------------------
    // Get the message timestamp.
    //-----------------------------------------------------------------
    RomaTimestamp timestamp() const;

    //-----------------------------------------------------------------
    // Get the message data buffer.
    //-----------------------------------------------------------------
    void* msgBuffer() const;
    int msgDataLength() const;

    //-----------------------------------------------------------------
    // Get the tag data buffer.
    //-----------------------------------------------------------------
    void* tagBuffer() const;
    int tagDataLength() const;

    //----------------------------------------------------------------
    // Get any inline service data related to this response.
    //----------------------------------------------------------------
    RomaILSData* getILSData() const;


    //-----------------------------------------------------------------
    // IMPLEMENTATION METHODS.
    //-----------------------------------------------------------------
    ROMA_DECLARE_TYPEID();
    virtual void trace(RomaTraceStream&) const;


protected:
    //----------------------------------------------------------------
    // Constructors for internal use only.
    //----------------------------------------------------------------
    RomaClientResponse();
    RomaClientResponse(RomaImplMsgIncoming*);

    //-----------------------------------------------------------------
    // Pointer to the internal implementation.
    //-----------------------------------------------------------------
    RomaImplMsgIncoming* m_impl;

    friend class RomaImplProxy;
};


#endif // __KXCOCRS_HPP

