//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// Module       : kxcoroma.hpp
//
// Description  : Common enum types
//
//                RomaMessage is just a namespace for the constants
//                used in the C++ language binding.
//
//                Constants need to be prefixed by Roma:: in use.
//
//--------------------------------------------------------------------
// $Id: kxcoroma.hpp,v 1.18 2004/01/22 17:53:26 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcoroma.hpp,v $
// $Revision: 1.18 $
//--------------------------------------------------------------------

#ifndef __KXCOROMA_HPP
#define __KXCOROMA_HPP

#include <kxc.h>


//--------------------------------------------------------------------
// Call type constants (used in enum CallType)
//--------------------------------------------------------------------
#define KXCOCALLTYPE_BECONNECT          (KXCCALLTYPE_OMBASE+0)
#define KXCOCALLTYPE_BEGET              (KXCCALLTYPE_OMBASE+1)
#define KXCOCALLTYPE_BEPUT              (KXCCALLTYPE_OMBASE+2)
#define KXCOCALLTYPE_CLIENTCONNECT      (KXCCALLTYPE_OMBASE+3)
#define KXCOCALLTYPE_CLIENTGET          (KXCCALLTYPE_OMBASE+4)
#define KXCOCALLTYPE_CLIENTPUT          (KXCCALLTYPE_OMBASE+5)
#define KXCOCALLTYPE_CLOSETHEME         (KXCCALLTYPE_OMBASE+6)
#define KXCOCALLTYPE_COMMITLOCAL        (KXCCALLTYPE_OMBASE+7)
#define KXCOCALLTYPE_DELETESTORAGEQUEUE (KXCCALLTYPE_OMBASE+8)
#define KXCOCALLTYPE_DISCONNECT         (KXCCALLTYPE_OMBASE+9)
#define KXCOCALLTYPE_INTERPRETEXCEPTION (KXCCALLTYPE_OMBASE+10)
#define KXCOCALLTYPE_PUTEXCEPTION       (KXCCALLTYPE_OMBASE+11)
#define KXCOCALLTYPE_REPORTERROR        (KXCCALLTYPE_OMBASE+12)
#define KXCOCALLTYPE_ROLLBACKLOCAL      (KXCCALLTYPE_OMBASE+13)
#define KXCOCALLTYPE_SENDALERT          (KXCCALLTYPE_OMBASE+14)
#define KXCOCALLTYPE_SQCLOSE            (KXCCALLTYPE_OMBASE+15)
#define KXCOCALLTYPE_SQOPEN             (KXCCALLTYPE_OMBASE+16)
#define KXCOCALLTYPE_SQRETRIEVE         (KXCCALLTYPE_OMBASE+17)
#define KXCOCALLTYPE_SQSTORE            (KXCCALLTYPE_OMBASE+18)
#define KXCOCALLTYPE_PUBLISH            (KXCCALLTYPE_OMBASE+19)
#define KXCOCALLTYPE_SUBSCRIBE          (KXCCALLTYPE_OMBASE+20)
#define KXCOCALLTYPE_REQUESTUPDATES     (KXCCALLTYPE_OMBASE+21)
#define KXCOCALLTYPE_DELETEPUBLICATION  (KXCCALLTYPE_OMBASE+22)
#define KXCOCALLTYPE_UNSUBSCRIBE        (KXCCALLTYPE_OMBASE+23)
#define KXCOCALLTYPE_UNSUBSCRIBEALL     (KXCCALLTYPE_OMBASE+24)
#define KXCOCALLTYPE_DIRLOOKUP          (KXCCALLTYPE_OMBASE+25)
#define KXCOCALLTYPE_DIRGETITEM         (KXCCALLTYPE_OMBASE+26)
#define KXCOCALLTYPE_DIRDELETEITEM      (KXCCALLTYPE_OMBASE+27)
#define KXCOCALLTYPE_DIRSTOREITEM       (KXCCALLTYPE_OMBASE+28)
#define KXCOCALLTYPE_REPLACEWORKFLOW    (KXCCALLTYPE_OMBASE+29)
#define KXCOCALLTYPE_INSERTWORKFLOW     (KXCCALLTYPE_OMBASE+30)
#define KXCOCALLTYPE_SETBUSFLOW         (KXCCALLTYPE_OMBASE+31)
#define KXCOCALLTYPE_COPYTHEME          (KXCCALLTYPE_OMBASE+32)
#define KXCOCALLTYPE_MERGEETEINFO       (KXCCALLTYPE_OMBASE+33)


//--------------------------------------------------------------------
// Base for the KXC Object Model return codes
//--------------------------------------------------------------------
#define KXCORC_ERROR_BASE        10000

// Error return code:
// Overriden RomaClientRuest::marshalRequest() returned false
#define KXCORC_MARSHAL_REQUEST   (KXCORC_ERROR_BASE+0)

// Error return code:
// Overriden RomaBeResponse::marshalResponse() returned false
#define KXCORC_MARSHAL_RESPONSE  (KXCORC_ERROR_BASE+1)

// Error return code:
// Reserved
#define KXCORC_MARSHAL_STORABLE  (KXCORC_ERROR_BASE+2)

// Error return code:
// RomaBusinessElement::createRequest() or overriden returned 0
// request or false return code
#define KXCORC_CREATE_REQUEST    (KXCORC_ERROR_BASE+3)

// Error return code:
// RomaClient::createResponse() or overriden returned 0 response
// or false return code
#define KXCORC_CREATE_RESPONSE   (KXCORC_ERROR_BASE+4)

// Error return code:
// Reserved
#define KXCORC_CREATE_STORABLE   (KXCORC_ERROR_BASE+5)

// Error return code:
// Tag or message buffer of the RomaClientRequest, RomaBeResponse
// or RomaStorable is in fail state
#define KXCORC_BUFFER_ERROR      (KXCORC_ERROR_BASE+6)

// Error return code:
// Client or BusinessElement is not connected
#define KXCORC_NOT_CONNECTED     (KXCORC_ERROR_BASE+7)

// Error return code:
// Client or BusinessElemeny is already connected
#define KXCORC_ALREADY_CONNECTED (KXCORC_ERROR_BASE+8)

// Error return code:
// Reserved
#define KXCORC_NOT_OPEN          (KXCORC_ERROR_BASE+9)

// Error return code:
// Reserved
#define KXCORC_ALREADY_OPEN      (KXCORC_ERROR_BASE+10)



//---------------------------------------------------------------------
// Class: Roma
//---------------------------------------------------------------------
class KXCODLL Roma
{
public:
    // ---------------------------------------------------------------
    // MESSAGE
    // ---------------------------------------------------------------
    enum MsgType
    {
        DATAGRAM        = KXCMSGTYPE_DATAGRAM,
        REQUEST         = KXCMSGTYPE_REQUEST,
        RESPONSE        = KXCMSGTYPE_REPLY,
        NOTIFICATION    = KXCMSGTYPE_REPLY | KXCMSGTYPE_NOTIFY,

        PUBLICATION     = KXCMSGTYPE_PUBLICATION,
        POSITIVE_PUBSUB = KXCMSGTYPE_POSITIVE_PUBSUB,
        NEGATIVE_PUBSUB = KXCMSGTYPE_NEGATIVE_PUBSUB,
        ADVISORY        = KXCMSGTYPE_ADVISORY,

        POSITIVE_COA    = KXCMSGTYPE_POSITIVE_COA,
        NEGATIVE_COA    = KXCMSGTYPE_NEGATIVE_COA,
        POSITIVE_COD    = KXCMSGTYPE_POSITIVE_COD,
        NEGATIVE_COD    = KXCMSGTYPE_NEGATIVE_COD
    };

    enum Priority
    {
        PRIORITY_DEFAULT = KXCMSGPRI_DEFAULT,
        PRIORITY_PROFILE = KXCMSGPRI_USEDEFAULT
    };

    enum Persistence
    {
        PERSISTENCE_YES     = KXCMSGPERSIST_YES,
        PERSISTENCE_NO      = KXCMSGPERSIST_NO,
        PERSISTENCE_PROFILE = KXCMSGPERSIST_USEDEFAULT
    };

    enum Format
    {
        FORMAT_BINARY          = KXCMSGFORMAT_BINARY,
        FORMAT_STRING          = KXCMSGFORMAT_STRING,
        FORMAT_BINARY_SIGN     = KXCMSGFORMAT_BINARY_SIGN,
        FORMAT_STRING_SIGN     = KXCMSGFORMAT_STRING_SIGN,
        FORMAT_BINARY_ENC      = KXCMSGFORMAT_BINARY_ENC,
        FORMAT_STRING_ENC      = KXCMSGFORMAT_STRING_ENC,
        FORMAT_BINARY_SIGN_ENC = KXCMSGFORMAT_BINARY_SIGN_ENC,
        FORMAT_STRING_SIGN_ENC = KXCMSGFORMAT_STRING_SIGN_ENC,
        FORMAT_USER            = KXCMSGFORMAT_USER,
        FORMAT_USENAME         = KXCMSGFORMAT_USENAME,
        FORMAT_PROFILE         = KXCMSGFORMAT_USEDEFAULT
    };


    // these settings are advisory only
    enum Feedback
    {
        FEEDBACK_NONE      = KXCMSGFBCK_NONE,
        FEEDBACK_ACK       = KXCMSGFBCK_ACK,
        FEEDBACK_NACK      = KXCMSGFBCK_NACK,
        FEEBBACK_BAD_DEST  = KXCMSGFBCK_BAD_DEST,
        FEEDBACK_Q_FULL    = KXCMSGFBCK_Q_FULL,
        FEEDBACK_EXPIRED   = KXCMSGFBCK_EXPIRED,
        FEEDBACK_ALLOC_ERR = KXCMSGFBCK_ALLOC_ERR,
        FEEDBACK_PROFILE   = KXCMSGFBCK_USEDEFAULT
    };


    enum Expiry
    {
        EXPIRY_NEVER   = KXCMSGEXP_NEVER,
        EXPIRY_PROFILE = KXCMSGEXP_USEDEFAULT
    };


    enum Codepage
    {
        CODEPAGE_USER    = KXCMSGCODEPAGE_USER,
        CODEPAGE_PROFILE = KXCMSGCODEPAGE_USEDEFAULT
    };


    enum Encoding
    {
        ENCODING_DEFAULT  = KXCMSGENC_DEFAULT,
        ENCODING_NORMAL   = KXCMSGENC_NORMAL,
        ENCODING_REVERSED = KXCMSGENC_REVERSED,
        ENCODING_PROFILE  = KXCMSGENC_USEDEFAULT
    };


    enum Status
    {
        STATUS_NO            = KXCMSGSTAT_ANONYMOUS,
        STATUS_AUTHENTICATED = KXCMSGSTAT_AUTHENTICATED,
        STATUS_NATIVE        = KXCMSGSTAT_NATIVE,
        STATUS_OK_TO_RESPOND = KXCMSGSTAT_OK_TO_RESPOND,
        STATUS_OK_TO_NOTIFY  = KXCMSGSTAT_OK_TO_NOTIFY
    };


    enum Backout
    {
        BACKOUT_UNSUPPORTED = KXCMSGBACKOUT_UNSUPPORTED
    };


    enum MaxLength
    {
        MAX_MSGLEN = KXCMSGMAX_LEN,
        MAX_TAGLEN = KXCMSGMAX_TAGLEN
    };


    enum InOut
    {
        INCOMING, 
        OUTGOING
    };


    // ---------------------------------------------------------------
    // EXCEPTIONCONFIG
    // ---------------------------------------------------------------
    enum AppType
    {
        SERVER        = KXCTYPE_SERVER,
        CLIENT        = KXCTYPE_CLIENT,
        BRIDGE        = KXCTYPE_BRIDGE,
        NATIVE_CLIENT = KXCTYPE_NATIVECLI,
        NATIVE_SERVER = KXCTYPE_NATIVESRV,
        UNKNOWN       = KXCTYPE_UNKNOWN
    };


    // ---------------------------------------------------------------
    // ILSDATA
    // ---------------------------------------------------------------
    enum ILSType
    {
        ILS_TYPE_KXC     = KXCXIT_TYPE_KXC,
        ILS_TYPE_XPORT   = KXCXIT_TYPE_XPORT,
        ILS_TYPE_USER    = KXCXIT_TYPE_SERVICE,
        ILS_TYPE_ROUTER  = KXCXIT_TYPE_ROUTER,
        ILS_TYPE_SYSMGMT = KXCXIT_TYPE_SYSMGMT,
        ILS_TYPE_ERROR   = KXCXIT_TYPE_ERROR,
        ILS_TYPE_XPTDATA = KXCXIT_TYPE_XPTDATA,
        ILS_TYPE_XML     = KXCXIT_TYPE_WEB
    };


    // ---------------------------------------------------------------
    // Transports - target for ILS data
    // ---------------------------------------------------------------
    enum XportType
    {
        XPORT_KXC      = KXCXIT_XPT_METHOD_KXC,
        XPORT_MQSERIES = KXCXIT_XPT_METHOD_MQSERIES,
        XPORT_MSMQ     = KXCXIT_XPT_METHOD_MSMQ,
        XPORT_BMQ      = KXCXIT_XPT_METHOD_BMQ,
        XPORT_MQTIB    = KXCXIT_XPT_METHOD_TIB,
        XPORT_VENDOR   = KXCXIT_XPT_METHOD_VENDOR,
        XPORT_USER     = KXCXIT_XPT_METHOD_USER
    };


    // ---------------------------------------------------------------
    // CLIENT and BUSINESS ELEMENT
    // ---------------------------------------------------------------
    enum AppOptions
    {
        APP_NONE              = KXCAPPOPT_NONE,
        APP_EXCLUSIVE         = KXCAPPOPT_EXCLUSIVE,
        APP_MQSERIES_XACTION  = KXCAPPOPT_MQSERIES_XACTION,
        APP_NON_XACTIONAL     = KXCAPPOPT_NON_XACTIONAL,
        APP_MQSERIES_FASTPATH = KXCAPPOPT_MQSERIES_FASTPATH,
        APP_ROLLBK_OS_XACT    = KXCAPPOPT_ROLLBK_OS_XACT
    };


    enum OpenOptions
    {
        OPEN_NON_XACTIONAL   = KXCOOPT_NON_XACTIONAL,
        OPEN_EXCLUSIVE       = KXCOOPT_EXCLUSIVE,
        OPEN_DELETE_ON_CLOSE = KXCOOPT_DELETE_ON_CLOSE,
        OPEN_MUST_EXIST      = KXCOOPT_MUST_EXIST,
        OPEN_OUTPUT          = KXCOOPT_OUTPUT,
        OPEN_INPUT           = KXCOOPT_INPUT,
        OPEN_PEEK            = KXCOOPT_PEEK,
        OPEN_DEFAULT         = KXCOOPT_NONE
    };


    // ---------------------------------------------------------------
    // PUT / STORE OPTIONS
    // ---------------------------------------------------------------
    enum PutSelector
    {
        USE_NONE               = KXCPOPT_NONE,
        USE_MSGID              = KXCPOPT_USEMSGID,
        USE_CORRELID           = KXCPOPT_USECORRELID,
        USE_MSGID_AND_CORRELID = USE_MSGID | USE_CORRELID
    };


    // ---------------------------------------------------------------
    // GET / RETRIEVE OPTIONS
    // ---------------------------------------------------------------
    enum ThemeControl
    {
        THEME_RETAIN          = KXCGOPT_RETAINTHEME,
        THEME_CLOSE           = KXCGOPT_CLOSETHEME,
        THEME_DEFAULT_CONTROL = KXCGOPT_NONE
    };


    enum GetSelector
    {
        BY_NONE               = KXCGOPT_NONE,
        BY_MSGID              = KXCGOPT_USEMSGID,
        BY_CORRELID           = KXCGOPT_USECORRELID,
        BY_THEME              = KXCGOPT_USETHEME,
        BY_MSGID_AND_THEME    = BY_MSGID | KXCGOPT_USETHEME,
        BY_MSGID_AND_CORRELID = BY_MSGID | BY_CORRELID,
        SELECTOR_MASK         = KXCGOPT_USETHEME | BY_MSGID | BY_CORRELID
    };


    enum Operation
    {
        GET_FIRST      = KXCGOPT_NONE,
        PEEK_FIRST     = KXCGOPT_PEEK_FIRST,
        PEEK_NEXT      = KXCGOPT_PEEK_NEXT,
        PEEK_SAME      = KXCGOPT_PEEK_SAME,
        GET_PEEKED     = KXCGOPT_GET_PEEKED,
        OPERATION_MASK = GET_FIRST|PEEK_FIRST|PEEK_NEXT|PEEK_SAME|GET_PEEKED
    };


    // ---------------------------------------------------------------
    // DIRECTORY ACCESS
    // ---------------------------------------------------------------
    enum SearchType
    {
        EXACTMATCH = KXCDOPT_SEARCH_EXACT,
        ONELEVEL   = KXCDOPT_SEARCH_ONELEVEL,
        SUBTREE    = KXCDOPT_SEARCH_SUBTREE
    };


    enum SearchOption
    {
        USE_NO_BASE   = KXCDOPT_NONE,
        USE_ROMA_BASE = KXCDOPT_USE_ROMA_BASE
    };


    enum StoreOption
    {
        MUST_EXIST     = KXCUPDOPT_MUSTEXIST,
        MUST_NOT_EXIST = KXCUPDOPT_MUSTNOTEXIST,
        FORCE          = KXCUPDOPT_NONE
    };


    // ---------------------------------------------------------------
    // ERROR HANDLING
    // ---------------------------------------------------------------
    enum CallType
    {
        CALLTYPE_APPLICATION             = KXCCALLTYPE_APPLICATION,
        CALLTYPE_PUTREQUEST              = KXCCALLTYPE_PUTREQUEST,
        CALLTYPE_GETREQUEST              = KXCCALLTYPE_GETREQUEST,
        CALLTYPE_PUTRESPONSE             = KXCCALLTYPE_PUTRESPONSE,
        CALLTYPE_GETRESPONSE             = KXCCALLTYPE_GETRESPONSE,
        CALLTYPE_BRIDGEPUT               = KXCCALLTYPE_BRIDGEPUT,
        CALLTYPE_BRIDGEGET               = KXCCALLTYPE_BRIDGEGET,
        CALLTYPE_PUTEXCEPTION            = KXCCALLTYPE_PUTEXCEPTION,
        CALLTYPE_INITIALIZE              = KXCCALLTYPE_INITIALIZE,
        CALLTYPE_TERMINATE               = KXCCALLTYPE_TERMINATE,
        CALLTYPE_COMMIT                  = KXCCALLTYPE_COMMIT,
        CALLTYPE_ROLLBACK                = KXCCALLTYPE_ROLLBACK,
        CALLTYPE_FREEBUFFER              = KXCCALLTYPE_FREEBUFFER,
        CALLTYPE_CLOSEHANDLE             = KXCCALLTYPE_CLOSEHANDLE,
        CALLTYPE_INTERPRETMSG            = KXCCALLTYPE_INTERPRETMSG,
        CALLTYPE_WRITELOG                = KXCCALLTYPE_WRITELOG,
        CALLTYPE_SENDALERT               = KXCCALLTYPE_SENDALERT,
        CALLTYPE_OPENSTORAGEQ            = KXCCALLTYPE_OPENSTORAGEQ,
        CALLTYPE_CLOSESTORAGEQ           = KXCCALLTYPE_CLOSESTORAGEQ,
        CALLTYPE_DELETESTORAGEQ          = KXCCALLTYPE_DELETESTORAGEQ,
        CALLTYPE_PUTTOSTORAGEQ           = KXCCALLTYPE_PUTTOSTORAGEQ,
        CALLTYPE_GETFROMSTORAGEQ         = KXCCALLTYPE_GETFROMSTORAGEQ,
        CALLTYPE_QUERYTHEME              = KXCCALLTYPE_QUERYTHEME,
        CALLTYPE_BUILDTHEME              = KXCCALLTYPE_BUILDTHEME,
        CALLTYPE_PURGEQTOEMS             = KXCCALLTYPE_PURGEQTOEMS,
        CALLTYPE_REPORTERROR             = KXCCALLTYPE_REPORTERROR,
        CALLTYPE_QUERYCONTEXT            = KXCCALLTYPE_QUERYCONTEXT,
        CALLTYPE_SETLDAPSERVICE          = KXCCALLTYPE_SETLDAPSERVICE,
        CALLTYPE_SETILSDATA              = KXCCALLTYPE_SETILSDATA,
        CALLTYPE_GETILSDATA              = KXCCALLTYPE_GETILSDATA,
        CALLTYPE_DIRLOOKUP               = KXCCALLTYPE_DIRLOOKUP,
        CALLTYPE_DIRGETDN                = KXCCALLTYPE_DIRGETDN,
        CALLTYPE_DIRUPDATEITEM           = KXCCALLTYPE_DIRUPDATEITEM,
        CALLTYPE_DIRDELETEITEM           = KXCCALLTYPE_DIRDELETEITEM,
        CALLTYPE_DIRGETMODCOUNT          = KXCCALLTYPE_DIRGETMODCOUNT,
        CALLTYPE_OM_QUERYTHEME           = KXCCALLTYPE_QUERYTHEME,
        CALLTYPE_OM_BUILDTHEME           = KXCCALLTYPE_BUILDTHEME,
        CALLTYPE_OM_INTERPRETMSG         = KXCCALLTYPE_INTERPRETMSG,
        CALLTYPE_OM_BECONNECT            = KXCOCALLTYPE_BECONNECT,
        CALLTYPE_OM_BEGET                = KXCOCALLTYPE_BEGET,
        CALLTYPE_OM_BEPUT                = KXCOCALLTYPE_BEPUT,
        CALLTYPE_OM_CLIENTCONNECT        = KXCOCALLTYPE_CLIENTCONNECT,
        CALLTYPE_OM_CLIENTGET            = KXCOCALLTYPE_CLIENTGET,
        CALLTYPE_OM_CLIENTPUT            = KXCOCALLTYPE_CLIENTPUT,
        CALLTYPE_OM_CLOSETHEME           = KXCOCALLTYPE_CLOSETHEME,
        CALLTYPE_OM_COMMITLOCAL          = KXCOCALLTYPE_COMMITLOCAL,
        CALLTYPE_OM_DELETESTORAGEQUEUE   = KXCOCALLTYPE_DELETESTORAGEQUEUE,
        CALLTYPE_OM_DISCONNECT           = KXCOCALLTYPE_DISCONNECT,
        CALLTYPE_OM_INTERPRETEXCEPTION   = KXCOCALLTYPE_INTERPRETEXCEPTION,
        CALLTYPE_OM_PUTEXCEPTION         = KXCOCALLTYPE_PUTEXCEPTION,
        CALLTYPE_OM_REPORTERROR          = KXCOCALLTYPE_REPORTERROR,
        CALLTYPE_OM_ROLLBACKLOCAL        = KXCOCALLTYPE_ROLLBACKLOCAL,
        CALLTYPE_OM_SENDALERT            = KXCOCALLTYPE_SENDALERT,
        CALLTYPE_OM_SQCLOSE              = KXCOCALLTYPE_SQCLOSE,
        CALLTYPE_OM_SQOPEN               = KXCOCALLTYPE_SQOPEN,
        CALLTYPE_OM_SQRETRIEVE           = KXCOCALLTYPE_SQRETRIEVE,
        CALLTYPE_OM_SQSTORE              = KXCOCALLTYPE_SQSTORE,
        CALLTYPE_OM_PUBLISH              = KXCOCALLTYPE_PUBLISH,
        CALLTYPE_OM_SUBSCRIBE            = KXCOCALLTYPE_SUBSCRIBE,
        CALLTYPE_OM_REQUESTUPDATES       = KXCOCALLTYPE_REQUESTUPDATES,
        CALLTYPE_OM_DELETEPUBLICATION    = KXCOCALLTYPE_DELETEPUBLICATION,
        CALLTYPE_OM_UNSUBSCRIBE          = KXCOCALLTYPE_UNSUBSCRIBE,
        CALLTYPE_OM_UNSUBSCRIBEALL       = KXCOCALLTYPE_UNSUBSCRIBEALL,
        CALLTYPE_OM_DIRLOOKUP            = KXCOCALLTYPE_DIRLOOKUP,
        CALLTYPE_OM_DIRGETITEM           = KXCOCALLTYPE_DIRGETITEM,
        CALLTYPE_OM_DIRDELETEITEM        = KXCOCALLTYPE_DIRDELETEITEM,
        CALLTYPE_OM_DIRSTOREITEM         = KXCOCALLTYPE_DIRSTOREITEM,
        CALLTYPE_OM_REPLACEWORKFLOW      = KXCOCALLTYPE_REPLACEWORKFLOW,
        CALLTYPE_OM_INSERTWORKFLOW       = KXCOCALLTYPE_INSERTWORKFLOW,
        CALLTYPE_OM_SETBUSFLOW           = KXCOCALLTYPE_SETBUSFLOW,
        CALLTYPE_OM_COPYTHEME            = KXCOCALLTYPE_COPYTHEME,
        CALLTYPE_OM_MERGEETEINFO         = KXCOCALLTYPE_MERGEETEINFO
    };


    enum CompCode
    {
        CC_OK      = KXCCC_OK,
        CC_WARNING = KXCCC_WARNING,
        CC_FAILED  = KXCCC_FAILED
    };


    enum OperationType
    {
        OP_KXC_OM   = KXCOP_KXC_OM,
        OP_KXC      = KXCOP_KXC,
        OP_OS       = KXCOP_OS,
        OP_LIB      = KXCOP_LIB,
        OP_CTLIB    = KXCOP_CTLIB,
        OP_MSMQ     = KXCOP_MSMQ,
        OP_MQSERIES = KXCOP_MQSERIES,
        OP_MQSI     = KXCOP_MQSI,
        OP_LDAP     = KXCOP_LDAP,
        OP_LDIF     = KXCOP_LDIF,
        OP_RPC      = KXCOP_RPC,
        OP_CICS     = KXCOP_CICS,
        OP_TIB      = KXCOP_TIB,
        OP_BMQ      = KXCOP_BMQ,
        OP_USER     = KXCOP_USER
    };


    enum ErrorType
    {
        TYPE_CRITICAL  = KXCETYPE_CRITICAL,
        TYPE_TRANSIENT = KXCETYPE_TRANSIENT,
        TYPE_ERROR     = KXCETYPE_ERROR,
        TYPE_API       = KXCETYPE_API
    };


    enum ErrorFormat
    {
        ERROR_FULL    = KXCERROR_FULL,
        ERROR_COMPACT = KXCERROR_COMPACT
    };


    // Constants PRIMARY and SECONDARY have been deprecated
    // from version 3.20, use constants SUBCLASS_PRIMARY or
    // SUBCLASS_SECONDARY instead.
    enum ErrorSubclass
    {
        PRIMARY            = 0,
        SECONDARY          = 1,
        SUBCLASS_PRIMARY   = 0,
        SUBCLASS_SECONDARY = 1
    };


    // Constant STATUS_PRIM has been deprecated from version 3.20,
    // use constant STATUS_PRIMARY instead.
    enum ErrorStatus
    {
        STATUS_NONE    = KXCERRSTAT_NONE,
        STATUS_PRIM    = KXCERRSTAT_MAIN,
        STATUS_PRIMARY = KXCERRSTAT_MAIN,
        STATUS_BOTH    = KXCERRSTAT_SUB | KXCERRSTAT_MAIN
    };


    enum ReturnCode
    {
        RC_OK                          = KXCRC_OK,
        RC_INVALID_CONTEXT             = KXCRC_INVALID_CONTEXT,
        RC_INVALID_OPTIONS             = KXCRC_INVALID_OPTIONS,
        RC_INVALID_ROUTE               = KXCRC_INVALID_ROUTE,
        RC_INVALID_THEME               = KXCRC_INVALID_THEME,
        RC_INVALID_MESSAGE             = KXCRC_INVALID_MESSAGE,
        RC_INVALID_RESPONSE            = KXCRC_INVALID_RESPONSE,
        RC_INVALID_PARAMETER           = KXCRC_INVALID_PARAMETER,
        RC_INVALID_APPDEF              = KXCRC_INVALID_APPDEF,
        RC_INVALID_SERVICE             = KXCRC_INVALID_SERVICE,
        RC_INVALID_HANDLE              = KXCRC_INVALID_HANDLE,
        RC_CONTEXT_MISMATCH            = KXCRC_CONTEXT_MISMATCH,
        RC_NONXACTIONAL                = KXCRC_NONXACTIONAL,
        RC_NON_LOCALQ                  = KXCRC_NON_LOCALQ,
        RC_INVALID_SYNCTYPE            = KXCRC_INVALID_SYNCTYPE,
        RC_INVALID_SECURITY            = KXCRC_INVALID_SECURITY,
        RC_CALL_IN_PROGRESS            = KXCRC_CALL_IN_PROGRESS,
        RC_BUFFER_EXISTS               = KXCRC_BUFFER_EXISTS,
        RC_NO_DATA_AVAILABLE           = KXCRC_NO_DATA_AVAILABLE,
        RC_NO_MSG_AVAILABLE            = KXCRC_NO_MSG_AVAILABLE,
        RC_BUFFER_TOO_SMALL            = KXCRC_BUFFER_TOO_SMALL,
        RC_BUFFER_OVERFLOW             = KXCRC_BUFFER_OVERFLOW,
        RC_NOT_SUPPORTED               = KXCRC_NOT_SUPPORTED,
        RC_VALIDATION_FAILED           = KXCRC_VALIDATION_FAILED,
        RC_INVALID_TRAILER             = KXCRC_INVALID_TRAILER,
        RC_MESSAGE_SUPPRESSED          = KXCRC_MESSAGE_SUPPRESSED,
        RC_BACKED_OUT                  = KXCRC_BACKED_OUT,
        RC_ACCESS_DENIED               = KXCRC_ACCESS_DENIED,
        RC_BUFFER_ERROR                = KXCRC_BUFFER_ERROR,
        RC_Q_DELETED                   = KXCRC_Q_DELETED,
        RC_Q_FULL                      = KXCRC_Q_FULL,
        RC_MSG_UNAVAILABLE             = KXCRC_MSG_UNAVAILABLE,
        RC_SIGNAL_ACCEPTED             = KXCRC_SIGNAL_ACCEPTED,
        RC_SIGNAL_INVALID              = KXCRC_SIGNAL_INVALID,
        RC_SIGNAL_OUTSTANDING          = KXCRC_SIGNAL_OUTSTANDING,
        RC_ROUTING_FAILURE             = KXCRC_ROUTING_FAILURE,
        RC_STORAGE_FAILURE             = KXCRC_STORAGE_FAILURE,
        RC_TRANSPORT_FAILURE           = KXCRC_TRANSPORT_FAILURE,
        RC_EXIT_FAILURE                = KXCRC_EXIT_FAILURE,
        RC_EXCEPTION_CAUGHT            = KXCRC_EXCEPTION_CAUGHT,
        RC_NOT_ACCESSIBLE              = KXCRC_NOT_ACCESSIBLE,
        RC_RESOURCE_PROBLEM            = KXCRC_RESOURCE_PROBLEM,
        RC_TRANSPORT_UNAVAILABLE       = KXCRC_TRANSPORT_UNAVAILABLE,
        RC_MUTEX_FAILURE               = KXCRC_MUTEX_FAILURE,
        RC_INVALID_MSGTYPE             = KXCRC_INVALID_MSGTYPE,
        RC_CACHE_FAILURE               = KXCRC_CACHE_FAILURE,
        RC_HANDLE_BROKEN               = KXCRC_HANDLE_BROKEN,
        RC_SYSCONFIG_FAILURE           = KXCRC_SYSCONFIG_FAILURE,
        RC_TRANSFORM_DISALLOWED        = KXCRC_TRANSFORM_DISALLOWED,
        RC_UNKNOWN_INITPROFILE         = KXCRC_UNKNOWN_INITPROFILE,
        RC_UPDATE_FAILED               = KXCRC_UPDATE_FAILED,
        RC_DELETE_FAILED               = KXCRC_DELETE_FAILED,
        RC_UNKNOWN_SERVICE             = KXCRC_UNKNOWN_SERVICE,
        RC_UNKNOWN_MSGFORMAT           = KXCRC_UNKNOWN_MSGFORMAT,
        RC_UNKNOWN_CLIENT              = KXCRC_UNKNOWN_CLIENT,
        RC_UNKNOWN_SERVER              = KXCRC_UNKNOWN_SERVER,
        RC_UNKNOWN_BRIDGE              = KXCRC_UNKNOWN_BRIDGE,
        RC_INVALID_QNAME               = KXCRC_INVALID_QNAME,
        RC_CONFIG_ERROR                = KXCRC_CONFIG_ERROR,
        RC_DIRECTORY_UNAVAILABLE       = KXCRC_DIRECTORY_UNAVAILABLE,
        RC_BRIDGE_UNAVAILABLE          = KXCRC_BRIDGE_UNAVAILABLE,
        RC_INVALID_COMPONENT           = KXCRC_INVALID_COMPONENT,
        RC_INVALID_ELEMENT             = KXCRC_INVALID_ELEMENT,
        RC_INVALID_SERVERNAME          = KXCRC_INVALID_SERVERNAME,
        RC_INVALID_FILENAME            = KXCRC_INVALID_FILENAME,
        RC_WAIT_CANCELLED              = KXCRC_WAIT_CANCELLED,
        RC_UNKNOWN_ERRPROFILE          = KXCRC_UNKNOWN_ERRPROFILE,
        RC_UNKNOWN_PUTPROFILE          = KXCRC_UNKNOWN_PUTPROFILE,
        RC_UNKNOWN_GETPROFILE          = KXCRC_UNKNOWN_GETPROFILE,
        RC_UNKNOWN_OBJECT              = KXCRC_UNKNOWN_OBJECT,
        RC_GET_ATTRIBUTE_FAILURE       = KXCRC_GET_ATTRIBUTE_FAILURE,
        RC_UNKNOWN_NODE                = KXCRC_UNKNOWN_NODE,
        RC_LDAP_GET_FAILURE            = KXCRC_LDAP_GET_FAILURE,
        RC_LDAP_OPEN_FAILURE           = KXCRC_LDAP_OPEN_FAILURE,
        RC_LDAP_BIND_FAILURE           = KXCRC_LDAP_BIND_FAILURE,
        RC_LDAP_SEARCH_FAILURE         = KXCRC_LDAP_SEARCH_FAILURE,
        RC_LDAP_FIRST_ENTRY_FAILURE    = KXCRC_LDAP_FIRST_ENTRY_FAILURE,
        RC_LDAP_GET_DN_FAILURE         = KXCRC_LDAP_GET_DN_FAILURE,
        RC_LDAP_NEXT_ENTRY_FAILURE     = KXCRC_LDAP_NEXT_ENTRY_FAILURE,
        RC_LDAP_COUNT_ENTRIES_FAILURE  = KXCRC_LDAP_COUNT_ENTRIES_FAILURE,
        RC_LDAP_MODIFY_FAILURE         = KXCRC_LDAP_MODIFY_FAILURE,
        RC_LDAP_DELETE_FAILURE         = KXCRC_LDAP_DELETE_FAILURE,
        RC_LDAP_ADD_FAILURE            = KXCRC_LDAP_ADD_FAILURE,
        RC_LDAP_UNBIND_FAILURE         = KXCRC_LDAP_UNBIND_FAILURE,
        RC_LDAP_CONNECTION_LOST        = KXCRC_LDAP_CONNECTION_LOST,
        RC_LDAP_INVALID_GATEWAY        = KXCRC_LDAP_INVALID_GATEWAY,
        RC_LDAP_MODRDN_FAILURE         = KXCRC_LDAP_MODRDN_FAILURE,
        RC_LDAP_IMPORT_FAILURE         = KXCRC_LDAP_IMPORT_FAILURE,
        RC_LDAP_EXPORT_FAILURE         = KXCRC_LDAP_EXPORT_FAILURE,
        RC_LDAP_MODCOUNT_FAILURE       = KXCRC_LDAP_MODCOUNT_FAILURE,
	    RC_LDAP_INVALID_RECORD         = KXCRC_LDAP_INVALID_RECORD,
        RC_LDIF_GET_FAILURE            = KXCRC_LDIF_GET_FAILURE,
        RC_LDIF_OPEN_FAILURE           = KXCRC_LDIF_OPEN_FAILURE,
        RC_LDIF_SEARCH_FAILURE         = KXCRC_LDIF_SEARCH_FAILURE,
        RC_LDIF_GET_DN_FAILURE         = KXCRC_LDIF_GET_DN_FAILURE,
        RC_LDIF_NEXT_ENTRY_FAILURE     = KXCRC_LDIF_NEXT_ENTRY_FAILURE,
        RC_LDIF_COUNT_ENTRIES_FAILURE  = KXCRC_LDIF_COUNT_ENTRIES_FAILURE,
        RC_LDIF_MODIFY_FAILURE         = KXCRC_LDIF_MODIFY_FAILURE,
        RC_LDIF_DELETE_FAILURE         = KXCRC_LDIF_DELETE_FAILURE,
        RC_LDIF_ADD_FAILURE            = KXCRC_LDIF_ADD_FAILURE,
        RC_LDIF_INVALID_GATEWAY        = KXCRC_LDIF_INVALID_GATEWAY,
        RC_LDIF_RENAME_FAILURE         = KXCRC_LDIF_RENAME_FAILURE,
        RC_LDIF_INVALID_ATTR_FAILURE   = KXCRC_LDIF_INVALID_ATTR_FAILURE,
        RC_LDIF_READ_FAILURE           = KXCRC_LDIF_READ_FAILURE,
        RC_LDIF_WRITE_FAILURE          = KXCRC_LDIF_WRITE_FAILURE,
        RC_INVALID_PS_OPTIONS          = KXCRC_INVALID_PS_OPTIONS,
        RC_INVALID_SUBJECT             = KXCRC_INVALID_SUBJECT,
        RC_THREAD_INIT_FAILURE         = KXCRC_THREAD_INIT_FAILURE,
        RC_PERSISTENCE_NOT_SUPPORTED   = KXCRC_PERSISTENCE_NOT_SUPPORTED,
        RC_MSGID_NOT_SUPPORTED         = KXCRC_MSGID_NOT_SUPPORTED,
        RC_CORRELID_NOT_SUPPORTED      = KXCRC_CORRELID_NOT_SUPPORTED,
        RC_PEEK_NOT_SUPPORTED          = KXCRC_PEEK_NOT_SUPPORTED,
        RC_KXCRC_FASTPATH_ERROR        = KXCRC_FASTPATH_ERROR,
	    RC_NO_AVAILABLE_ELEMENT        = KXCRC_NO_AVAILABLE_ELEMENT,
        RC_CICS_GETMAIN_FAILURE        = KXCRC_CICS_GETMAIN_FAILURE,
        RC_CICS_COMMIT_ROLLBACK        = KXCRC_CICS_COMMIT_ROLLBACK,
        RC_CICS_NO_TCTLS               = KXCRC_CICS_NO_TCTLS,
        RC_CICS_QMGR_QUIESCE           = KXCRC_CICS_QMGR_QUIESCE,
        RC_CICS_CONN_QUIESCE           = KXCRC_CICS_CONN_QUIESCE,
        RC_CICS_WAITFAIL               = KXCRC_CICS_WAITFAIL,
        RC_CICS_COMMAND_TIMEOUT        = KXCRC_CICS_COMMAND_TIMEOUT,
        RC_CICS_INVALID_POSTCODE       = KXCRC_CICS_INVALID_POSTCODE,
        RC_CICS_EXEC_WAIT_ERROR        = KXCRC_CICS_EXEC_WAIT_ERROR,
        RC_CICS_GETSIG_FAIL            = KXCRC_CICS_GETSIG_FAIL,
        RC_CICS_CALL_ABANDONED         = KXCRC_CICS_CALL_ABANDONED,
        RC_CICS_KXCCTPI_UNAVAILABLE    = KXCRC_CICS_KXCCTPI_UNAVAILABLE,
        RC_CICS_KXCCCDIR_UNAVAILABLE   = KXCRC_CICS_KXCCCDIR_UNAVAILABLE,
        RC_CICS_TRUE_INACTIVE          = KXCRC_CICS_TRUE_INACTIVE,
        RC_CMQI_CALL_ABANDONED         = KXCRC_CMQI_CALL_ABANDONED
    };


    // ---------------------------------------------------------------
    // EXCEPTIONS
    // These constants are used both with RomaExceptionMessage and
    // with putException. They are mapped to the required KXC flags
    // using matchFlags.
    // ---------------------------------------------------------------
    enum Elements
    {
        ELEMENT_NONE      = 0,
        ELEMENT_TIMESTAMP = 1,
        ELEMENT_CONFIG    = 2,
        ELEMENT_SYSCONFIG = 4,
        ELEMENT_ERROR     = 8,
        ELEMENT_USERDATA  = 16,
        ELEMENT_MESSAGE   = 32,
        ELEMENT_ALL_PUT   = ELEMENT_TIMESTAMP | ELEMENT_CONFIG |
                            ELEMENT_SYSCONFIG | ELEMENT_ERROR  |
                            ELEMENT_USERDATA
    };


    enum Actions
    {
        ACTION_DEFAULT  = KXCEP_ACTION_DEFAULT,
        ACTION_NOTIFY   = KXCEP_ACTION_NOTIFY,
        ACTION_ALERT    = KXCEP_ACTION_ALERT,
        ACTION_LOG      = KXCEP_ACTION_LOG,
        ACTION_RESET    = KXCEP_ACTION_RESET,
        ACTION_NOACTION = KXCEP_ACTION_NOACTION,
        ACTION_SYSLOG   = KXCEP_ACTION_SYSLOG,
        ACTION_EMS      = KXCEP_ACTION_EMS,
        ACTION_EMSRAW   = KXCEP_ACTION_EMSRAW
    };


    // ---------------------------------------------------------------
    // LOG INFORMATION
    // ---------------------------------------------------------------
    enum Severity
    {
        SEV_INFO    = KXCLOGSEVERITY_INFO,
        SEV_WARNING = KXCLOGSEVERITY_WARNING,
        SEV_ERROR   = KXCLOGSEVERITY_ERROR,
        SEV_SEVERE  = KXCLOGSEVERITY_SEVERE
    };


    enum LogFormat
    {
        LOGFORMAT_NONE        = KXCLOGFORMAT_NONE,
        LOGFORMAT_HEXDUMP     = KXCLOGFORMAT_HEXDUMP,
        LOGFORMAT_NOTIMESTAMP = KXCLOGFORMAT_NOTIMESTAMP,
        LOGFORMAT_NOSEVERITY  = KXCLOGFORMAT_NOSEVERITY
    };


    // ---------------------------------------------------------------
    // PUBLISH-SUBSCRIBE
    // ---------------------------------------------------------------
    enum Style
    {
        STYLE_NONE          = KXCSUBJECTSTYLE_NONE,
        STYLE_TIBRENDEZVOUS = KXCSUBJECTSTYLE_TIBRENDEZVOUS,
        STYLE_MQSERIES      = KXCSUBJECTSTYLE_MQSERIES,
        STYLE_ROMA          = KXCSUBJECTSTYLE_ROMA
    };


    // ---------------------------------------------------------------
    // MESSAGE ID / CORRELATION ID
    // ---------------------------------------------------------------
    enum
    {
        ID_LENGTH = KXCMSG_ID_LENGTH
    };


    // ---------------------------------------------------------------
    // SYNCPOINTDEF
    // ---------------------------------------------------------------
    enum SyncType
    {
        SYNC_NONE    = KXCSYNC_NONE,
        SYNC_LOCAL   = KXCSYNC_LOCAL,
        SYNC_GLOBAL  = KXCSYNC_GLOBAL,
        SYNC_PROFILE = KXCSYNC_USEDEFAULT
    };

    // ---------------------------------------------------------------
    // buildTheme() options
    // ---------------------------------------------------------------
    enum ThemeInfoOptions
    {
        THEMEINFO_THEMEID       = 1,
        THEMEINFO_RETRYCOUNT    = 2,
        THEMEINFO_WORKFLOW      = 4,
        THEMEINFO_ETEINFO       = 8,
        THEMEINFO_TRAILERS      = 16,
        THEMEINFO_CONFIRMFLAGS  = 32,
        THEMEINFO_ALL = (THEMEINFO_THEMEID | THEMEINFO_RETRYCOUNT |
                         THEMEINFO_WORKFLOW | THEMEINFO_ETEINFO |
                         THEMEINFO_TRAILERS | THEMEINFO_CONFIRMFLAGS)
    };

    // ---------------------------------------------------------------
    // Environment type
    // ---------------------------------------------------------------
    enum EnvType
    {
        ENVTYPE_INTEL     = 0x01000000,
        ENVTYPE_UNIX      = 0x02000000,
        ENVTYPE_MVS       = 0x04000000,
        ENVTYPE_OS400     = 0x08000000,
        ENVTYPE_VMS       = 0x10000000
    };

    // ---------------------------------------------------------------
    // Environment OS
    // ---------------------------------------------------------------
    enum EnvOS
    {
        ENVOS_AIX         = (0x00010000 | ENVTYPE_UNIX),
        ENVOS_HPUX102     = (0x00080000 | ENVTYPE_UNIX),
        ENVOS_HPUX11      = (0x00020000 | ENVTYPE_UNIX),
        ENVOS_DUX         = (0x00100000 | ENVTYPE_UNIX),
        ENVOS_MVSBATCH    = (0x00020000 | ENVTYPE_MVS),
        ENVOS_MVSCICS     = (0x00010000 | ENVTYPE_MVS),
        ENVOS_SUN         = (0x00040000 | ENVTYPE_UNIX),
        ENVOS_WIN95       = (0x00020000 | ENVTYPE_INTEL),
        ENVOS_WINNT       = (0x00010000 | ENVTYPE_INTEL),
        ENVOS_OS400       = (0x00100000 | ENVTYPE_OS400)
    };

    // ---------------------------------------------------------------
    // Confirmation flags
    // ---------------------------------------------------------------
    enum ConfirmFlags
    {
        CONFIRM_NONE                    = 0,
        CONFIRM_DELIVERY                = KXCPOPT_COD,
        CONFIRM_ARRIVAL                 = KXCPOPT_COA,
        CONFIRM_DELIVERY_AND_ARRIVAL    = (CONFIRM_DELIVERY | CONFIRM_ARRIVAL)
    };

    // ---------------------------------------------------------------
    // Route flags
    // ---------------------------------------------------------------
    enum RouteFlags
    {
        ROUTEFLAG_RESTRICTED    = 1,
        ROUTEFLAG_XML_ILS       = 2,
        ROUTEFLAG_NOTRANSFORM   = 4
    };

    // ---------------------------------------------------------------
    // interpretMsg() options
    // ---------------------------------------------------------------
    enum IntMsgOptions
    {
        INTMSG_THEME = 1
    };
};


#endif // __KXCOROMA_HPP

