/*********************************************************************
 * $Id: kxmerror.h,v 1.44 2004/09/24 17:10:38 e_ab Exp $     
 * $Source: /romacvs/root/roma/src/kxm/kxmutil/kxmerror.h,v $
 *********************************************************************
 * Description: KXM Error Codes
 *********************************************************************
 *                                                                    
 * Copyright 1999-2002, Candle Corporation, a California                   
 * corporation.  All rights reserved.                                 
 * International copyright secured.                                   
 *                                                                    
 * This material is proprietary to Candle Corporation and is not to   
 * be reproduced, used, or disclosed except in accordance with        
 * program licenses or upon written authorization of Candle           
 * Corporation.                                                       
 *                                                                    
 ********************************************************************/
#ifndef __KXMERROR_H
#define __KXMERROR_H

#include <krrerror.h>

/* Layers                                                           */

#define KXMOP_BASE                      110
#define KXMOP_CONVERSION                (KXMOP_BASE)
#define KXMOP_HFAPI                     (KXMOP_BASE+1)
#define KXMOP_CORE                      (KXMOP_BASE+2)

/* Error code ranges                                                */

#define KXMRC_BASE                      20000
#define KXMRC_CONVBASE                  KXMRC_BASE
#define KXMRC_HFBASE                    KXMRC_BASE+200
#define KXMRC_CPPHFBASE                 KXMRC_BASE+300
#define KXMRC_CHFBASE                   KXMRC_BASE+400
#define KXMRC_LDAP                      KXMRC_BASE+500
#define KXMRC_ILS                       KXMRC_BASE+600
#define KXMRC_XHFBASE                   KXMRC_BASE+700

/* Common error codes                                               */

#define KXMRC_OK                        0

#define KXMRC_OUT_OF_MEMORY             KRRRC_OUT_OF_MEMORY

/*******************************************************************/
/* Core error codes                                                */
/*******************************************************************/

/* Supplied buffer is too small to store data in                   */
/* and auto-allocation is disabled.                                */
#define KXMRC_BUFFER_TOO_SMALL          (KXMRC_CONVBASE+0)

/* Internal logic error.                                           */
#define KXMRC_INTERNAL_ERROR            (KRRRC_INTERNAL_ERROR)

/* Map definition or one of the definitions                        */
/* it refers to is invalid.                                        */
#define KXMRC_INVALID_MAP               (KXMRC_CONVBASE+1)

/* Map failed to convert the data.                                 */
#define KXMRC_MAP_ERROR                 (KXMRC_CONVBASE+2)

/* Name of the XML document does not match                         */
/* the name specified in the repository.                           */
#define KXMRC_DOC_TYPE_MISMATCH         (KXMRC_CONVBASE+3)

/* Field value does not meet the restriction specified             */
/* in the repository.                                              */
#define KXMRC_RESTRICTION_FAILED        (KXMRC_CONVBASE+4)

/* Error in parsing the message.                                   */
#define KXMRC_PARSE_ERROR               (KXMRC_CONVBASE+5)

/* Error in formatting the message.                                */
#define KXMRC_FORMAT_ERROR              (KXMRC_CONVBASE+6)

/* Multiple conversion maps exist to do conversion                 */
/* between the formats.                                            */
#define KXMRC_MULTIPLE_MAPS             (KXMRC_CONVBASE+7)

/* Expression passed to the KxmBoolExpr class is not boolean.      */
#define KXMRC_EXPR_NOT_BOOLEAN          (KXMRC_CONVBASE+8)

/* Expression is invalid.                                          */
#define KXMRC_INVALID_EXPR              (KXMRC_CONVBASE+9)

/* Expression evaluated to NOTHING                                 */
#define KXMRC_NOTHING_VALUE             (KXMRC_CONVBASE+10)

/* Expression evaluation error.                                    */
#define KXMRC_EVAL_ERROR                (KXMRC_CONVBASE+11)

#define KXMRC_EXPR_NOT_COMPARABLE       (KXMRC_CONVBASE+12)

#define KXMRC_EXPR_DEF_MISMATCH         (KXMRC_CONVBASE+13)

#define KXMRC_CORRUPTED_DATA            (KXMRC_CONVBASE+14)

#define KXMRC_FORMAT_MISMATCH           (KXMRC_CONVBASE+15)

/* Invalid value in the KXMTRANHEAD structure                      */
#define KXMRC_INVALID_TRANHEAD          (KXMRC_CONVBASE+16)

#define KXMRC_FUNCLIB_ERROR             (KXMRC_CONVBASE+17)
#define KXMRC_NO_MATCHED_FIELDS         (KXMRC_CONVBASE+18)
#define KXMRC_UNSUPPORTED_TYPE          (KXMRC_CONVBASE+19)

#define KXMRC_MDR_NOT_AVAILABLE         (KXMRC_CONVBASE+20)

#define KXMRC_CANT_SET_FIELD            (KXMRC_CONVBASE+21)

#define KXMRC_OS_ERROR                  (KXMRC_CONVBASE+22)

#define KXMRC_UNKNOWN_XMLSCHEMA_TYPE    (KXMRC_CONVBASE+23)

/*******************************************************************/
/* Common Helper Functions error codes                             */
/*******************************************************************/

/* Obsolete */
#define KXMRC_ARRAY_TOO_SMALL           (KXMRC_HFBASE+0)

/* Mismatch between type of a value and                            */
/* type of the set or get function called.                         */
#define KXMRC_CONVERSION_IMPOSSIBLE     (KXMRC_HFBASE+1)

/* Value cannot be converted for the set or get function.          */
/* Reasons:                                                        */
/* 1) string value does not contain proper number or binary        */
/* 2) numeric value cannot fit the target precision                */
/* 3) negative numeric value being set or get as an unsigned       */
#define KXMRC_CONVERSION_FAILED         (KXMRC_HFBASE+2)

/* Index is out of range.                                          */
#define KXMRC_INVALID_INDEX             (KXMRC_HFBASE+3)

/* Field is empty.                                                 */
#define KXMRC_EMPTY_FIELD               (KXMRC_HFBASE+4)

/* Message (group) does not contain specified field.               */
#define KXMRC_UNKNOWN_FIELD             (KXMRC_HFBASE+5)

/* The value is not an array                                       */
#define KXMRC_NOT_ARRAY                 (KXMRC_HFBASE+6)

/* The value is not a group                                        */
#define KXMRC_NOT_GROUP                 (KXMRC_HFBASE+7)

/* The value is not an XML element                                 */
#define KXMRC_NOT_XML_ELEMENT           (KXMRC_HFBASE+8)

/* The value found is not unique in the message                    */
#define KXMRC_FIELD_NOT_UNIQUE          (KXMRC_HFBASE+9)

/* The value is not a properly structured date/time value.         */
#define KXMRC_INVALID_DATETIME          (KXMRC_HFBASE+10)

#define KXMRC_NO_DEFAULT                (KXMRC_HFBASE+11)

/* The value is not a primitive (string, binary, number, etc.)     */
#define KXMRC_NOT_PRIMITIVE             (KXMRC_HFBASE+12)

#define KXMRC_KXC_ERROR                 (KXMRC_HFBASE+13)

/* The value is not an XML field (element, attribute, etc.)     */
#define KXMRC_NOT_XML_FIELD             (KXMRC_HFBASE+14)

/* XML namespaces handling disabled */
#define KXMRC_XMLNS_DISABLED            (KXMRC_HFBASE+15)

/* The value is not a properly structured timezone value.         */
#define KXMRC_INVALID_TIMEZONE          (KXMRC_HFBASE+16)

/* The value is not a properly structured duration value.         */
#define KXMRC_INVALID_DURATION          (KXMRC_HFBASE+17)

/* The value is not a properly structured qname value.         */
#define KXMRC_INVALID_QNAME             (KXMRC_HFBASE+18)

/* The value is not a properly structured encoding value.         */
#define KXMRC_INVALID_ENCODING          (KXMRC_HFBASE+19)

/* The field is not an union.         */
#define KXMRC_NOT_UNION                 (KXMRC_HFBASE+20)

/*******************************************************************/
/* C++ Helper Functions error codes                                */
/*******************************************************************/

#define KXMRC_INVALID_FIELD             (KXMRC_CPPHFBASE+0)

#define KXMRC_INVALID_MESSAGE           (KXMRC_CPPHFBASE+1)

#define KXMRC_INVALID_REPOSITORY        (KXMRC_CPPHFBASE+2)

#define KXMRC_INVALID_FIELD_NAME        (KXMRC_CPPHFBASE+3)

#define KXMRC_INVALID_XML_FILTER        (KXMRC_CPPHFBASE+4)

#define KXMRC_READ_ONLY                 (KXMRC_CPPHFBASE+5)

#define KXMRC_INVALID_FORMAT_NAME       (KXMRC_CPPHFBASE+6)

#define KXMRC_INVALID_STRING            (KXMRC_CPPHFBASE+7)

#define KXMRC_INVALID_TYPE              (KXMRC_CPPHFBASE+8)

#define KXMRC_INVALID_DATALEN           (KXMRC_CPPHFBASE+9)

#define KXMRC_INVALID_TYPEID            (KXMRC_CPPHFBASE+10)

#define KXMRC_DEFAULT_VALUE             (KXMRC_CPPHFBASE+11)

#define KXMRC_INVALID_PARAM             (KXMRC_CPPHFBASE+12)

#define KXMRC_MAP_NOT_FOUND             (KXMRC_CPPHFBASE+13)

#define KXMRC_NOT_APPLICABLE            (KXMRC_CPPHFBASE+14)

#define KXMRC_NOT_SUPPORTED             (KXMRC_CPPHFBASE+15)

#define KXMRC_API_MISUSE                (KXMRC_CPPHFBASE+16)

/*******************************************************************/
/* C Helper Functions error codes                                  */
/*******************************************************************/

/* Array does not contain the specified number of elements.        */
#define KXMRC_COUNT_TOO_BIG             (KXMRC_CHFBASE+0)

/* Message or field handle is invalid.                             */
#define KXMRC_INVALID_HANDLE            (KXMRC_CHFBASE+1)

/* Options structure is invalid.                                   */
#define KXMRC_INVALID_OPTIONS           (KXMRC_CHFBASE+2)

/* Pointer is NULL or does not point to a valid address.           */
#define KXMRC_INVALID_POINTER           (KXMRC_CHFBASE+3)

/* Buffer structure is invalid.                                    */
#define KXMRC_INVALID_BUFFER            (KXMRC_CHFBASE+4)

/* Library is not initialized.                                     */
#define KXMRC_LIB_NOT_INITIALIZED       (KXMRC_CHFBASE+5)

/* Library already initialized                                     */
#define KXMRC_LIB_ALREADY_INITIALIZED   (KXMRC_CHFBASE+6)

/* Array value structure is invalid                                */
#define KXMRC_INVALID_ARRAY_VALUE       (KXMRC_CHFBASE+7)

/* Field info structure is invalid                                 */
#define KXMRC_INVALID_FIELD_INFO        (KXMRC_CHFBASE+8)

/* Message info structure is invalid                               */
#define KXMRC_INVALID_MESSAGE_INFO      (KXMRC_CHFBASE+9)

/* Count is zero                                                   */
#define KXMRC_INVALID_COUNT             (KXMRC_CHFBASE+10)

#define KXMRC_INVALID_ARRAYVAL          (KXMRC_CHFBASE+11)

#define KXMRC_PARENT_IS_ABSENT          (KXMRC_CHFBASE+12)

/*******************************************************************/
/* ActiveX error codes                                             */
/*******************************************************************/

#define KXMRC_CONV_NOT_INSTALLED        (KXMRC_XHFBASE+1)

#define KXMRC_NO_WRITE                  (KXMRC_XHFBASE+2)

/*******************************************************************/
/* ldap access error                                               */
/*******************************************************************/

#define KXMRC_UNEXPECTED_CHANGES        (KXMRC_LDAP+0)

#define KXMRC_LDAP_WRONG_CONFIG         (KXMRC_LDAP+1)

#define KXMRC_CANNOT_LOAD_ILS           (KXMRC_ILS+0)


/* Symbolic constants for the error positions.                     */

#define KXMPOS_0                        0L
#define KXMPOS_1                        1L
#define KXMPOS_2                        2L
#define KXMPOS_3                        3L
#define KXMPOS_4                        4L
#define KXMPOS_5                        5L
#define KXMPOS_6                        6L
#define KXMPOS_7                        7L
#define KXMPOS_8                        8L
#define KXMPOS_9                        9L
#define KXMPOS_10                       10L
#define KXMPOS_11                       11L
#define KXMPOS_12                       12L
#define KXMPOS_13                       13L
#define KXMPOS_14                       14L

#endif
