@rem ######################################################################
@rem # Copyright 1999-2002, Candle Corporation, a California
@rem # corporation.  All rights reserved. International copyright secured.
@rem #
@rem # This material is proprietary to Candle Corporation and is not to be
@rem # reproduced, used, or disclosed except in accordance with program
@rem # licenses or upon written authorization of Candle Corporation.
@rem #
@rem # This file runs a set of functions that show various capabilities
@rem # available in the CandleNet eBP Metadata Repository.
@rem # The command syntax is:
@rem #		krrivrun.bat userid password
@rem #
@rem # For example:
@rem #		krrivrun.bat manager secret
@rem #
@rem # $Id: krrivrun.bat,v 1.22 2004/09/23 19:50:13 e_vvb Exp $
@rem # $Source: /romacvs/root/roma/src/krr/krrivp/krrivrun.bat,v $
@rem #######################################################################

@echo off
setlocal

@rem Usage
if Z%2 == Z goto BadUsage

@rem Get the parameters
set DN=%1
set PASSWD=%2

@rem Cleans IVP metadata from the repository
echo --- Preliminary cleaning of old IVP metadata from the repository
krrdel -D %DN% -w %PASSWD% -ns CandleSamples.KrrIVP
if errorlevel 1 goto BadClean

@rem Import metadata into the repository
echo --- Importing metadata into the repository
krrimp -D %DN% -w %PASSWD% -f krriv.mdr
if errorlevel 1 goto BadImport

@rem Importing COBOL definitions into the repository
echo --- Importing COBOL definitions into the repository
krrfcbl -i krrivcbl.cbl -o krrivcbl.mdr -ns CandleSamples.KrrIVP.Cobol
if errorlevel 1 goto BadCblImport
krrimp -D %DN% -w %PASSWD% -f krrivcbl.mdr
if errorlevel 1 goto BadCblImport

@rem Generating DTD(s) from raw format definition
echo --- Generating DTD(s) from raw format definition
krrschgn -dtd -mf :CandleSamples.KrrIVP.Cobol:* 
if errorlevel 1 goto BadDtdGen

@rem Importing generated DTD definitions back into the repository
echo --- Importing generated DTD definitions back into the repository
krrschim -dtd -D %DN% -w %PASSWD% -ns CandleSamples.KrrIVP.Cobol *.dtd
if errorlevel 1 goto BadDtdImport

@rem Generating XML Schema(s) from raw format definition
echo --- Generating XML Schema(s) from raw format definition
krrschgn -xmlschema -mf :CandleSamples.KrrIVP.Cobol:* -tn "http://www.candle.com/CandleSamples/KrrIVP/Cobol" -pr ivp -q -f krrivp.xsd
if errorlevel 1 goto BadSchemaGen

@rem Importing generated XML Schema definition back into the repository
echo --- Importing generated XML Schema definitions back into the repository
krrschim -xmlschema -D %DN% -w %PASSWD% -ns CandleSamples.KrrIVP.Cobol *.xsd
if errorlevel 1 goto BadSchemaImport

@rem Checks repository metadata
echo --- Checking metadata within the repository
krrcheck -D %DN% -w %PASSWD% -ns CandleSamples.KrrIVP -syntax_only
if errorlevel 1 goto BadCheck

@rem Exports repository metadata
echo --- Exporting metadata from the repository
krrexp -D %DN% -w %PASSWD% -ns CandleSamples.KrrIVP -f export.mdr
if errorlevel 1 goto BadExport

@rem Cleans IVP metadata from the repository
echo --- Final cleaning of the IVP metadata from the repository
krrdel -D %DN% -w %PASSWD% -ns CandleSamples.KrrIVP
if errorlevel 1 goto BadClean

@rem Remove generated .dtd files
echo --- Remove temporary generated files
del *.dtd export.mdr krrivcbl.mdr

@rem SUCCESS
echo ">>>>> IVP Finished Succesfully <<<<<<<<<<<"
goto done

:BadUsage
echo Usage: krrivrun.bat userid password
goto end

:BadImport
echo krrimp test failed.
goto end

:BadCblImport
echo krrfcbl test failed.
goto end

:BadDtdGen
echo krrschgn test failed.
goto end

:BadDtdImport
echo krrschim test failed.
goto end

:BadSchemaGen
echo krrschgn (XML Schema) test failed.
goto end

:BadSchemaImport
echo krrschim (XML Schema) test failed.
goto end

:BadCheck
echo krrcheck test failed.
goto end

:BadExport
echo krrexp test failed.
goto end

:BadClean
echo krrdel test failed.
goto end

:end
@rem Remove generated .dtd files
echo --- Remove tempory generated files
del *.dtd *.xsd export.mdr krrivcbl.mdr

:done
@rem done
endlocal
